'                                  ASTA.BAS
'
'         procedure per la valutazione delle matrici di rigidezza,
'         dei vettori di incastro perfetto e delle caratteristiche
'         di sollecitazione nei punti interni delle aste
'
'                                                                 rev. 01.99
'
'         unificate due versioni (09.89 e 03.95) che correggono errori
'         in Sl.Inc.DT, Sl.Inc.FM e Kf.Rig

' ------------------ Dichiarazione delle procedure utilizzate ------------------

' procedure del file MATRIX.BAS
DECLARE SUB AzzeraMat (A!())
DECLARE SUB InserMat (A!(), B!(), RI!, CI!)
DECLARE SUB InversaMat (A!())
DECLARE SUB LimitiMat (A!(), DA!, LR!, LC!, NR!, NC!)
DECLARE SUB ProdottoCostMat (K!, A!(), B!())
DECLARE SUB ProdottoMat (A!(), B!(), C!())
DECLARE SUB SommaMat (A!(), B!(), C!())
DECLARE SUB TrasponeMat (A!(), B!())

' procedure di questo stesso file
DECLARE SUB Geometria (X1!, X2!, Y1!, Y2!, L!, SnAl!, CsAl!, Ld!())
DECLARE SUB Kf.Cer (ES!, e!, A!, I!, L!, Kf!())
DECLARE SUB Kf.CerTag (ES!, e!, G!, CHI!, A!, I!, L!, Kf!())
DECLARE SUB Kf.Inc (e!, A!, I!, L!, Kf!())
DECLARE SUB Kf.IncTag (e!, G!, CHI!, A!, I!, L!, Kf!())
DECLARE SUB Kf.NodRig (e!, A!, I!, L!, Sx!, Sy!, Tx!, Ty!, Kf!())
DECLARE SUB Kf.Rig (e!, A!, I!, L!, S!, T!, Kf!())
DECLARE SUB Kf.RigTag (e!, G!, CHI!, A!, I!, L!, S!, T!, Kf!())
DECLARE SUB Kf.Wnk (K!, B!, e!, A!, I!, L!, Kf!())
DECLARE SUB Kf.WnkRig (K!, B!, e!, A!, I!, L!, S!, T!, Kf!())
DECLARE SUB Kg.f (Ld!(), T!(), Kf!(), Kg!())
DECLARE SUB T.Cer (ES!, L!, T!())
DECLARE SUB T.Inc (L!, T!())
DECLARE SUB T.Wnk (T!())

' ================================ Asta.Inc ====================================
'
'   Procedura per la valutazione diretta della matrice di rotazione,
'   dei parametri che dipendono solo dalla geometria dell'asta
'   e della matrice di rigidezza nel riferimento globale
'   per un'asta perfettamente incastrata ai nodi di estremita`
'
'   NOTA: per ridurre il tempo di calcolo,  la matrice di rigidezza
'         viene valutata con le formulazioni dirette, senza passare
'         per le matrici fondamentale e locale
'
'   Variabili di ingresso:
'      X1        ascissa primo estremo
'      X2        ascissa secondo estremo
'      Y1        ordinata primo estremo
'      Y2        ordinata secondo estremo
'      E         modulo di elasticita` normale
'      A         area della sezione trasversale dell'asta
'      I         momento d'inerzia della sezione trasversale dell'asta
'
'   Variabili di uscita:
'      L         lunghezza dell'asta
'      SnAl      seno dell'angolo di inclinazione dell'asta
'      CsAl      coseno dell'angolo di inclinazione dell'asta
'      Ld()      matrice di rotazione dell'asta
'      Kg()      matrice di rigidezza globale
'
'   Procedure utilizzate:
'      Geometria
'
' ------------------------------------------------------------------------------
'
SUB Asta.Inc (X1, X2, Y1, Y2, e, A, I, L, SnAl, CsAl, Ld(), Kg())

   CALL Geometria(X1, X2, Y1, Y2, L, SnAl, CsAl, Ld())

   EIL = e * I / L
   EIL2 = EIL / L
   EIL3 = EIL2 / L
   EAL = e * A / L
   SN2 = SnAl ^ 2
   CS2 = CsAl ^ 2

   K1 = EAL * CS2 + 12 * EIL3 * SN2
   K2 = EAL * SN2 + 12 * EIL3 * CS2
   K3 = 6 * EIL2 * SnAl
   K4 = 6 * EIL2 * CsAl
   K5 = 4 * EIL
   K6 = 2 * EIL
   K7 = (EAL - 12 * EIL3) * SnAl * CsAl

   Kg(1, 1) = K1
   Kg(1, 2) = K7
   Kg(1, 3) = -K3
   Kg(1, 4) = -K1
   Kg(1, 5) = -K7
   Kg(1, 6) = -K3

   Kg(2, 1) = K7
   Kg(2, 2) = K2
   Kg(2, 3) = K4
   Kg(2, 4) = -K7
   Kg(2, 5) = -K2
   Kg(2, 6) = K4
 
   Kg(3, 1) = -K3
   Kg(3, 2) = K4
   Kg(3, 3) = K5
   Kg(3, 4) = K3
   Kg(3, 5) = -K4
   Kg(3, 6) = K6

   Kg(4, 1) = -K1
   Kg(4, 2) = -K7
   Kg(4, 3) = K3
   Kg(4, 4) = K1
   Kg(4, 5) = K7
   Kg(4, 6) = K3

   Kg(5, 1) = -K7
   Kg(5, 2) = -K2
   Kg(5, 3) = -K4
   Kg(5, 4) = K7
   Kg(5, 5) = K2
   Kg(5, 6) = -K4

   Kg(6, 1) = -K3
   Kg(6, 2) = K4
   Kg(6, 3) = K6
   Kg(6, 4) = K3
   Kg(6, 5) = -K4
   Kg(6, 6) = K5

END SUB

' ================================ CarSol.AzEstr ===============================
'
'   Procedura per il calcolo di momento, taglio e sforzo normale:
'   effetto delle azioni di estremita`
'
'   Variabili di ingresso:
'      Fx        componente parallela all'asta dell'azione al primo estremo
'      Fy        componente ortogonale all'asta dell'azione al primo estremo
'      Mz        coppia concentrata Mz al primo estremo
'      X         ascissa punto calcolo delle sollecitazioni
'      L         lunghezza dell'asta
'
'   Variabili di uscita:
'      Sn        sforzo normale all'ascissa X
'      Mf        momento flettente all'ascissa X
'      Ta        taglio all'ascissa X
'
'   Procedure utilizzate:
'      ===
'
' ------------------------------------------------------------------------------
'
SUB Carsol.AzEstr (Fx, Fy, Mz, X, L, Sn, Mf, Ta)

    IF X < 0 OR X > L THEN
       PRINT "ERRORE 1 in CarSol.AzEstr - valori dell'ascissa non accettabili"
       END
    END IF

    Sn = -Fx
    Mf = Fy * X - Mz
    Ta = Fy

END SUB

' ================================ CarSol.FM ===================================
'
'   Procedura per il calcolo di momento, taglio e sforzo normale:
'   forza F (di componenti Fx e Fy) e coppia Mz concentrate
'
'   Variabili di ingresso:
'      Fx        componente parallela all'asta della forza concentrata F
'      Fy        componente ortogonale all'asta della forza concentrata F
'      Mz        coppia concentrata M
'      X0        ascissa punto applicazione azioni concentrate
'      X         ascissa punto calcolo delle sollecitazioni
'      L         lunghezza dell'asta
'
'   Variabili di uscita:
'      Sn        sforzo normale all'ascissa X
'      Mf        momento flettente all'ascissa X
'      Ta        taglio all'ascissa X
'
'   Procedure utilizzate:
'      ===
'
' ------------------------------------------------------------------------------
'
SUB Carsol.FM (Fx, Fy, Mz, X0, X, L, Sn, Mf, Ta)

    IF X0 < 0 OR X0 > L OR X < 0 OR X > L THEN
       PRINT "ERRORE 1 in CarSol.FM - valori delle ascisse non accettabili"
       END
    END IF

    IF X <= X0 THEN
       Sn = 0
       Mf = 0
       Ta = 0
    ELSE
       Sn = -Fx
       Mf = Fy * (X - X0) - Mz
       Ta = Fy
    END IF

END SUB

' ================================ CarSol.Qunif ================================
'
'   Procedura per il calcolo di momento, taglio e sforzo normale:
'   carico uniformemente distribuito
'
'   Variabili di ingresso:
'      Q         carico uniformemente distribuito ortogonale all'asta
'      N         carico uniformemente distribuito parallelo  all'asta
'      X         ascissa per il calcolo delle sollecitazioni
'      L         lunghezza dell'asta
'
'   Variabili di uscita:
'      Sn        sforzo normale all'ascissa X
'      Mf        momento flettente all'ascissa X
'      Ta        taglio all'ascissa X
'
'   Procedure utilizzate:
'      ===
'
' ------------------------------------------------------------------------------
'
SUB CarSol.Qunif (Q, N, X, L, Sn, Mf, Ta)

    IF X < 0 OR X > L THEN
       PRINT "ERRORE 1 in CarSol.Qunif - valori dell'ascissa non accettabili"
       END
    END IF

    Sn = -N * X
    Mf = Q * X ^ 2 / 2
    Ta = Q * X

END SUB

' ================================ CarSol.Qvar =================================
'
'   Procedura per il calcolo di momento e taglio:
'   carico distribuito con andamento lineare su parte dell'asta
'
'   NOTA: il carico e` ortogonale all'asse e non provoca sforzo normale
'
'   Variabili di ingresso:
'      Q1        valore del carico a sinistra
'      Q2        valore del carico a destra
'      X1        ascissa inizio carico
'      X2        ascissa fine carico
'      X         ascissa per calcolo sollecitazioni
'      L         lunghezza dell'asta
'
'   Variabili di uscita:
'      Mf        momento flettente all'ascissa X
'      Ta        taglio all'ascissa X
'
'   Procedure utilizzate:
'      ===
'
' ------------------------------------------------------------------------------
'
SUB Carsol.Qvar (Q1, Q2, X1, X2, X, L, Mf, Ta)

    IF X1 < 0 OR X1 > L OR X2 < 0 OR X2 > L OR X < 0 OR X > L THEN
       PRINT "ERRORE 1 in CarSol.Qvar - valori delle ascisse non accettabili"
       END
    END IF
    IF X1 > X2 THEN
       PRINT "ERRORE 2 in CarSol.Qvar - ordine delle ascisse non accettabile"
       END
    END IF

    IF X <= X1 THEN
       Mf = 0
       Ta = 0
    ELSEIF X < X2 THEN
       Dx1 = X - X1
       Dx2 = X - X2
       D = X2 - X1
       U = Dx1 / D
       Mf = ((1 - U / 3) * Q1 + U / 3 * Q2) * Dx1 ^ 2 / 2
       Ta = ((1 - U / 2) * Q1 + U / 2 * Q2) * Dx1
    ELSE
       Dx2 = X - X2
       D = X2 - X1
       T0 = (Q1 + Q2) / 2 * D
       M0 = (2 / 3 * Q1 + 1 / 3 * Q2) * D ^ 2 / 2
       Mf = T0 * Dx2 + M0
       Ta = T0
    END IF

END SUB

' ================================ Geometria ===================================
'
'   Procedura  per  la valutazione  della matrice  di rotazione
'   e di parametri che dipendono solo dalla geometria dell'asta
'
'   Variabili di ingresso:
'      X1        ascissa primo estremo
'      X2        ascissa secondo estremo
'      Y1        ordinata primo estremo
'      Y2        ordinata secondo estremo
'
'   Variabili di uscita:
'      L         lunghezza dell'asta
'      SnAl      seno dell'angolo di inclinazione dell'asta
'      CsAl      coseno dell'angolo di inclinazione dell'asta
'      Ld()      matrice di rotazione dell'asta
'
'   Procedure utilizzate:
'      AzzeraMat
'
' ------------------------------------------------------------------------------
'
SUB Geometria (X1, X2, Y1, Y2, L, SnAl, CsAl, Ld())
    
    L = SQR((X2 - X1) ^ 2 + (Y2 - Y1) ^ 2)
    IF L = 0 THEN
       PRINT "ERRORE 1 in Geometria - asta di lunghezza nulla"
       END
    END IF
    SnAl = (Y2 - Y1) / L
    CsAl = (X2 - X1) / L

    CALL AzzeraMat(Ld())
    Ld(1, 1) = CsAl
    Ld(1, 2) = -SnAl
    Ld(2, 1) = SnAl
    Ld(2, 2) = CsAl
    Ld(3, 3) = 1
    Ld(4, 4) = CsAl
    Ld(4, 5) = -SnAl
    Ld(5, 4) = SnAl
    Ld(5, 5) = CsAl
    Ld(6, 6) = 1

END SUB

' ================================ Kf.Cer ======================================
'
'   Procedura per la definizione della matrice di rigidezza fondamentale
'   per un'asta incernierata agli estremi
'
'   Variabili di ingresso:
'      ES        estremo incernierato
'                   1 = primo estremo
'                   2 = secondo estremo
'                   3 = entrambi gli estremi
'      E         modulo di elasticita` normale
'      A         area della sezione trasversale dell'asta
'      I         momento d'inerzia della sezione trasversale dell'asta
'      L         lunghezza dell'asta
'
'   Variabili di uscita:
'      Kf()      matrice di rigidezza fondamentale
'
'   Procedure utilizzate:
'      AzzeraMat
'
' ------------------------------------------------------------------------------
'
SUB Kf.Cer (ES, e, A, I, L, Kf())
  
    CALL AzzeraMat(Kf())
    SELECT CASE ES
       CASE 1, 2
          Kf(1, 1) = 3 * e * I / L
          Kf(2, 2) = e * A / L
       CASE 3
          Kf(1, 1) = e * A / L
    END SELECT

END SUB

' ================================ Kf.CerTag ===================================
'
'   Procedura per la definizione della matrice di rigidezza fondamentale
'   per asta incernierata agli estremi tenendo conto della deformazione a taglio
'
'   Variabili di ingresso:
'      ES        estremo incernierato
'                   1 = primo estremo
'                   2 = secondo estremo
'                   3 = entrambi gli estremi
'      E         modulo di elasticita` normale
'      G         modulo di elasticita` tangenziale
'      CHI       fattore di taglio
'      A         area della sezione trasversale dell'asta
'      I         momento d'inerzia della sezione trasversale dell'asta
'      L         lunghezza dell'asta
'
'   Variabili di uscita:
'      Kf()      matrice fondamentale dell'asta
'
'   Procedure utilizzate:
'      AzzeraMat
'
' ------------------------------------------------------------------------------
'
SUB Kf.CerTag (ES, e, G, CHI, A, I, L, Kf())

    eps = 3 * e * I * CHI / (G * A * L ^ 2)

    CALL AzzeraMat(Kf())
    SELECT CASE ES
       CASE 1, 2
          Kf(1, 1) = 3 * e * I / L / (1 + eps)
          Kf(2, 2) = e * A / L
       CASE 3
          Kf(1, 1) = e * A / L
    END SELECT

END SUB

' ================================ Kf.Inc ======================================
'
'   Procedura per la definizione della matrice di rigidezza fondamentale
'   per un'asta perfettamente incastrata ai nodi di estremita`
'
'   Variabili di ingresso:
'      E         modulo di elasticita` normale
'      A         area della sezione trasversale dell'asta
'      I         momento d'inerzia della sezione trasversale dell'asta
'      L         lunghezza dell'asta                                          
'
'   Variabili di uscita:
'      Kf()      matrice di rigidezza fondamentale
'
'   Procedure utilizzate:
'      AzzeraMat
'
' ------------------------------------------------------------------------------
'
SUB Kf.Inc (e, A, I, L, Kf())
   
    CALL AzzeraMat(Kf())
    EIL = 2 * e * I / L
    Kf(1, 1) = 2 * EIL
    Kf(1, 2) = EIL
    Kf(2, 1) = EIL
    Kf(2, 2) = Kf(1, 1)
    Kf(3, 3) = e * A / L

END SUB

' ================================ Kf.IncTag ===================================
'
'   Procedura per la definizione della matrice di rigidezza fondamentale
'   per un'asta perfettamente incastrata ai nodi di estremita`
'   tenendo conto della deformazione a taglio
'
'   Variabili in ingresso:
'      E         modulo di elasticita` normale
'      G         modulo di elasticita` tangenziale
'      CHI       fattore di taglio
'      A         area della sezione trasversale dell'asta
'      I         momento d'inerzia della sezione trasversale dell'asta
'      L         lunghezza dell'asta
'
'   Variabili in uscita:
'      Kf()      matrice di rigidezza fondamentale
'
'   Procedure utilizzate:
'      AzzeraMat
'
' ------------------------------------------------------------------------------
'
SUB Kf.IncTag (e, G, CHI, A, I, L, Kf())
    
    eps = 3 * e * I * CHI / (G * A * L ^ 2)

    CALL AzzeraMat(Kf())
    EIL = 2 * e * I / L
    Kf(1, 1) = 2 * EIL * (1 + eps) / (1 + 4 * eps)
    Kf(1, 2) = EIL * (1 - 2 * eps) / (1 + 4 * eps)
    Kf(2, 1) = Kf(1, 2)
    Kf(2, 2) = Kf(1, 1)
    Kf(3, 3) = e * A / L

END SUB

' ================================ Kf.NodRig ===================================
'
'   Procedura per la determinazione della matrice di rigidezza fondamentale
'   nel caso di nodi rigidi non puntiformi
'   (asta con tratti rigidi non coassiali)
'
'   Variabili di ingresso:
'      E         modulo di elasticita` normale
'      A         area della sezione trasversale
'      I         momento d'inerzia della sezione trasversale
'      L         lunghezza dell'asta
'      Sx        coefficiente lunghezza x tratto sinistro
'      Sy        coefficiente lunghezza y tratto sinistro
'      Tx        coefficiente lunghezza x tratto destro
'      Ty        coefficiente lunghezza y tratto destro
'
'   Variabili di uscita:
'      Kf()      matrice di rigidezza fondamentale
'
'   Procedure utilizzate:
'      ProdottoMat
'      TrasponeMat
'      Kf.Inc
'
' ------------------------------------------------------------------------------
'
SUB Kf.NodRig (e, A, I, L, Sx, Sy, Tx, Ty, Kf())

    C = 1 / SQR((1 + Tx - Sx) ^ 2 + (Ty - Sy) ^ 2)
    Ls = L / C
    SnAl = (Ty - Sy) * C
    CsAl = (1 - Sx + Tx) * C

    DIM C(3, 3)
    C(1, 1) = 1 + C * (Sy * SnAl + Sx * CsAl)
    C(2, 1) = -C * (Ty * SnAl + Tx * CsAl)
    C(3, 1) = C * SnAl / L
    C(1, 2) = C * (Sy * SnAl + Sx * CsAl)
    C(2, 2) = 1 - C * (Ty * SnAl + Tx * CsAl)
    C(3, 2) = C * SnAl / L
    C(1, 3) = (Sy * CsAl - Sx * SnAl) * L
    C(2, 3) = -(Ty * CsAl - Tx * SnAl) * L
    C(3, 3) = CsAl

    CALL Kf.Inc(e, A, I, Ls, Kf())
    CALL ProdottoMat(C(), Kf(), Kf())
    CALL TrasponeMat(C(), C())
    CALL ProdottoMat(Kf(), C(), Kf())

END SUB

' ================================ Kf.Rig ======================================
'
'   Procedura per la definizione della matrice di rigidezza fondamentale
'   per un'asta perfettamente incastrata ai nodi di estremita`
'   avente tratti rigidi di estremita`
'
'   Variabili di ingresso:
'      E         modulo di elasticita` normale
'      A         area della sezione trasversale dell'asta
'      I         momento d'inerzia della sezione trasversale dell'asta
'      L         lunghezza dell'asta
'      S         coefficiente lunghezza tratto rigido sinistro
'      T         coefficiente lunghezza tratto rigido destro
'
'   Variabili di uscita:
'      Kf()      matrice di rigidezza fondamentale
'
'   Procedure utilizzate:
'      AzzeraMat
'
' ------------------------------------------------------------------------------
'
SUB Kf.Rig (e, A, I, L, S, T, Kf())
   
   C = 1 / (1 - S - T)
   '* le espressioni riportate nel libro sono sbagliate, perch manca ^3;
   '* le espressioni sotto riportate sono corrette, ma qui uso ora una
   '* versione differente (equivalente) delle formule
   '* A1 = C ^ 3 * ((1 - T) ^ 2 + S * (1 - T) + S ^ 2)
   '* A2 = C ^ 3 * ((1 - S) ^ 2 + T * (1 - S) + T ^ 2)
   '* B = C ^ 3 * (1 + S + T - 2 * (S ^ 2 - S * T + T ^ 2))
   A1 = C * (1 + 3 * C * S + 3 * C ^ 2 * S ^ 2)
   A2 = C * (1 + 3 * C * T + 3 * C ^ 2 * T ^ 2)
   B = C * (1 + 3 * C * S + 3 * C * T + 6 * C ^ 2 * S * T)

   CALL AzzeraMat(Kf())
   EIL = 2 * e * I / L
   Kf(1, 1) = 2 * EIL * A1
   Kf(1, 2) = EIL * B
   Kf(2, 1) = Kf(1, 2)
   Kf(2, 2) = 2 * EIL * A2
   Kf(3, 3) = e * A / L * C

END SUB

' ================================ Kf.RigTag ===================================
'
'   Procedura per la definizione della matrice di rigidezza fondamentale
'   per un'asta perfettamente incastrata ai nodi di estremita`
'   avente tratti rigidi di estremita` - tenendo conto di deform. taglio
'
'   Variabili di ingresso:
'      E         modulo di elasticita` normale
'      G         modulo di elasticita` tangenziale
'      CHI       fattore di taglio
'      A         area della sezione trasversale dell'asta
'      I         momento d'inerzia della sezione trasversale dell'asta
'      L         lunghezza dell'asta
'      S         coefficiente lunghezza tratto rigido sinistro
'      T         coefficiente lunghezza tratto rigido destro
'
'   Variabili di uscita:
'      Kf()      matrice di rigidezza fondamentale
'
'   Procedure utilizzate:
'      ProdottoMat
'      TrasponeMat
'      Kf.IncTag
'
' ------------------------------------------------------------------------------
'
SUB Kf.RigTag (e, G, CHI, A, I, L, S, T, Kf())

   DIM C(3, 3)
   C = 1 / (1 - S - T)
   C(1, 1) = 1 + C * S
   C(1, 2) = C * S
   C(2, 1) = C * T
   C(2, 2) = 1 + C * T
   C(3, 3) = 1
 
   CALL Kf.IncTag(e, G, CHI, A, I, L * (1 - S - T), Kf())
   CALL ProdottoMat(C(), Kf(), Kf())
   CALL TrasponeMat(C(), C())
   CALL ProdottoMat(Kf(), C(), Kf())

END SUB

' ================================ Kf.Wnk ======================================
'
'   Procedura per la definizione della matrice di rigidezza fondamentale
'   per un'asta su suolo elastico alla Winkler
'
'   Variabili di ingresso:
'      K         costante di sottofondo
'      B         larghezza trave
'      E         modulo di elasticita` normale
'      A         area della sezione trasversale dell'asta
'      I         momento d'inerzia della sezione trasversale dell'asta
'      L         lunghezza dell'asta
'
'   Variabili di uscita:
'      Kf()      matrice di rigidezza fondamentale
'
'   Procedure utilizzate:
'      AzzeraMat
'      InserMat
'      InversaMat
'      ProdottoCostMat
'      ProdottoMat
'
' ------------------------------------------------------------------------------
'
SUB Kf.Wnk (K, B, e, A, I, L, Kf())

    CONST en = 2.718
    lambda = (K * B / (4 * e * I)) ^ .25
    lambdaL = lambda * L
    e1 = en ^ lambdaL
    e2 = en ^ (-lambdaL)
    SnL = SIN(lambdaL)
    CsL = COS(lambdaL)
    sp = e1 * SnL
    cp = e1 * CsL
    ss = e2 * SnL
    cs = e2 * CsL

    DIM K1(4, 4)
    K1(1, 1) = 0
    K1(1, 2) = 1
    K1(1, 3) = 0
    K1(1, 4) = 1
    K1(2, 1) = lambda
    K1(2, 2) = lambda
    K1(2, 3) = lambda
    K1(2, 4) = -lambda
    K1(3, 1) = sp
    K1(3, 2) = cp
    K1(3, 3) = ss
    K1(3, 4) = cs
    K1(4, 1) = lambda * (sp + cp)
    K1(4, 2) = lambda * (-sp + cp)
    K1(4, 3) = lambda * (-ss + cs)
    K1(4, 4) = lambda * (-ss - cs)

    DIM K2(4, 4)
    K2(1, 1) = lambda
    K2(1, 2) = -lambda
    K2(1, 3) = lambda
    K2(1, 4) = lambda
    K2(2, 1) = -1
    K2(2, 2) = 0
    K2(2, 3) = 1
    K2(2, 4) = 0
    K2(3, 1) = lambda * (sp - cp)
    K2(3, 2) = lambda * (sp + cp)
    K2(3, 3) = lambda * (-ss - cs)
    K2(3, 4) = lambda * (ss - cs)
    K2(4, 1) = cp
    K2(4, 2) = -sp
    K2(4, 3) = -cs
    K2(4, 4) = ss
    CALL ProdottoCostMat(2 * e * I * lambda ^ 2, K2(), K2())

    CALL InversaMat(K1())
    CALL ProdottoMat(K2(), K1(), K2())
    CALL AzzeraMat(Kf())
    CALL InserMat(K2(), Kf(), 1, 1)
    Kf(5, 5) = e * A / L

END SUB

' ================================ Kf.WnkRig ===================================
'
'   Procedura per la definizione della matrice di rigidezza fondamentale
'   per un'asta su suolo elastico alla Winkler
'   avente tratti rigidi di estremita`
'
'   Variabili di ingresso:
'      K         costante di sottofondo
'      B         larghezza trave
'      E         modulo di elasticita` normale
'      A         area della sezione trasversale dell'asta
'      I         momento d'inerzia della sezione trasversale dell'asta
'      L         lunghezza dell'asta
'      S         coefficiente lunghezza tratto rigido sinistro
'      T         coefficiente lunghezza tratto rigido destro
'
'   Variabili di uscita:
'      Kf()      matrice di rigidezza fondamentale
'
'   Procedure utilizzate:
'      Kf.Wnk
'      ProdottoMat
'      SommaMat
'      TrasponeMat
'
' ------------------------------------------------------------------------------
'
SUB Kf.WnkRig (K, B, e, A, I, L, S, T, Kf())

   DIM T(5, 5), TT(5, 5), K1(5, 5), KB(5, 5)
   T(1, 1) = 1
   T(2, 2) = 1
   T(3, 3) = 1
   T(4, 4) = 1
   T(5, 5) = 1
   T(2, 1) = S * L
   T(4, 3) = -T * L
   CALL TrasponeMat(T(), TT())
   CALL Kf.Wnk(K, B, e, A, I, L * (1 - S - T), Kf())
   CALL ProdottoMat(T(), Kf(), K1())
   CALL ProdottoMat(K1(), TT(), Kf())
   KB(1, 1) = K * B * S * L
   KB(2, 1) = K * B * (S * L) ^ 2 / 2
   KB(1, 2) = KB(2, 1)
   KB(2, 2) = K * B * (S * L) ^ 3 / 3
   KB(3, 3) = K * B * T * L
   KB(4, 3) = -K * B * (T * L) ^ 2 / 2
   KB(3, 4) = KB(4, 3)
   KB(4, 4) = K * B * (T * L) ^ 3 / 3
   CALL SommaMat(Kf(), KB(), Kf())

END SUB

' ================================ Kg.Cer ======================================
'
'   Procedura per la definizione della matrice di rigidezza globale
'   per un'asta incernierata agli estremi
'
'   Variabili di ingresso:
'      ES        estremo incernierato
'                   1 = primo estremo
'                   2 = secondo estremo
'                   3 = entrambi gli estremi
'      E         modulo di elasticita` normale
'      A         area della sezione trasversale dell'asta
'      I         momento d'inerzia della sezione trasversale dell'asta
'      L         lunghezza dell'asta
'      Ld()      matrice di rotazione
'
'   Variabili di uscita:
'      Kg()      matrice di rigidezza globale
'
'   Procedure utilizzate:
'      Kf.Cer
'      Kg.f
'      T.Cer
'
' ------------------------------------------------------------------------------
'
SUB Kg.Cer (ES, e, A, I, L, Ld(), Kg())
 
    SELECT CASE ES
       CASE 1, 2
          N = 2
       CASE 3
          N = 1
    END SELECT
    DIM Kf(N, N), T(6, N)
   
    CALL Kf.Cer(ES, e, A, I, L, Kf())
    CALL T.Cer(ES, L, T())
    CALL Kg.f(Ld(), T(), Kf(), Kg())

END SUB

' ================================ Kg.CerTag ===================================
'
'   Procedura per la definizione della matrice di rigidezza globale
'   per un'asta incernierata agli estremi
'   tenendo conto della deformazione a taglio
'
'   Variabili di ingresso:
'      ES        estremo incernierato
'                   1 = primo estremo
'                   2 = secondo estremo
'                   3 = entrambi gli estremi
'      E         modulo di elasticita` normale
'      G         modulo di elasticita` tangenziale
'      CHI       fattore di taglio
'      A         area della sezione trasversale dell'asta
'      I         momento d'inerzia della sezione trasversale dell'asta
'      L         lunghezza dell'asta
'      Ld()      matrice di rotazione
'
'   Variabili di uscita:
'      Kg()      matrice di rigidezza globale
'
'   Procedure utilizzate:
'      Kf.CerTag
'      Kg.f
'      T.Cer
'
' ------------------------------------------------------------------------------
'
SUB Kg.CerTag (ES, e, G, CHI, A, I, L, Ld(), Kg())
 
    SELECT CASE ES
       CASE 1, 2
          N = 2
       CASE 3
          N = 1
    END SELECT
    DIM Kf(N, N), T(6, N), Kloc(6, 6)
  
    CALL Kf.CerTag(ES, e, G, CHI, A, I, L, Kf())
    CALL T.Cer(ES, L, T())
    CALL Kg.f(Ld(), T(), Kf(), Kg())

END SUB

' ================================ Kg.f ========================================
'
'   Procedura generale per la definizione della matrice di rigidezza globale
'   a partire dalla matrice di rigidezza fondamentale
'
'   Variabili di ingresso:
'      Ld()      matrice di rotazione
'      T()       matrice di trasformazione
'      Kf()      matrice di rigidezza fondamentale
'
'   Variabili di uscita:
'      Kg()      matrice di rigidezza globale
'
'   Variabili interne:
'      LT()      prodotto di Ld per T
'      LTT()     trasposta del prodotto di Ld per T
'
'   Procedure utilizzate:
'      ProdottoMat
'      TrasponeMat
'
' ------------------------------------------------------------------------------
'
SUB Kg.f (Ld(), T(), Kf(), Kg())

    CALL LimitiMat(Kf(), DK, LRK, LCK, NRK, NCK)
    DIM LT(6, NRK), LTT(NRK, 6), K1(6, NRK)
  
    CALL ProdottoMat(Ld(), T(), LT())
    CALL TrasponeMat(LT(), LTT())
    CALL ProdottoMat(LT(), Kf(), K1())
    CALL ProdottoMat(K1(), LTT(), Kg())

END SUB

' ================================ Kg.Inc ======================================
'
'   Procedura per la definizione della matrice di rigidezza globale
'   per un'asta perfettamente incastrata ai nodi di estremita`
'
'   Variabili di ingresso:
'      E         modulo di elasticita` normale
'      A         area della sezione trasversale dell'asta
'      I         momento d'inerzia della sezione trasversale dell'asta
'      L         lunghezza dell'asta                                        
'      Ld()      matrice di rotazione
'
'   Variabili di uscita:
'      Kg()      matrice di rigidezza globale
'
'   Procedure utilizzate:
'      Kf.Inc
'      Kg.f
'      T.Inc
'
' ------------------------------------------------------------------------------
'
SUB Kg.Inc (e, A, I, L, Ld(), Kg())

    DIM Kf(3, 3), T(6, 3), Kloc(6, 6)
    CALL Kf.Inc(e, A, I, L, Kf())
    CALL T.Inc(L, T())
    CALL Kg.f(Ld(), T(), Kf(), Kg())

END SUB

' ================================ Kg.IncTag ===================================
'
'   Procedura per la definizione della matrice di rigidezza globale
'   per un'asta perfettamente incastrata ai nodi di estremita`
'   tenendo conto della deformazione a taglio
'
'   Variabili in ingresso:
'      E         modulo di elasticita` normale
'      G         modulo di elasticita` tangenziale
'      CHI       fattore di taglio
'      A         area della sezione trasversale dell'asta
'      I         momento d'inerzia della sezione trasversale dell'asta
'      L         lunghezza dell'asta
'      Ld()      matrice di rotazione
'
'   Variabili in uscita:
'      Kg()      matrice di rigidezza globale
'
'   Procedure utilizzate:
'      Kf.IncTag
'      Kg.f
'      T.Inc
'
' ------------------------------------------------------------------------------
'
SUB Kg.IncTag (e, G, CHI, A, I, L, Ld(), Kg())

    DIM Kf(3, 3), T(6, 3), Kloc(6, 6)
    CALL Kf.IncTag(e, G, CHI, A, I, L, Kf())
    CALL T.Inc(L, T())
    CALL Kg.f(Ld(), T(), Kf(), Kg())

END SUB

' ================================ Kg.l ========================================
'
'   Procedura generale per la definizione della matrice di rigidezza globale
'   a partire dalla matrice di rigidezza nel sistema di riferimento locale
'
'   Variabili di ingresso:
'      Ld()      matrice di rotazione
'      Kloc()    matrice di rigidezza locale
'
'   Variabili di uscita:
'      Kg()      matrice di rigidezza globale
'
'   Procedure utilizzate:
'      ProdottoMat
'      TrasponeMat
'
' ------------------------------------------------------------------------------
'
SUB Kg.l (Ld(), Kloc(), Kg())

    DIM LdT(6, 6)
    CALL ProdottoMat(Ld(), Kloc(), Kg())
    CALL TrasponeMat(Ld(), LdT())
    CALL ProdottoMat(Kg(), LdT(), Kg())

END SUB

' ================================ Kg.NodRig ===================================
'
'   Procedura per la determinazione della matrice di rigidezza globale
'   nel caso di nodi rigidi non puntiformi
'   (asta con tratti rigidi non coassiali)
'
'   Variabili di ingresso:
'      E         modulo di elasticita` normale
'      A         area della sezione trasversale
'      I         momento d'inerzia della sezione trasversale
'      L         lunghezza dell'asta
'      Sx        coefficiente lunghezza x tratto sinistro
'      Sy        coefficiente lunghezza y tratto sinistro
'      Tx        coefficiente lunghezza x tratto destro
'      Ty        coefficiente lunghezza y tratto destro
'      Ld()      matrice di rotazione
'
'   Variabili di uscita:
'      Kg()      matrice di rigidezza globale
'
'   Procedure utilizzate:
'      Kf.NodRig
'      Kg.f
'      T.Inc
'
' ------------------------------------------------------------------------------
'
SUB Kg.NodRig (e, A, I, L, Sx, Sy, Tx, Ty, Ld(), Kg())

    DIM Kf(3, 3), T(6, 3), Kloc(6, 6)
    CALL Kf.NodRig(e, A, I, L, Sx, Sy, Tx, Ty, Kf())
    CALL T.Inc(L, T())
    CALL Kg.f(Ld(), T(), Kf(), Kg())

END SUB

' ================================ Kg.Rig ======================================
'
'   Procedura per la definizione della matrice di rigidezza globale
'   per un'asta perfettamente incastrata ai nodi di estremita`
'   con tratti rigidi di estremita'
'
'   Variabili di ingresso:
'      E         modulo di elasticita` normale
'      A         area della sezione trasversale dell'asta
'      I         momento d'inerzia della sezione trasversale dell'asta
'      L         lunghezza dell'asta                                      
'      S         coefficiente lunghezza tratto rigido sinistro
'      T         coefficiente lunghezza tratto rigido destro
'      Ld()      matrice di rotazione
'
'   Variabili di uscita:
'      Kg()      matrice di rigidezza globale
'
'   Procedure utilizzate:
'      Kf.Rig
'      Kg.f
'      T.Inc
'
' ------------------------------------------------------------------------------
'
SUB Kg.Rig (e, A, I, L, S, T, Ld(), Kg())
    
    DIM Kf(3, 3), T(6, 3), Kloc(6, 6)
    CALL Kf.Rig(e, A, I, L, S, T, Kf())
    CALL T.Inc(L, T())
    CALL Kg.f(Ld(), T(), Kf(), Kg())

END SUB

' ================================ Kg.RigTag ===================================
'
'   Procedura per la definizione della matrice di rigidezza globale
'   per un'asta perfettamente incastrata ai nodi di estremita`
'   con tratti rigidi di estremita' - tenendo conto di deform. taglio
'
'   Variabili di ingresso:
'      E         modulo di elasticita` normale
'      G         modulo di elasticita` tangenziale
'      CHI       fattore di taglio
'      A         area della sezione trasversale dell'asta
'      I         momento d'inerzia della sezione trasversale dell'asta
'      L         lunghezza dell'asta                              
'      S         coefficiente lunghezza tratto rigido sinistro
'      T         coefficiente lunghezza tratto rigido destro
'      Ld()      matrice di rotazione
'
'   Variabili di uscita:
'      Kg()      matrice di rigidezza globale
'
'   Procedure utilizzate:
'      Kf.RigTag
'      Kg.f
'      T.Inc
'
' ------------------------------------------------------------------------------
'
SUB Kg.RigTag (e, G, CHI, A, I, L, S, T, Ld(), Kg())
 
   DIM Kf(3, 3), T(6, 3)
   CALL Kf.RigTag(e, G, CHI, A, I, L, S, T, Kf())
   CALL T.Inc(L, T())
   CALL Kg.f(Ld(), T(), Kf(), Kg())

END SUB

' ================================ Kg.Wnk ======================================
'
'   Procedura per la definizione della matrice di rigidezza globale
'   per un'asta su suolo elastico alla Winkler
'
'   Variabili di ingresso:
'      K         costante di sottofondo
'      B         larghezza trave
'      E         modulo di elasticita` normale
'      A         area della sezione trasversale dell'asta
'      I         momento d'inerzia della sezione trasversale dell'asta
'      L         lunghezza dell'asta
'      Ld()      matrice di rotazione
'
'   Variabili di uscita:
'      Kg()      matrice di rigidezza globale
'
'   Procedure utilizzate:
'      Kf.Wnk
'      Kg.f
'      T.Wnk
'
' ------------------------------------------------------------------------------
'
SUB Kg.Wnk (K, B, e, A, I, L, Ld(), Kg())

    DIM Kf(5, 5), T(6, 5), Kloc(6, 6)
    CALL Kf.Wnk(K, B, e, A, I, L, Kf())
    CALL T.Wnk(T())
    CALL Kg.f(Ld(), T(), Kf(), Kg())

END SUB

' ================================ Kg.WnkRig ===================================
'
'   Procedura per la definizione della matrice di rigidezza globale
'   per un'asta su suolo elastico alla Winkler
'   avente tratti rigidi di estremita`
'
'   Variabili di ingresso:
'      K         costante di sottofondo
'      B         larghezza trave
'      E         modulo di elasticita` normale
'      A         area della sezione trasversale dell'asta
'      I         momento d'inerzia della sezione trasversale dell'asta
'      L         lunghezza dell'asta
'      S         coefficiente lunghezza tratto rigido sinistro
'      T         coefficiente lunghezza tratto rigido destro
'      Ld()      matrice di rotazione
'
'   Variabili di uscita:
'      Kg()      matrice di rigidezza globale
'
'   Procedure utilizzate:
'      Kf.WnkRig
'      Kg.f
'      T.Wnk
'
' ------------------------------------------------------------------------------
'
SUB Kg.WnkRig (K, B, e, A, I, L, S, T, Ld(), Kg())

    DIM Kf(5, 5), T(6, 5), Kloc(6, 6)
    CALL Kf.WnkRig(K, B, e, A, I, L, S, T, Kf())
    CALL T.Wnk(T())
    CALL Kg.f(Ld(), T(), Kf(), Kg())

END SUB

' ================================ Kl. =========================================
'
'   Procedura generale per la definizione della matrice di rigidezza locale
'   a partire dalla matrice di rigidezza fondamentale
'
'   Variabili di ingresso:
'      T()       matrice di trasformazione
'      Kf()      matrice di rigidezza fondamentale
'
'   Variabili di uscita:
'      Kloc()    matrice di rigidezza locale
'
'   Procedure utilizzate:
'      LimitiMat
'      ProdottoMat
'      TrasponeMat
'
' ------------------------------------------------------------------------------
'
SUB Kl. (T(), Kf(), Kloc())

    CALL LimitiMat(Kf(), DK, LRK, LCK, NRK, NCK)
    DIM K1(6, NRK), TT(NRK, 6)
   
    CALL ProdottoMat(T(), Kf(), K1())
    CALL TrasponeMat(T(), TT())
    CALL ProdottoMat(K1(), TT(), Kloc())

END SUB

' ================================ Sl.Cer.Qunif ================================
'
'   Procedura per il calcolo delle azioni di incastro perfetto
'   per un'asta incernierata agli estremi: carico uniforme
'
'   Variabili di ingresso:
'      ES        estremo incernierato
'                   1 = primo estremo
'                   2 = secondo estremo
'                   3 = entrambi gli estremi
'      Q         carico uniformemente distribuito ortogonale all'asse
'      N         carico uniformemente distribuito parallelo  all'asse
'      L         lunghezza dell'asta
'
'   Variabili di uscita:
'      Sl()      vettore azioni di incastro perfetto
'
'   Procedure utilizzate:
'      AzzeraMat
'
' ------------------------------------------------------------------------------
'
SUB Sl.Cer.Qunif (ES, Q, N, L, Sl())

    CALL AzzeraMat(Sl())
    Sl(1, 1) = -N * L / 2
    Sl(4, 1) = Sl(1, 1)
    SELECT CASE ES
       CASE 1
          Sl(2, 1) = -3 / 8 * L * Q
          Sl(5, 1) = -5 / 8 * L * Q
          Sl(6, 1) = 1 / 8 * L ^ 2 * Q
       CASE 2
          Sl(2, 1) = -5 / 8 * L * Q
          Sl(3, 1) = -1 / 8 * L ^ 2 * Q
          Sl(5, 1) = -3 / 8 * L * Q
       CASE 3
          Sl(2, 1) = -1 / 2 * L * Q
          Sl(5, 1) = Sl(2, 1)
    END SELECT

END SUB

' ================================ Sl.Inc.DT ===================================
'
'   Procedura per il calcolo delle azioni di incastro perfetto
'   per un'asta perfettamente incastrata ai nodi di estremita`:
'   distorsione termica trapezia
'
'   Variabili di ingresso:
'      DTu       variazione termica uniforme
'      DTf       variazione termica a farfalla
'      E         modulo di elasticita` normale
'      A         area della sezione trasversale dell'asta
'      I         momento d'inerzia della sezione trasversale dell'asta
'      H         altezza della sezione trasversale dell'asta
'      At        coefficiente di dilatazione termica
'
'   Variabili di uscita:
'      Sl()      vettore azioni di incastro perfetto
'
'   Procedure utilizzate:
'      ===
'
' ------------------------------------------------------------------------------
'
SUB Sl.Inc.DT (DTu, DTf, e, A, I, H, At, Sl())
   
    Sl(1, 1) = e * A * At * DTu
    Sl(2, 1) = 0
    Sl(3, 1) = -e * I * At / H * DTf
    Sl(4, 1) = -Sl(1, 1)
    Sl(5, 1) = 0
    Sl(6, 1) = -Sl(3, 1)

END SUB

' ================================ Sl.Inc.FM ===================================
'
'   Procedura per il calcolo delle azioni di incastro perfetto
'   per un'asta perfettamente incastrata ai nodi di estremita`:
'   forza F e coppia M concentrate
'
'   Variabili di ingresso:
'      Fx        componente della forza concentrata parallela all'asta
'      Fy        componente della forza concentrata normale all'asta
'      Mz        coppia concentrata
'      X0        distanza dal primo estremo delle azioni concentrate
'      L         lunghezza dell'asta
'
'   Variabili di uscita:
'      Sl()      vettore azioni di incastro perfetto
'
'   Procedure utilizzate:
'      ===
'
' ------------------------------------------------------------------------------
'
SUB Sl.Inc.FM (Fx, Fy, Mz, X0, L, Sl())

    IF X0 < 0 OR X0 > L THEN
       PRINT "ERRORE 1 in Sl.Inc.FM - valori dell'ascissa non accettabili"
       END
    END IF
   
    A = X0 / L
    B = 1 - A
    Sl(1, 1) = -B * Fx
    Sl(2, 1) = -B ^ 2 * (1 + 2 * A) * Fy + 6 * A * B / L * Mz
    Sl(3, 1) = -A * B ^ 2 * L * Fy + B * (2 * A - B) * Mz
    Sl(4, 1) = -A * Fx
    Sl(5, 1) = -A ^ 2 * (1 + 2 * B) * Fy - 6 * A * B / L * Mz
    Sl(6, 1) = A ^ 2 * B * L * Fy + A * (2 * B - A) * Mz

END SUB

' ================================ Sl.Inc.Qunif ================================
'
'   Procedura per il calcolo delle azioni di incastro perfetto
'   per un'asta perfettamente incastrata ai nodi di estremita`:
'   carico uniformemente distribuito
'
'   Variabili di ingresso:
'      Q         carico uniformemente distribuito ortogonale all'asta
'      N         carico uniformemente distribuito parallelo  all'asta
'      L         lunghezza dell'asta
'
'   Variabili di uscita:
'      Sl()      vettore azioni di incastro perfetto
'
'   Procedure utilizzate:
'      ===
' ------------------------------------------------------------------------------
'
SUB Sl.Inc.Qunif (Q, N, L, Sl())
  
    Sl(1, 1) = -N * L / 2
    Sl(2, 1) = -Q * L / 2
    Sl(3, 1) = -Q * L ^ 2 / 12
    Sl(4, 1) = Sl(1, 1)
    Sl(5, 1) = Sl(2, 1)
    Sl(6, 1) = -Sl(3, 1)

END SUB

' ================================ Sl.Inc.Qvar =================================
'
'   Procedura per il calcolo delle azioni di incastro perfetto
'   per un'asta perfettamente incastrata ai nodi di estremita`:
'   carico distribuito con andamento lineare su parte dell'asta
'
'   Variabili di ingresso:
'      Q1        valore del carico a sinistra
'      Q2        valore del carico a destra
'      L         lunghezza dell'asta
'      X1        distanza dal primo estremo dell'inizio carico
'      X2        distanza dal primo estremo della fine carico
'
'   Variabili di uscita:
'      Sl()      vettore azioni di incastro perfetto
'
'   Procedure utilizzate:
'      ===
'
' ------------------------------------------------------------------------------
'
SUB Sl.Inc.Qvar (Q1, Q2, L, X1, X2, Sl())

    IF X1 < 0 OR X1 > L OR X2 < 0 OR X2 > L THEN
       PRINT "ERRORE 1 in Sl.Inc.Qvar - valori delle ascisse non accettabili"
       END
    END IF
    IF X1 > X2 THEN
       PRINT "ERRORE 2 in Sl.Inc.Qvar - ordine delle ascisse non accettabile"
       END
    END IF
   
    r1 = 3 / 2 - (X1 + X2 / 2) / L
    s1 = 3 - 2 * r1
    r2 = (X2 + X1 / 2) / L
    s2 = 3 - 2 * r2
    T = (X2 - X1) / L
  
    r = r1: S = s1
    GOSUB FGH
    f1 = F: g1 = G: h1 = H
  
    r = r2: S = s2
    GOSUB FGH
    f2 = F: g2 = G: h2 = H
  
    Sl(1, 1) = 0
    Sl(2, 1) = -(20 * r1 * T + h1) / 60 * L * Q1 - (10 * s2 * T - h2) / 60 * L * Q2
    Sl(3, 1) = -f1 * L ^ 2 / 20 * Q1 - g2 * L ^ 2 / 30 * Q2
    Sl(4, 1) = 0
    Sl(5, 1) = -(10 * s1 * T - h1) / 60 * L * Q1 - (20 * r2 * T - h2) / 60 * L * Q2
    Sl(6, 1) = g1 * L ^ 2 / 30 * Q1 + f2 * L ^ 2 / 20 * Q2

    EXIT SUB

FGH:
    F = T / 27 * (40 * r ^ 2 * S + T ^ 2 * (2 * T - 30 * r + 15))
    G = T / 18 * (20 * r * S ^ 2 - T ^ 2 * (2 * T - 30 * r + 30))
    H = 3 * F - 2 * G
    RETURN

END SUB

' ================================ Sl.Rig.Qunif ================================
'
'   Procedura per il calcolo delle azioni di incastro perfetto
'   per un'asta perfettamente incastrata ai nodi di estremita`
'   avente tratti rigidi di estremita`: carico uniforme
'
'   Variabili di ingresso:
'      Q         carico uniformemente distribuito ortogonale all'asta
'      N         carico uniformemente distribuito parallelo  all'asta
'      L         lunghezza asta
'      S         coefficiente lunghezza tratto rigido sinistro
'      T         coefficiente lunghezza tratto rigido destro
'
'   Variabili di uscita:
'      Sl()      vettore azioni di incastro perfetto
'
'   Procedure utilizzate:
'      ===
'
' ------------------------------------------------------------------------------
'
SUB Sl.Rig.Qunif (Q, N, L, S, T, Sl())

    Z1 = (1 - S - T) ^ 2 + 6 * S * (1 - S - T) + 6 * S ^ 2
    Z2 = (1 - S - T) ^ 2 + 6 * T * (1 - S - T) + 6 * T ^ 2

    Sl(1, 1) = -N * L / 2
    Sl(2, 1) = -(1 + S - T) / 2 * L * Q
    Sl(3, 1) = -Z1 * L ^ 2 / 12 * Q
    Sl(4, 1) = Sl(1, 1)
    Sl(5, 1) = -(1 - S + T) / 2 * L * Q
    Sl(6, 1) = Z2 * L ^ 2 / 12 * Q

END SUB

' ================================ Sl.Wnk.Qunif ================================
'
'   Procedura per il calcolo delle azioni di incastro perfetto
'   per un'asta su suolo elastico alla Winkler: carico uniforme
'   N.B. vale anche se vi sono tratti rigidi, purch sia fornita la
'        giusta matrice di rigidezza locale
'
'   Variabili di ingresso:
'      Q         carico uniformemente distribuito ortogonale all'asta
'      N         carico uniformemente distribuito parallelo  all'asta
'      L         lunghezza dell'asta
'      K         costante di sottofondo
'      B         larghezza della trave
'      Kloc()    matrice di rigidezza locale
'
'   Variabili di uscita:
'      Sl()      vettore azioni di incastro perfetto
'
'   Procedure utilizzate:
'      AzzeraMat
'      ProdottoMat
'
' ------------------------------------------------------------------------------
'
SUB Sl.Wnk.Qunif (Q, N, L, K, B, Kloc(), Sl())

    CALL AzzeraMat(Sl())
    Sl(2, 1) = -1 / (K * B) * Q
    Sl(5, 1) = Sl(2, 1)
    CALL ProdottoMat(Kloc(), Sl(), Sl())
    Sl(1, 1) = Sl(1, 1) - N * L / 2
    Sl(4, 1) = Sl(4, 1) - N * L / 2

END SUB

' ================================ T.Cer =======================================
'
'   Procedura per la definizione della matrice di trasformazione
'   per un'asta incernierata agli estremi
'
'   Variabili di ingresso:
'      ES        estremo incernierato
'                   1 = primo estremo
'                   2 = secondo estremo
'                   3 = entrambi gli estremi
'      L         lunghezza dell'asta
'
'   Variabili di uscita:
'      T()       matrice di trasformazione
'
'   Procedure utilizzate:
'      AzzeraMat
'
' ------------------------------------------------------------------------------
'
SUB T.Cer (ES, L, T())
  
   CALL AzzeraMat(T())
   SELECT CASE ES
      CASE 1
         T(1, 2) = -1
         T(2, 1) = 1 / L
         T(4, 2) = 1
         T(5, 1) = -1 / L
         T(6, 1) = 1
      CASE 2
         T(1, 2) = -1
         T(2, 1) = 1 / L
         T(3, 1) = 1
         T(4, 2) = 1
         T(5, 1) = -1 / L
      CASE 3
         T(1, 1) = -1
         T(4, 1) = 1
   END SELECT

END SUB

' ================================ T.Inc =======================================
'
'   Procedura per la definizione della matrice di trasformazione
'   per un'asta perfettamente incastrata ai nodi di estremita`
'
'   Variabili di ingresso:
'      L      lunghezza dell'asta
'
'   Variabili di uscita:
'      T()    matrice di trasformazione
'
'   Procedure utilizzate:
'      AzzeraMat
'
' ------------------------------------------------------------------------------
'
SUB T.Inc (L, T())

    CALL AzzeraMat(T())
    T(1, 3) = -1
    T(2, 1) = 1 / L
    T(2, 2) = 1 / L
    T(3, 1) = 1
    T(4, 3) = 1
    T(5, 1) = -1 / L
    T(5, 2) = -1 / L
    T(6, 2) = 1

END SUB

' ================================ T.Wnk =======================================
'
'   Procedura per la definizione della matrice di trasformazione
'   per un'asta su suolo elastico alla Winkler
'
'   Variabili di ingresso:
'      ===
'
'   Variabili di uscita:
'      T()       matrice di trasformazione
'
'   Procedure utilizzate:
'      AzzeraMat
'
' ------------------------------------------------------------------------------
'
SUB T.Wnk (T())

    CALL AzzeraMat(T())
    T(1, 5) = -1
    T(2, 1) = 1
    T(3, 2) = 1
    T(4, 5) = 1
    T(5, 3) = 1
    T(6, 4) = 1

END SUB

