'                                  MOANA.BAS

'     MOdal ANAlysis - procedure per l'analisi modale

'     ver. 1.3                                            rev. 05.94

' procedure del file MATRIX.BAS
DECLARE SUB LimitiMat (A!(), DA!, LR!, LC!, NR!, NC!)
DECLARE SUB ProdottoMat (A!(), B!(), C!())

' procedure di questo stesso file
DECLARE SUB CopiaMaRig (Rig!(), Mas!(), n!, A() AS DOUBLE, B() AS DOUBLE)
DECLARE SUB CopiaMas1 (Mas!(), n!, B() AS DOUBLE)
DECLARE SUB CopiaMas2 (Mas!(), n!, B() AS DOUBLE)
DECLARE SUB Jacobi (Rig!(), Mas!(), n!, RTol!, ItMax!, F!, EigVet!(), EigVal!())
DECLARE FUNCTION Risposta! (T!, SpePar!())
DECLARE FUNCTION Risposta2! (T!, nCop!, SpeT!(), SpeR!())

CONST PI = 3.141593

' ================================ AnalisiModale ===============================
'
'  Effettua analisi modale
'
'  Variabili di ingresso:
'     Klat()    matrice di rigidezza laterale del telaio
'     MA()      vettore delle masse
'     KZ        numero di traversi
'     nMod      numero di modi richiesti
'
'  Variabili di uscita:
'     EigVal()  autovalori, disposti in ordine crescente
'     EigVet()  matrice che contiene gli autovettori, disposti in colonne
'     T()       periodo dei singoli modi
'     Part()    coefficienti di partecipazione modale
'     Forze()   forze modali (a meno di una costante)
'     Energia() energia (a meno di una costante)
'
' ------------------------------------------------------------------------------
SUB AnalisiModale (Klat(), MA(), KZ, nMod, EigVal(), EigVet(), T(), Part(), Forze(), Energia())
  
   CALL Jacobi(Klat(), MA(), KZ, 0, 0, 0, EigVet(), EigVal())
   FOR j = 1 TO nMod
      T(j) = 2 * PI / SQR(EigVal(j))
      Numer = 0
      Denom = 0
      FOR i = 1 TO KZ
         Forze(i, j) = MA(i) * EigVet(i, j)
         Numer = Numer + Forze(i, j)
         Denom = Denom + Forze(i, j) * EigVet(i, j)
      NEXT i
      Part(j) = Numer / Denom    ' N.B. Denom dovrebbe essere sempre 1
      Energia(j) = Denom
   NEXT j

END SUB

' ================================ AnalisiModale3D =============================
'
'  Effettua analisi modale per schema spaziale
'
'  Variabili di ingresso:
'     Klat()    matrice di rigidezza laterale del telaio
'     MA()      vettore delle masse
'     PSZ       numero di impalcati
'     nMod      numero di modi richiesti
'     AngSisma  angolo del sisma (in gradi)
'
'  Variabili di uscita:
'     EigVal()  autovalori, disposti in ordine crescente
'     EigVet()  matrice che contiene gli autovettori, disposti in colonne
'     T()       periodo dei singoli modi
'     Part()    coefficienti di partecipazione modale
'     Forze()   forze modali (a meno di una costante)
'     Energia() energia (a meno di una costante)
'
' ------------------------------------------------------------------------------
SUB AnalisiModale3D (Klat(), MA(), PSZ, nMod, AngSisma, EigVal(), EigVet(), T(), Part(), Forze(), Energia())

   nGdL = 3 * PSZ
   DIM Coef(1 TO nGdL, 1 TO 1)
   Cx = COS(AngSisma / 180 * PI)
   Cy = SIN(AngSisma / 180 * PI)
   FOR PS = 1 TO PSZ
      Coef(PS, 1) = Cx:   Coef(PS + PSZ, 1) = Cy
   NEXT PS
   CALL Jacobi(Klat(), MA(), nGdL, 0, 0, 0, EigVet(), EigVal())
   FOR j = 1 TO nMod
      T(j) = 2 * PI / SQR(EigVal(j))
      REDIM Eig(1 TO 1, 1 TO nGdL), EigT(1 TO nGdL, 1 TO 1)
      REDIM EigM(1 TO 1, 1 TO nGdL)
      REDIM Numer(1 TO 1, 1 TO 1), Denom(1 TO 1, 1 TO 1)
      FOR i = 1 TO nGdL
         Eig(1, i) = EigVet(i, j)
         EigT(i, 1) = Eig(1, i)
      NEXT i
      CALL ProdottoMat(Eig(), MA(), EigM())
      CALL ProdottoMat(EigM(), Coef(), Numer())
      CALL ProdottoMat(EigM(), EigT(), Denom())
      Part(j) = Numer(1, 1) / Denom(1, 1)  ' N.B. Denom dovrebbe essere sempre 1
      FOR i = 1 TO nGdL
         Forze(i, j) = EigM(1, i)
      NEXT i
      Energia(j) = Denom(1, 1)
   NEXT j

END SUB

' ================================ CopiaMaRig ==================================
'
'  Copia Rig in A e Mas in B per evitare che vengano modificati
'
'  Variabili di ingresso:
'     Rig()  matrice di rigidezza
'     Mas()  matrice (o vettore) delle masse
'     n      ordine delle matrici K ed M
'
'  Variabili di uscita:
'     A()   copia della matrice di rigidezza
'     B()   copia della matrice delle masse
'
'  Annotazioni:
'     Mas()  pu essere un vettore o una matrice;  per tale motivo usa due
'     procedure (rispettivamente CopiaMas1 e CopiaMas2) per farne la copia
'
' ------------------------------------------------------------------------------
SUB CopiaMaRig (Rig(), Mas(), n, A() AS DOUBLE, B() AS DOUBLE)
  
   CALL LimitiMat(Mas(), DA!, LR, LC, NR, NC)
   IF DA = 1 THEN
      CALL CopiaMas1(Mas(), n, B())
   ELSE
      CALL CopiaMas2(Mas(), n, B())
   END IF
   ' --- copia K in A e M in B per evitare che vengano modificati
   FOR i = 1 TO n
      FOR j = 1 TO n
         A(i, j) = Rig(i, j)
      NEXT j
      IF A(i, i) <= 0 OR B(i, i) <= 0 THEN
         PRINT "ERRORE - le matrici non sono definite positive"
         STOP
      END IF
   NEXT i

END SUB

SUB CopiaMas1 (Mas(), n, B() AS DOUBLE)

   FOR i = 1 TO n
      B(i, i) = Mas(i)
   NEXT i

END SUB

SUB CopiaMas2 (Mas(), n, B() AS DOUBLE)

   FOR i = 1 TO n
      FOR j = 1 TO n
         B(i, j) = Mas(i, j)
      NEXT j
   NEXT i

END SUB

' ================================ InputSpettro ================================
'   Procedura per la lettura dello spettro di progetto (per l'analisi modale)
' ------------------------------------------------------------------------------
SUB InputSpettro (Contr, SpePar(), SpeT(), SpeR())
 
   INPUT #1, SpePar(1)
   IF Contr THEN PRINT #2, "tipo di spettro"; SpePar(1)
   SELECT CASE SpePar(1)
      CASE 1
         IF Contr THEN PRINT #2, "spettro di risposta assegnato con coppie T-R"
         INPUT #1, SpePar(2)
         IF Contr THEN PRINT #2, "numero coppie ="; SpePar(2)
         IF SpePar(2) < 1 THEN
            PRINT #2, "ERRORE - numero di coppie T-R non ammissibile"
            END
         END IF
         REDIM SpeT(1 TO SpePar(2)), SpeR(1 TO SpePar(2))
         FOR Cop = 1 TO SpePar(2)
            INPUT #1, SpeT(Cop), SpeR(Cop)
            IF Contr THEN
               PRINT #2, "T ="; SpeT(Cop); "     R ="; SpeR(Cop)
            END IF
         NEXT Cop
      CASE 2
         IF Contr THEN PRINT #2, "spettro tipo EC8 generalizzato"
         FOR i = 2 TO 12
            INPUT #1, SpePar(i)
         NEXT i
         IF Contr THEN
            PRINT #2, "alfa ="; SpePar(2), "  q ="; SpePar(3)
            PRINT #2, "S ="; SpePar(4), "  beta0 ="; SpePar(5)
            PRINT #2, "periodi    TB ="; SpePar(6), "  TC ="; SpePar(7), "  TD ="; SpePar(8)
            PRINT #2, "esponenti  k1 ="; SpePar(9), "  k2 ="; SpePar(10)
            PRINT #2, "R minimo ="; SpePar(11)
            PRINT #2, "smorzamento del telaio ="; SpePar(12); " %"
         END IF
      CASE 3
         IF Contr THEN PRINT #2, "EC8 (ottobre 1993) - spettro di progetto"
         INPUT #1, SpePar(2), SpePar(3), Suolo$
         S$ = UCASE$(LEFT$(Suolo$, 1))
         SpePar(12) = INSTR("ABC", S$)
         IF Contr THEN
            PRINT #2, "alfa ="; SpePar(2), "  q ="; SpePar(3)
            PRINT #2, "tipo di suolo: "; Suolo$
         END IF
         IF SpePar(12) = 0 THEN
            PRINT #2, "ERRORE - il tipo di suolo "; S$; " non  ammissibile"
            END
         ELSEIF SpePar(12) = 1 THEN
            SpePar(4) = 1:   SpePar(6) = .1:    SpePar(7) = .4
         ELSEIF SpePar(12) = 2 THEN
            SpePar(4) = 1:   SpePar(6) = .15:   SpePar(7) = .6
         ELSE
            SpePar(4) = .9:  SpePar(6) = .2:    SpePar(7) = .8
         END IF
         SpePar(5) = 2.5
         SpePar(8) = 3
         SpePar(9) = .6666667
         SpePar(10) = 1.666667
         SpePar(11) = .2
      CASE 4
         IF Contr THEN PRINT #2, "DM 1986 - spettro di progetto"
         INPUT #1, SpePar(2), SpePar(3), SpePar(4), SpePar(5)
         IF Contr THEN
            PRINT #2, "grado di sismicita' S ="; SpePar(2)
            PRINT #2, "coefficiente di fondazione epsilon ="; SpePar(3)
            PRINT #2, "coefficiente di struttura beta ="; SpePar(4)
            PRINT #2, "coefficiente di destinazione I ="; SpePar(5)
         END IF
      CASE ELSE
         PRINT #2, "ERRORE - il tipo di spettro"; SpePar(1); " non  ammissibile"
         END
   END SELECT

END SUB

DEFDBL A-D, S-T, X
' ================================ Jacobi ======================================
'
'  Determinazione di autovalori ed autovettori mediante il metodo di Jacobi
'  generalizzato
'
'  Variabili di ingresso:
'     Rig()  matrice di rigidezza (definita positiva)
'     Mas()  vettore o matrice delle masse (definita positiva)
'     n      ordine delle matrici K ed M
'     RTol   tolleranza nella convergenza (se nullo si assume 1E-12)
'     ItMax  massimo numero di iterazioni consentite (se nullo si assume 15)
'     F      flag per stampare risultati intermedi durante le iterazioni
'               F=0     non stampare
'               F<>0    stampa nel buffer F (il file di uscita deve essere
'                                gi stato aperto dal programma chiamante)
'
'  Variabili di uscita:
'     EigVet()  matrice che contiene gli autovettori, disposti in colonne
'     EigVal()  autovalori, disposti in ordine crescente
'
'  Annotazioni:
'     La procedura  derivata dalla subroutine JACOBI, scritta in FORTRAN,
'     descritta nel testo:
'     K.J. Bathe, E.L. Wilson, Numerical Methods in Finite Element Analysis,
'     Prentice-Hall, 1976
'
' ------------------------------------------------------------------------------
SUB Jacobi (Rig(), Mas(), n, RTol, ItMax, F, EigVet(), EigVal())

   IF ItMax = 0 THEN ItMax = 15
   IF RTol = 0 THEN RTol = 1E-12

   '  Variabili interne:
   DIM A(1 TO n, 1 TO n) AS DOUBLE        ' matrice di rigidezza
   DIM B(1 TO n, 1 TO n) AS DOUBLE        ' matrice delle masse
   DIM X(1 TO n, 1 TO n) AS DOUBLE        ' autovettori
   DIM D(1 TO n) AS DOUBLE                ' autovalori del ciclo precedente
   DIM EIGV(1 TO n) AS DOUBLE             ' autovalori del ciclo corrente
  
   ' Gli array Rig() ed Mas() vengono duplicati sia per evitarne la modifica
   '     che per averli in doppia precisione
   ' Gli array EigVet() ed EigVal() vengono duplicati solo per averli in
   '     doppia precisione
 
   ' --- copia K in A e M in B per evitare che vengano modificati
   CALL CopiaMaRig(Rig(), Mas(), n, A(), B())

   ' --- inizializza le matrici di autovalori ed autovettori
   FOR i = 1 TO n
      D(i) = A(i, i) / B(i, i)
      EIGV(i) = D(i)
   NEXT i
   '     X()  gi stata azzerata dal dimensionamento
   FOR i = 1 TO n
      X(i, i) = 1
   NEXT i

   IF n = 1 THEN
      EigVal(1) = EIGV(1)
      EigVet(1, 1) = X(1, 1)
      EXIT SUB
   END IF

   ' --- inizializza il contatore delle iterazioni e inizia l'iterazione
   Iter = 0
   DO
      Iter = Iter + 1
      IF F <> 0 THEN PRINT #F, "Procedura Jacobi - iterazione"; Iter
      ' --- controlla se il corrente elemento fuori dalla diagonale 
      '     abbastanza grande da richiedere azzeramento
      Eps = .01 ^ (2 * Iter)
      FOR j = 1 TO n - 1
         FOR K = j + 1 TO n
            EptolA = A(j, K) ^ 2 / (A(j, j) * A(K, K))
            EptolB = B(j, K) ^ 2 / (B(j, j) * B(K, K))
            IF EptolA >= Eps OR EptolB >= Eps THEN
               ' --- l'azzeramento  richiesto:
               '     calcola gli elementi CA e CG della matrice di rotazione
               Akk = A(K, K) * B(j, K) - B(K, K) * A(j, K)
               Ajj = A(j, j) * B(j, K) - B(j, j) * A(j, K)
               AB = A(j, j) * B(K, K) - A(K, K) * B(j, j)
               Check = AB ^ 2 / 4 + Akk * Ajj
               IF Check < 0 THEN
                  PRINT "ERRORE - le matrici non sono definite positive"
                  STOP
               END IF
               SqCh = SQR(Check)
               D1 = AB / 2 + SqCh
               D2 = AB / 2 - SqCh
               Den = D1
               IF ABS(D2) > ABS(D1) THEN Den = D2
               IF Den = 0 THEN
                  CA = 0
                  CG = -A(j, K) / A(K, K)
               ELSE
                  CA = Akk / Den
                  CG = -Ajj / Den
               END IF
               ' --- effettua la rotazione generalizzata per azzerare
               '     il corrente elemento fuori dalla diagonale
               IF n <> 2 THEN
                  IF j - 1 >= 1 THEN
                     FOR i = 1 TO j - 1
                        Aj = A(i, j)
                        Bj = B(i, j)
                        Ak = A(i, K)
                        Bk = B(i, K)
                        A(i, j) = Aj + CG * Ak
                        B(i, j) = Bj + CG * Bk
                        A(i, K) = Ak + CA * Aj
                        B(i, K) = Bk + CA * Bj
                     NEXT i
                  END IF
                  IF K + 1 <= n THEN
                     FOR i = K + 1 TO n
                        Aj = A(j, i)
                        Bj = B(j, i)
                        Ak = A(K, i)
                        Bk = B(K, i)
                        A(j, i) = Aj + CG * Ak
                        B(j, i) = Bj + CG * Bk
                        A(K, i) = Ak + CA * Aj
                        B(K, i) = Bk + CA * Bj
                     NEXT i
                  END IF
                  IF j + 1 <= K - 1 THEN
                     FOR i = j + 1 TO K - 1
                        Aj = A(j, i)
                        Bj = B(j, i)
                        Ak = A(i, K)
                        Bk = B(i, K)
                        A(j, i) = Aj + CG * Ak
                        B(j, i) = Bj + CG * Bk
                        A(i, K) = Ak + CA * Aj
                        B(i, K) = Bk + CA * Bj
                     NEXT i
                  END IF
               END IF
               Ak = A(K, K)
               Bk = B(K, K)
               A(K, K) = Ak + 2 * CA * A(j, K) + CA ^ 2 * A(j, j)
               B(K, K) = Bk + 2 * CA * B(j, K) + CA ^ 2 * B(j, j)
               A(j, j) = A(j, j) + 2 * CG * A(j, K) + CG ^ 2 * Ak
               B(j, j) = B(j, j) + 2 * CG * B(j, K) + CG ^ 2 * Bk
               A(j, K) = 0
               B(j, K) = 0
               ' --- aggiorna la matrice degli autovettori dopo ogni rotazione
               FOR i = 1 TO n
                  Xj = X(i, j)
                  Xk = X(i, K)
                  X(i, j) = Xj + CG * Xk
                  X(i, K) = Xk + CA * Xj
               NEXT i
            END IF
         NEXT K
      NEXT j
      ' --- aggiorna gli autovalori dopo ogni iterazione
      FOR i = 1 TO n
         IF A(i, i) <= 0 OR B(i, i) <= 0 THEN
            PRINT "ERRORE - le matrici non sono definite positive"
            STOP
         END IF
         EIGV(i) = A(i, i) / B(i, i)
      NEXT i
      IF F <> 0 THEN
         PRINT #F, "I correnti autovalori sono:"
         FOR i = 1 TO n
            PRINT #F, EIGV(i),
         NEXT i
         PRINT #F,
         PRINT #F,
      END IF
      GOSUB CheckConverg
      IF NOT Converg THEN
         ' --- aggiorna la matrice D
         FOR i = 1 TO n
            D(i) = EIGV(i)
         NEXT i
      END IF
   LOOP UNTIL Converg OR Iter > ItMax
   ' --- riempe il triangolo inferiore della matrice risultante e
   '     scala gli autovalori
   FOR i = 1 TO n
      FOR j = 1 TO n
         A(j, i) = A(i, j)
         B(j, i) = B(i, j)
      NEXT j
   NEXT i
   FOR j = 1 TO n
      BB = SQR(B(j, j))
      FOR K = 1 TO n
         X(K, j) = X(K, j) / BB
      NEXT K
   NEXT j
 
   ' --- copia X in EigVet e EIGV in EigVal
   FOR i = 1 TO n
      FOR j = 1 TO n
         EigVet(i, j) = X(i, j)
      NEXT j
      EigVal(i) = EIGV(i)
   NEXT i
   ' --- ordina gli autovalori in senso crescente
   FOR i = 1 TO n - 1
      EigMin = EigVal(i)
      kMin = i
      FOR K = i + 1 TO n
         IF EigVal(K) < EigMin THEN EigMin = EigVal(K): kMin = K
      NEXT K
      IF kMin <> i THEN
         SWAP EigVal(i), EigVal(kMin)
         FOR j = 1 TO n
            SWAP EigVet(j, i), EigVet(j, kMin)
         NEXT j
      END IF
   NEXT i

EXIT SUB
     
CheckConverg:
' --- controlla la convergenza per vedere se  necessaria un'altra iterazione
'     Converg=0  (falso)   indica che la convergenza non  stata raggiunta
'     Converg=-1  (vero)   indica che la convergenza  stata raggiunta
  
   ' --- controlla gli elementi della diagonale
   FOR i = 1 TO n
      Tol = RTol * D(i)
      Dif = ABS(EIGV(i) - D(i))
      IF Dif > Tol THEN Converg = 0: RETURN
   NEXT i
   ' --- controlla gli elementi fuori diagonale
   Eps = RTol ^ 2
   FOR j = 1 TO n - 1
      FOR K = j + 1 TO n
         EpsA = A(j, K) ^ 2 / (A(j, j) * A(K, K))
         EpsB = B(j, K) ^ 2 / (B(j, j) * B(K, K))
         IF EpsA >= Eps OR EpsB >= Eps THEN Converg = 0: RETURN
      NEXT K
   NEXT j
   ' --- superati i controlli, la convergenza  raggiunta
   Converg = -1
RETURN

END SUB

DEFSNG A-D, S-T, X
' ================================ OutputModi ==================================
'
'  Stampa un singolo modo
'
'  Variabili di ingresso:
'     KZ        numero di traversi
'     j         modo richiesto
'     EigVal()  autovalori, disposti in ordine crescente
'     EigVet()  matrice che contiene gli autovettori, disposti in colonne
'     T()       periodo dei singoli modi
'     Beta()    ordinata dello spettro di risposta
'     Part()    coefficienti di partecipazione modale
'     Forza()   forze corrispondenti al modo
'     R()       spostamento corrispondente alle forze
'     EnIso     energia isolatori
'     EnTot     energia totale
'     Smorz     smorzamento
'
' ------------------------------------------------------------------------------
SUB OutputModi (SM, KZ, j, EigVal(), EigVet(), T(), Beta(), Part(), Forza(), R(), EnIso, EnTot, Smorz)
 
   PRINT #2, : PRINT #2, : PRINT #2,
   PRINT #2, "ANALISI MODALE - modo"; j
   PRINT #2,
   PRINT #2, USING "T =##.#### s          "; T(j);
   PRINT #2, USING "omega2 =#####.####         "; EigVal(j);
   PRINT #2, USING "p =####.####"; Part(j)
   PRINT #2,
   IF SM = 0 THEN F$ = "kNm     " ELSE F$ = "tm      "
   IF EnTot <> 0 THEN
      PRINT #2, USING "energia - isol. ##.####^^^^ " + F$; EnIso;
      PRINT #2, USING "tot. ##.####^^^^ " + F$; EnTot;
      PRINT #2, USING "rapporto ##.####"; EnIso / EnTot
      PRINT #2,
   END IF
   IF Smorz <> 0 THEN PRINT #2, USING "smorzamento   ##.### %              "; Smorz;
   PRINT #2, USING "ordinata spettrale  beta =##.####       "; Beta(j)
   PRINT #2,
   IF SM = 0 THEN
      PRINT #2, " traverso       def.modale    forze (kN)"
   ELSE
      PRINT #2, " traverso       def.modale     forze (t)"
   END IF
   PRINT #2,
   FOR i = KZ TO 1 STEP -1
      PRINT #2, USING "   ##          ###.######     "; i; EigVet(i, j);
      IF SM = 0 THEN
         PRINT #2, USING "#####.##"; Forza(i)
      ELSE
         PRINT #2, USING "####.###"; Forza(i)
      END IF
   NEXT i

END SUB

' ================================ OutputModi3D ================================
'
'  Stampa un singolo modo per lo schema spaziale
'
'  Variabili di ingresso:
'     PSZ       numero di impalcati
'     j         modo richiesto
'     EigVal()  autovalori, disposti in ordine crescente
'     EigVet()  matrice che contiene gli autovettori, disposti in colonne
'     T()       periodo dei singoli modi
'     Beta()    ordinata dello spettro di risposta
'     Part()    coefficienti di partecipazione modale
'     EnIso     energia isolatori
'     EnTot     energia totale
'     Smorz     smorzamento
'
' ------------------------------------------------------------------------------
SUB OutputModi3D (SM, PSZ, j, EigVal(), EigVet(), T(), Beta(), Part(), EnIso, EnTot, Smorz)

   PRINT #2, : PRINT #2, : PRINT #2,
   PRINT #2, "ANALISI MODALE - modo"; j
   PRINT #2,
   PRINT #2, USING "T =##.#### s          "; T(j);
   PRINT #2, USING "omega2 =#####.####         "; EigVal(j);
   PRINT #2, USING "p =####.####"; Part(j)
   PRINT #2,
   IF SM = 0 THEN F$ = "kNm     " ELSE F$ = "tm      "
   IF EnTot <> 0 THEN
      PRINT #2, USING "energia - isol. ##.####^^^^ " + F$; EnIso;
      PRINT #2, USING "tot. ##.####^^^^ " + F$; EnTot;
      PRINT #2, USING "rapporto ##.####"; EnIso / EnTot
      PRINT #2,
   END IF
   IF Smorz <> 0 THEN PRINT #2, USING "smorzamento   ##.### %              "; Smorz;
   PRINT #2, USING "ordinata spettrale  beta =##.####       "; Beta(j)
   PRINT #2,
   PRINT #2, "impalcato                deformata modale"
   PRINT #2, "                   Vx            Vy          rotaz."
   PRINT #2,
   FOR P = PSZ TO 1 STEP -1
      PRINT #2, USING "   ##          ###.######    ###.######    ###.######"; P; EigVet(P, j); EigVet(P + PSZ, j); EigVet(P + 2 * PSZ, j)
   NEXT P

END SUB

' ================================ OutputSpettro ===============================
'   Procedura per la stampa dello spettro
' ------------------------------------------------------------------------------
SUB OutputSpettro (SpePar(), SpeT(), SpeR())

   PRINT #2, : PRINT #2, : PRINT #2,
   SELECT CASE SpePar(1)
      CASE 1
         PRINT #2, "SPETTRO DI RISPOSTA DEFINITO CON COPPIE T-R": PRINT #2,
         PRINT #2, "  T (s)   R (a meno di g)"
         PRINT #2,
         FOR Cop = 1 TO SpePar(2)
            PRINT #2, USING "###.###     ##.####"; SpeT(Cop); SpeR(Cop)
         NEXT Cop
      CASE 2
         PRINT #2, "SPETTRO TIPO EC8 GENERALIZZATO": PRINT #2,
         PRINT #2, USING "alfa =##.### g           q =##.##"; SpePar(2); SpePar(3)
         PRINT #2,
         PRINT #2, USING "S =##.###         beta0 =##.##"; SpePar(4); SpePar(5)
         PRINT #2,
         PRINT #2, USING "TB =##.## s       TC =##.## s       TD =##.## s"; SpePar(6); SpePar(7); SpePar(8)
         PRINT #2,
         PRINT #2, USING "k1 =##.######     k2 =##.######     Rmin =##.###"; SpePar(9); SpePar(10); SpePar(11)
         PRINT #2,
         PRINT #2, USING "smorzamento del telaio =###.## %"; SpePar(12)
      CASE 3
         PRINT #2, "SPETTRO DI PROGETTO DELL'EC8 (1993)": PRINT #2,
         PRINT #2, USING "suolo !           alfa =##.### g           q =##.##"; CHR$(64 + SpePar(12)); SpePar(2); SpePar(3)
         PRINT #2,
         PRINT #2, USING "S =##.###         beta0 =##.##"; SpePar(4); SpePar(5)
         PRINT #2,
         PRINT #2, USING "TB =##.## s       TC =##.## s       TD =##.## s"; SpePar(6); SpePar(7); SpePar(8)
         PRINT #2,
         PRINT #2, USING "kd1 =##.######    kd2 =##.######    Rmin =##.###"; SpePar(9); SpePar(10); SpePar(11)
      CASE 4
         PRINT #2, "SPETTRO DI PROGETTO DELLA NORMA ITALIANA (DM 86)": PRINT #2,
         PRINT #2, USING "grado di sismicita'  S = ##"; SpePar(2)
         PRINT #2,
         PRINT #2, USING "epsilon =##.##     beta =##.##     I =##.##"; SpePar(3); SpePar(4); SpePar(5)
   END SELECT

END SUB

' ================================ Risposta ====================================
'
'  Fornisce il coefficiente di risposta corrispondente a un dato periodo
'  per uno spettro di progetto riconducibile a quello dell'EC8 (1993)
'
'  Variabili di ingresso:
'     T         periodo della struttura
'     SpePar()  parametri che definiscono lo spettro
'
'  Variabili di uscita:
'     Risposta  espresso a meno dell'accelerazione di gravit
'
' ------------------------------------------------------------------------------
FUNCTION Risposta (T, SpePar())

   SELECT CASE SpePar(1)
      CASE 2, 3
         Alfa = SpePar(2)
         q = SpePar(3)
         S = SpePar(4)
         Beta0 = SpePar(5)
         IF SpePar(1) = 2 THEN
            Smorz = SpePar(12)
            Eta = SQR(7 / (2 + Smorz)): IF Eta < .7 THEN Eta = .7
         ELSE
            Eta = 1
         END IF
         TB = SpePar(6)
         TC = SpePar(7)
         TD = SpePar(8)
         k1 = SpePar(9)
         k2 = SpePar(10)
         Rmin = SpePar(11)
      CASE 4
         Alfa = (SpePar(2) - 2) / 100
         q = 1:      S = 1
         Beta0 = SpePar(3) * SpePar(4) * SpePar(5)
         Eta = 1:    TB = 0:     TC = .8:    TD = 100
         k1 = .6666667:          k2 = 1:     Rmin = 0
   END SELECT
   IF T < TB THEN
      Beta = Alfa * S * (1 + T / TB * (Eta * Beta0 / q - 1))
   ELSEIF T < TC THEN
      Beta = Alfa * Eta * S * Beta0 / q
   ELSEIF T < TD THEN
      Beta = Alfa * Eta * S * Beta0 / q * (TC / T) ^ k1
      IF Beta < Rmin * Alfa THEN Beta = Rmin * Alfa
   ELSE
      Beta = Alfa * Eta * S * Beta0 / q * (TC / TD) ^ k1 * (TD / T) ^ k2
      IF Beta < Rmin * Alfa THEN Beta = Rmin * Alfa
   END IF
   Risposta = Beta

END FUNCTION

' ================================ Risposta2 ===================================
'
'  Fornisce il coefficiente di risposta corrispondente a un dato periodo
'  per uno spettro dato per punti
'
'  Variabili di ingresso:
'     T         periodo della struttura
'     nCop      numero di coppie T-R
'     SpeT()    valori di T
'     SpeR()    valori di R a meno dell'accelerazione di gravit
'
'  Variabili di uscita:
'     Risposta2 espresso a meno dell'accelerazione di gravit
'
' ------------------------------------------------------------------------------
FUNCTION Risposta2 (T, nCop, SpeT(), SpeR())

   Trovato = 0
   FOR Cop = 2 TO nCop
      IF T < SpeT(Cop) THEN
         T1 = SpeT(Cop - 1)
         R1 = SpeR(Cop - 1)
         T2 = SpeT(Cop)
         R2 = SpeR(Cop)
         Trovato = -1
         EXIT FOR
      END IF
   NEXT Cop
   IF Trovato THEN
      Risposta2 = R1 + (R2 - R1) * (T - T1) / (T2 - T1)
   ELSE
      Risposta2 = SpeR(nCop)
   END IF

END FUNCTION

