'                                  PROCOM.BAS
'
'  procedure comuni ai programmi TELGEN, TELRET, TELSPA
'
'  ver. 2.1                                                     rev. 02.99

'  Attenzione: cambiate variabili di scambio nelle procedure
'     CarSoll, DisegnaSchema, FormaMat, Incognite, TermNoto


' ------------------ Dichiarazione delle procedure utilizzate ------------------

' procedure del file ASTA.BAS
DECLARE SUB Asta.Inc (X1!, X2!, Y1!, Y2!, E!, A!, I!, L!, SnAl!, CsAl!, Ld!(), Kg!())
DECLARE SUB Geometria (X1!, X2!, Y1!, Y2!, L!, SnAl!, CsAl!, Ld!())
DECLARE SUB Kg.Cer (ES!, E!, A!, I!, L!, Ld!(), Kg!())
DECLARE SUB Kg.CerTag (ES!, E!, G!, CHI!, A!, I!, L!, Ld!(), Kg!())
DECLARE SUB Kg.IncTag (E!, G!, CHI!, A!, I!, L!, Ld!(), Kg!())
DECLARE SUB Kg.Rig (E!, A!, I!, L!, S!, T!, Ld!(), Kg!())
DECLARE SUB Kg.RigTag (E!, G!, CHI!, A!, I!, L!, S!, T!, Ld!(), Kg!())
DECLARE SUB Kg.Wnk (K!, B!, E!, A!, I!, L!, Ld!(), Kg!())
DECLARE SUB Kg.WnkRig (K!, B!, E!, A!, I!, L!, S!, T!, Ld!(), Kg!())
DECLARE SUB Sl.Cer.Qunif (ES!, Q!, N!, L!, Sl!())
DECLARE SUB Sl.Inc.Qunif (Q!, N!, L!, Sl!())
DECLARE SUB Sl.Rig.Qunif (Q!, N!, L!, S!, T!, Sl!())
DECLARE SUB Sl.Wnk.Qunif (Q!, N!, L!, K!, B!, Kloc!(), Sl!())

' procedure del file MATRIX.BAS
DECLARE SUB AzzeraMat (A!())
DECLARE SUB ProdottoMat (A!(), B!(), C!())
DECLARE SUB SommaMat (A!(), B!(), C!())
DECLARE SUB TrasponeMat (A!(), B!())

' procedure di questo stesso file
DECLARE SUB Aspetta (Sec!)
DECLARE SUB AzIncLocale (PX!, PY!, TA$, B!, K!, TrRigSin!, TrRigDes!, Lun!, SnAl!, CsAl!, Ld!(), LdT!(), Kg!(), S!())
DECLARE SUB Inserimento (P!, Q!, V!, NL!, SB!, KTT!(), KTA!(), KAA!())
DECLARE SUB LCOPY (SCR!, DENS!)
DECLARE SUB MatAstaGlobale (TA$, DefTag!, X1!, X2!, Y1!, Y2!, A!, AT!, Iner!, K!, B!, TrRigSin!, TrRigDes!, E!, G!, Lun!, SnAl!, CsAl!, Ld!(), Kg!())
DECLARE SUB ModGraf (Scheda!)
DECLARE SUB NomeFile (Tipo$, File$)
DECLARE SUB Scala (X1!, Y1!, X2!, Y2!)


DIM SHARED Scheda1

ModGrafErr:              ' individuazione della modalit grafica
SELECT CASE Scheda1
   CASE 12
      Scheda1 = 11       ' MCGA
   CASE 11
      Scheda1 = 8        ' EGA 64k
   CASE 8
      Scheda1 = 2        ' CGA
   CASE 2
      Scheda1 = 3        ' Hercules
   CASE ELSE
      Scheda1 = 0        ' scheda grafica non presente o non riconosciuta
END SELECT
RESUME

' ================================ Aspetta =====================================
'   Procedura che lascia il programma in attesa per un assegnato tempo
'   (in secondi)
'   N.B. Esce comunque se scatta la mezzanotte
' ------------------------------------------------------------------------------
SUB Aspetta (Sec)

   T = TIMER
   DO UNTIL (TIMER - T > Sec) OR (TIMER - T < 0)
   LOOP

END SUB

' ================================ AzIncLocale =================================
'   Procedura che determina il vettore di incastro perfetto S()
'   di un'asta nel sistema di riferimento locale
'   NOTA: per asta tipo W o WR occorre avere gi definito Ld(), LdT() e Kg()
' ------------------------------------------------------------------------------
SUB AzIncLocale (PX, PY, TA$, B, K, TrRigSin, TrRigDes, Lun, SnAl, CsAl, Ld(), LdT(), Kg(), S())
  
   DIM Kl(6, 6)
     
   PN = PX * CsAl + PY * SnAl
   PQ = PY * CsAl - PX * SnAl
   SELECT CASE TA$
      CASE "T"
         CALL Sl.Inc.Qunif(PQ, PN, Lun, S())
      CASE "W", "WR"
         CALL ProdottoMat(LdT(), Kg(), Kl())
         CALL ProdottoMat(Kl(), Ld(), Kl())
         CALL Sl.Wnk.Qunif(PQ, PN, Lun, K, B, Kl(), S())
      CASE "C1"
         CALL Sl.Cer.Qunif(1, PQ, PN, Lun, S())
      CASE "C2"
         CALL Sl.Cer.Qunif(2, PQ, PN, Lun, S())
      CASE "P"
         CALL Sl.Cer.Qunif(3, PQ, PN, Lun, S())
      CASE "TR"
         TrRigS = TrRigSin / Lun
         TrRigT = TrRigDes / Lun
         CALL Sl.Rig.Qunif(PQ, PN, Lun, TrRigS, TrRigT, S())
   END SELECT

END SUB

' ================================ CarSoll =====================================
'   Procedura per la valutazione delle caratteristiche di sollecitazione
'   nelle aste
'
'   N.B. Nel vettore F() vengono aggiunti ai carichi nodali esterni
'        le azioni trasmesse ai nodi dalle aste
' ------------------------------------------------------------------------------
SUB CarSoll (DefTag, TrRig, JZ, X(), Y(), ZA(), ZAT(), ZI(), Easta(), Gasta(), TA$(), Estr(), NT(), K(), B(), TrRig(), N(), P(), F(), R(), N1(), T1(), M1(), N2(), T2(), M2())

   DIM Ld(6, 6), LdT(6, 6), Kg(6, 6), S(6, 1), Sl(6, 1), Sg(6, 1), S1(6, 1)

   FOR J = 1 TO JZ
      IF TA$(J) <> "Q8" THEN
         L = NT(J)
         IF TrRig <> 0 THEN
            TrRigSin = TrRig(J, 1)
            TrRigDes = TrRig(J, 2)
         END IF
         E1 = Estr(J, 1)
         E2 = Estr(J, 2)
         CALL MatAstaGlobale(TA$(J), DefTag, X(E1), X(E2), Y(E1), Y(E2), ZA(L), ZAT(L), ZI(L), K(J) * 1000, B(J), TrRigSin, TrRigDes, Easta(J), Gasta(J), Lun, SnAl, CsAl, Ld(), Kg())
         FOR K = 1 TO 6
            IF K <= 3 THEN N = N(E1, K) ELSE N = N(E2, K - 3)
            IF N = 0 THEN S1(K, 1) = 0 ELSE S1(K, 1) = R(N)
         NEXT K
         CALL ProdottoMat(Kg(), S1(), Sg())
         CALL TrasponeMat(Ld(), LdT())
         CALL ProdottoMat(LdT(), Sg(), Sl())
         IF P(J, 1) <> 0 OR P(J, 2) <> 0 THEN
            CALL AzIncLocale(P(J, 1), P(J, 2), TA$(J), B(J), K(J) * 1000, TrRigSin, TrRigDes, Lun, SnAl, CsAl, Ld(), LdT(), Kg(), S())
            CALL SommaMat(S(), Sl(), Sl())
         END IF
         CALL ProdottoMat(Ld(), Sl(), Sg())
         FOR K = 1 TO 3
            F(E1, K) = F(E1, K) - Sg(K, 1)
            F(E2, K) = F(E2, K) - Sg(K + 3, 1)
         NEXT K
         N1(J) = -Sl(1, 1)
         T1(J) = Sl(2, 1)
         M1(J) = -Sl(3, 1)
         N2(J) = Sl(4, 1)
         T2(J) = -Sl(5, 1)
         M2(J) = Sl(6, 1)
      END IF
      '
      LOCATE CSRLIN - 1, 44
      PRINT USING "eseguito per il ### %"; J / JZ * 100
   NEXT J

END SUB

' ================================ DefinisceIO =================================
'   Procedura per la definizione delle unit di ingresso ed uscita
' ------------------------------------------------------------------------------
SUB DefinisceIO (Prog$, FI$, FO$, FT$)

   ' file di input
   CALL NomeFile("/INP=", FI$)
   ' unit di output
   CALL NomeFile("/OUT=", FO$)
   ' drive e directory per file temporanei
   IF Prog$ = "SPA" THEN CALL NomeFile("/TEM=", FT$)
  
   PRINT "       Nome del file che contiene i dati:        "
   LOCATE CSRLIN - 1, 50
   IF FI$ = "" THEN
      INPUT "", FI$
      FI$ = UCASE$(FI$)
      LOCATE CSRLIN - 1, 50: PRINT FI$
   ELSE
      PRINT FI$
   END IF
   PRINT
   PRINT "       Unit di uscita:"
   PRINT "       (CON = schermo, PRN = stampante, oppure nome di un file)"
   LOCATE CSRLIN - 2, 50
   IF FO$ = "" THEN
      INPUT "", FO$
      FO$ = UCASE$(FO$)
      IF FO$ = "" THEN
         PosPunto = INSTR(FI$, ".")
         IF PosPunto = 0 THEN PosPunto = LEN(FI$) + 1
         FO$ = UCASE$(LEFT$(FI$, PosPunto - 1)) + ".OUT"
      END IF
      LOCATE CSRLIN - 1, 50: PRINT FO$
   ELSE
      PRINT FO$
   END IF
   PRINT
   PRINT
   IF Prog$ = "SPA" THEN
      PRINT "       Drive e directory per file temporanei:    "
      LOCATE CSRLIN - 1, 50
      IF FT$ = "" THEN
         INPUT "", FT$
         FT$ = UCASE$(FT$)
         LOCATE CSRLIN - 1, 50: PRINT FT$
      ELSE
         PRINT FT$
      END IF
      PRINT
      IF FT$ <> "" AND RIGHT$(FT$, 1) <> "\" THEN FT$ = FT$ + "\"
   END IF

END SUB

' ================================ DefinisceMO =================================
'   Procedura per la definizione delle modalit operative:
'      - trascurabilit o meno della deformabilit estensionale
'      - trascurabilit o meno della deformabilit a taglio
'      - stampa dei dati man mano che vengono letti
' ------------------------------------------------------------------------------
SUB DefinisceMO (Prog$, DefEst, DefTag, Contr)

   C$ = UCASE$(COMMAND$)
   ' deformabilit estensionale
   IF INSTR(C$, "/DE") <> 0 THEN
      DefEst = 1
   ELSEIF INSTR(C$, "/NDE") <> 0 THEN
      DefEst = 0
   ELSE
      DefEst = -1
   END IF
   ' deformabilit a taglio
   IF INSTR(C$, "/DT") <> 0 THEN
      DefTag = 1
   ELSEIF INSTR(C$, "/NDT") <> 0 THEN
      DefTag = 0
   ELSE
      DefTag = -1
   END IF
   ' controllo dati
   IF INSTR(C$, "/C") <> 0 THEN
      Contr = 1
   ELSEIF INSTR(C$, "/NC") <> 0 THEN
      Contr = 0
   ELSE
      Contr = -1
   END IF
  
   IF Prog$ = "ISO" OR Prog$ = "RET" OR Prog$ = "SPA" THEN
      PRINT "       Deformabilit estensionale dei pilastri (0=no, 1=si): "
      LOCATE CSRLIN - 1, 63
      IF DefEst = -1 THEN
         INPUT "", DefEst
         LOCATE CSRLIN - 1, 63: PRINT LTRIM$(STR$(DefEst))
      ELSE
         PRINT LTRIM$(STR$(DefEst))
      END IF
      PRINT
   ELSE
      DefEst = 1
   END IF
   PRINT "       Deformabilit a taglio delle aste (0=no, 1=si):       "
   LOCATE CSRLIN - 1, 63
   IF DefTag = -1 THEN
      INPUT "", DefTag
      LOCATE CSRLIN - 1, 63: PRINT LTRIM$(STR$(DefTag))
   ELSE
      PRINT LTRIM$(STR$(DefTag))
   END IF
   PRINT
   PRINT "       Controllo dei dati man mano che vengono letti (0=no, 1=si): "
   LOCATE CSRLIN - 1, 68
   IF Contr = -1 THEN
      INPUT "", Contr
      LOCATE CSRLIN - 1, 68: PRINT LTRIM$(STR$(Contr))
   ELSE
      PRINT LTRIM$(STR$(Contr))
   END IF

END SUB

' ============================== Definisce SM ==================================
'   Procedura per la definizione del Sistema di Misura
'   0 = Sistema Internazionale
'   <>  Sistema Tecnico
' ------------------------------------------------------------------------------
SUB DefinisceSM (SM)
 
   C$ = COMMAND$
   SM = INSTR(C$, "/ST")

END SUB

' ================================ DisegnaSchema ===============================
'   Procedura per il disegno dello schema del telaio
'      (vedi: A.Ghersi, Introduzione alla programmazione, CUEN, Napoli, 1987)
' ------------------------------------------------------------------------------
SUB DisegnaSchema (IZ, JZ, X(), Y(), TA$(), Estr())
 
   CALL ModGraf(Scheda)
   IF Scheda <> 0 THEN
      ' adatta la scheda per consentire la copia su stampante
      IF Scheda > 3 AND FO$ = "PRN" THEN
         Scheda = 2
         SCREEN Scheda
      END IF
      ' determina i valori limite di X ed Y
      X1 = X(1)
      X2 = X(1)
      Y1 = Y(1)
      Y2 = Y(1)
      FOR I = 2 TO IZ
         IF X1 > X(I) THEN X1 = X(I)
         IF X2 < X(I) THEN X2 = X(I)
         IF Y1 > Y(I) THEN Y1 = Y(I)
         IF Y2 < Y(I) THEN Y2 = Y(I)
      NEXT I
      ' individua la scala del disegno
      CALL Scala(X1, Y1, X2, Y2)
      WINDOW (X1, Y1)-(X2, Y2)
      ' disegna lo schema
      FOR J = 1 TO JZ
         IF TA$(J) = "Q8" THEN NumNodi = 8 ELSE NumNodi = 2
         FOR nodo = 1 TO NumNodi - 1
            E1 = Estr(J, nodo)
            E2 = Estr(J, nodo + 1)
            LINE (X(E1), Y(E1))-(X(E2), Y(E2))
         NEXT nodo
         IF NumNodi = 8 THEN
            E1 = Estr(J, 8)
            E2 = Estr(J, 1)
            LINE (X(E1), Y(E1))-(X(E2), Y(E2))
         END IF
      NEXT J
      ' effettua la copia su stampante
      IF FO$ = "PRN" AND Scheda = 2 THEN
         CALL LCOPY(Scheda, 1)
      ELSE
         CALL Aspetta(2)
      END IF
      ' torna alla modalita' testo
      SCREEN 0
   END IF

END SUB

' ================================ FormaMat ====================================
'   Procedura per la valutazione del contributo di un'asta alla matrice di
'   rigidezza della struttura
' ------------------------------------------------------------------------------
SUB FormaMat (NumNodi, NumEstr(), N(), Kg(), NL, SB, KTT(), KTA(), KAA())

   NumCompMovim = 3 * NumNodi
   DIM G(1 TO NumCompMovim)

   ' prepara il vettore di corrispondenza per l'asta
   FOR nodo = 1 TO NumNodi
      FOR K = 1 TO 3
         G(3 * (nodo - 1) + K) = N(NumEstr(nodo), K)
      NEXT K
   NEXT nodo
   ' inserisce i termini nella matrice di rigidezza della struttura
   FOR R = 1 TO NumCompMovim
      P = G(R)
      IF P <> 0 THEN
         FOR C = 1 TO NumCompMovim
            Q = G(C)
            IF Q <> 0 THEN
               CALL Inserimento(P, Q, Kg(R, C), NL, SB, KTT(), KTA(), KAA())
            END IF
         NEXT C
      END IF
   NEXT R

END SUB

' ================================ Incognite ===================================
'   Procedura per la numerazione delle incognite, valutazione dell'ampiezza
'   della semibanda, ridimensionamento della matrice di rigidezza
' ------------------------------------------------------------------------------
SUB Incognite (IZ, JZ, KZ, TA$(), Estr(), V$(), NI(), N(), NL, NZ, SB)
 
   GOSUB Numera
   GOSUB Ampiezza
   EXIT SUB

' numerazione automatica delle incognite
Numera:
   ' inizializzazione N(,)
   FOR I = 1 TO IZ
      FOR K = 1 TO 3
         N(I, K) = -1
      NEXT K
   NEXT I
   ' prima fase - spostamenti impalcati
   N = 0
   IF KZ <> 0 THEN
      I1 = 1
      FOR K = 1 TO KZ
         VN = N + 1
         I = I1
         DO WHILE NI(I) <> 0
            IF MID$(V$(NI(I)), 1, 1) = "1" THEN VN = 0
            I = I + 1
         LOOP
         I = I1
         DO WHILE NI(I) <> 0
            N(NI(I), 1) = VN
            I = I + 1
         LOOP
         IF VN <> 0 THEN N = N + 1
         I1 = I + 1
      NEXT K
   END IF
   NL = N
   ' seconda fase - altre incognite
   FOR I = 1 TO IZ
      FOR K = 1 TO 3
         IF N(I, K) = -1 THEN
            IF MID$(V$(I), K, 1) = "1" THEN
               N(I, K) = 0
            ELSE
               N = N + 1
               N(I, K) = N
            END IF
         END IF
      NEXT K
   NEXT I
   NZ = N
RETURN

' calcolo dell'ampiezza della semibanda
Ampiezza:
   SB = 0
   FOR J = 1 TO JZ
      N1 = NZ
      N2 = NL
      IF TA$(J) = "Q8" THEN NumNodi = 8 ELSE NumNodi = 2
      FOR K = 1 TO 3
         FOR nodo = 1 TO NumNodi
            N = N(Estr(J, nodo), K)
            IF N > NL THEN
               IF N1 > N THEN N1 = N
               IF N2 < N THEN N2 = N
            END IF
         NEXT nodo
      NEXT K
      IF N2 > N1 AND SB < N2 - N1 THEN SB = N2 - N1
   NEXT J
   SB = SB + 1
RETURN

END SUB

' ================================ InputIntestaz ===============================
'   Procedura per la lettura della intestazione
' ------------------------------------------------------------------------------
SUB InputIntestaz (Contr, Intest$)

   INPUT #1, Intest$
   IF Contr = 1 THEN
      PRINT #2, "intestazione: "; CHR$(34); Intest$; CHR$(34)
   END IF

END SUB

' ================================ InputModElast ===============================
'   Procedura per la lettura del modulo di elasticit normale ed il calcolo
'   del modulo di elasticit tangenziale
'
'   N.B. si assume G = 0.45xE, con riferimento al calcestruzzo
' ------------------------------------------------------------------------------
SUB InputModElast (Contr, SM, EL, GL, E, G)

   INPUT #1, EL
   IF Contr = 1 THEN PRINT #2, "modulo elastico E ="; EL
   GL = .45 * EL
   IF SM = 0 THEN
      E = EL * 1000
      G = GL * 1000
   ELSE
      E = EL * 10
      G = GL * 10
   END IF

END SUB

' ================================ InputSezioni ================================
'   Procedura per la lettura dei dati delle sezioni tipo
' ------------------------------------------------------------------------------
SUB InputSezioni (Contr, LZ, FS$(), ZB(), ZH(), ZA(), ZI(), ZAT(), ER)

   ' sezioni tipo
   FOR L = 1 TO LZ
      INPUT #1, FS$(L)
      FS$(L) = UCASE$(FS$(L))
      IF Contr = 1 THEN
         PRINT #2, "tipo di sezione"; L
         PRINT #2, "   forma della sezione "; CHR$(34); FS$(L); CHR$(34)
      END IF
      ' legge dati sezioni
      IF FS$(L) = "R" THEN
         INPUT #1, ZB(L), ZH(L)
         IF Contr = 1 THEN
            PRINT #2, "   sezione rettangolare     B ="; ZB(L); "   H ="; ZH(L)
         END IF
         ZA(L) = ZB(L) * ZH(L)
         ZI(L) = ZB(L) * ZH(L) ^ 3 / 12
         ZAT(L) = ZA(L) / 1.2
      ELSEIF FS$(L) = "G" THEN
         INPUT #1, ZA(L), ZAT(L), ZI(L)
         IF Contr = 1 THEN
            PRINT #2, "   sezione generica     A ="; ZA(L);
            PRINT #2, "   A/Chi ="; ZAT(L); "   I ="; ZI(L)
         END IF
      ELSE
         ER = 1
         PRINT "Il codice che identifica la forma della sezione"; L;
         PRINT " non  ammissibile"
      END IF
   NEXT L

END SUB

' ================================ Inserimento =================================
'
'   Procedura per l'inserimento di un termine
'   nella matrice di rigidezza globale della struttura
'
'   Variabili di ingresso:
'      P         primo indice
'      Q         secondo indice
'      V         valore da inserire
'      NL        ordine della matrice KTT()
'      SB        ampiezza della semibanda della matrice KAA()
'      KTT()     sottomatrice Ktt della matrice di rigidezza della struttura
'      KTA()     sottomatrice Kta della matrice di rigidezza della struttura
'      KAA()     sottomatrice Kaa della matrice di rigidezza della struttura
'
'   Variabili di uscita:
'      KTT()     sottomatrice Ktt della matrice di rigidezza della struttura
'      KTA()     sottomatrice Kta della matrice di rigidezza della struttura
'      KAA()     sottomatrice Kaa della matrice di rigidezza della struttura
'
' ------------------------------------------------------------------------------
SUB Inserimento (P, Q, V, NL, SB, KTT(), KTA(), KAA())

   IF P > Q THEN
      ' non memorizza il valore, perch situato nel triangolo inferiore
   ELSEIF P <= NL AND Q <= NL THEN
      KTT(P, Q) = KTT(P, Q) + V
   ELSEIF P <= NL AND Q > NL THEN
      KTA(P, Q - NL) = KTA(P, Q - NL) + V
   ELSEIF P > NL AND Q > NL AND Q - P < SB THEN
      KAA(P - NL, Q - P + 1) = KAA(P - NL, Q - P + 1) + V
   END IF

END SUB

' ================================ LCOPY =======================================
'
'   Effettua la copia dello schermo, in modalit grafica, sulla stampante
'
'   Variabili di ingresso:
'      SCR       modalit grafica corrente:
'                   2 = CGA
'      DENS      densit di stampa:
'                   1 = densit singola (pi veloce, meno precisa)
'                   2 = densit doppia (meno veloce, pi precisa)
'
' ------------------------------------------------------------------------------
SUB LCOPY (SCR, DENS)

   SELECT CASE SCR
      CASE 2
         DEF SEG = &HB800
         LPRINT CHR$(27); CHR$(51); CHR$(24);
         WIDTH "LPT1:", 255
         IF DENS = 2 THEN
            C$ = CHR$(27) + CHR$(76) + CHR$(32) + CHR$(3)
         ELSE
            C$ = CHR$(27) + CHR$(75) + CHR$(144) + CHR$(1)
         END IF
         FOR M = 0 TO 79
            LPRINT C$;
            FOR L = 7920 TO 0 STEP -80
               P = M + L
               P1$ = CHR$(PEEK(P + 8192))
               P2$ = CHR$(PEEK(P))
               IF DENS = 2 THEN
                  LPRINT P1$; P1$; P1$; P1$; P2$; P2$; P2$; P2$;
               ELSE
                  LPRINT P1$; P1$; P2$; P2$;
               END IF
            NEXT L
            LPRINT
         NEXT M
         WIDTH "LPT1:", 80
         LPRINT CHR$(27); CHR$(51); CHR$(36);
      CASE ELSE
         ' attualmente non previsto
   END SELECT

END SUB

' ================================ MatAstaGlobale ==============================
'   Procedura che determina geometria e matrice di rigidezza globale di un'asta
' ------------------------------------------------------------------------------
SUB MatAstaGlobale (TA$, DefTag, X1, X2, Y1, Y2, A, AT, Iner, K, B, TrRigSin, TrRigDes, E, G, Lun, SnAl, CsAl, Ld(), Kg())

   SELECT CASE TA$
      CASE "T"
         IF DefTag = 1 THEN
            CALL Geometria(X1, X2, Y1, Y2, Lun, SnAl, CsAl, Ld())
            CHI = A / AT
            CALL Kg.IncTag(E, G, CHI, A, Iner, Lun, Ld(), Kg())
         ELSE
            CALL Asta.Inc(X1, X2, Y1, Y2, E, A, Iner, Lun, SnAl, CsAl, Ld(), Kg())
         END IF
      CASE "W"
         CALL Geometria(X1, X2, Y1, Y2, Lun, SnAl, CsAl, Ld())
         CALL Kg.Wnk(K, B, E, A, Iner, Lun, Ld(), Kg())
      CASE "C1", "C2", "P"
         IF TA$ = "C1" THEN
            ES = 1
         ELSEIF TA$ = "C2" THEN
            ES = 2
         ELSE
            ES = 3
         END IF
         CALL Geometria(X1, X2, Y1, Y2, Lun, SnAl, CsAl, Ld())
         IF DefTag = 1 THEN
            CHI = A / AT
            CALL Kg.CerTag(ES, E, G, CHI, A, Iner, Lun, Ld(), Kg())
         ELSE
            CALL Kg.Cer(ES, E, A, Iner, Lun, Ld(), Kg())
         END IF
      CASE "TR"
         CALL Geometria(X1, X2, Y1, Y2, Lun, SnAl, CsAl, Ld())
         TrRigS = TrRigSin / Lun
         TrRigT = TrRigDes / Lun
         IF DefTag = 1 THEN
            CHI = A / AT
            CALL Kg.RigTag(E, G, CHI, A, Iner, Lun, TrRigS, TrRigT, Ld(), Kg())
         ELSE
            CALL Kg.Rig(E, A, Iner, Lun, TrRigS, TrRigT, Ld(), Kg())
         END IF
      CASE "WR"
         CALL Geometria(X1, X2, Y1, Y2, Lun, SnAl, CsAl, Ld())
         TrRigS = TrRigSin / Lun
         TrRigT = TrRigDes / Lun
         CALL Kg.WnkRig(K, B, E, A, Iner, Lun, TrRigS, TrRigT, Ld(), Kg())
   END SELECT

END SUB

'================================= ModGraf =====================================
'
'   Definisce la migliore modalit grafica possibile
'
'   Variabili di uscita:
'      Scheda    migliore modalit grafica possibile; si prova nell'ordine:
'                   12 = VGA
'                   11 = MCGA
'                    8 = EGA
'                    2 = CGA
'                    3 = Hercules
'                    0 = possibile solo modalit testo
'
' ------------------------------------------------------------------------------
SUB ModGraf (Scheda)

   Scheda1 = 12          ' VGA
   ON ERROR GOTO ModGrafErr
   SCREEN Scheda1
   ON ERROR GOTO 0
   Scheda = Scheda1

END SUB

' ================================ NomeFile ====================================
'  Legge un nome di file dal COMMAND$, basandosi sulla stringa di tipo fornita
'  (ad esempio "/INP=")
' ------------------------------------------------------------------------------
SUB NomeFile (Tipo$, File$)

   C$ = UCASE$(COMMAND$)
   VL = LEN(C$)
   V1 = INSTR(C$, Tipo$)
   IF V1 = 0 THEN
      File$ = ""
   ELSE
      LT = LEN(Tipo$)
      V2 = INSTR(V1 + LT, C$, " "): IF V2 = 0 THEN V2 = VL + 1
      V3 = INSTR(V1 + LT, C$, "/"): IF V3 = 0 THEN V3 = VL + 1
      IF V2 > V3 THEN V2 = V3
      File$ = UCASE$(MID$(C$, V1 + LT, V2 - V1 - LT))
   END IF

END SUB

' ================================ OutputIntestaz ==============================
'   Procedura per la stampa dell'intestazione
' ------------------------------------------------------------------------------
SUB OutputIntestaz (NomeProg$, FI$, Intest$)

   PRINT #2, "Programma: "; NomeProg$;
   PRINT #2, "         data: "; MID$(DATE$, 4, 2) + "/" + MID$(DATE$, 1, 2);
   PRINT #2, "/" + RIGHT$(DATE$, 4); "     ora: "; LEFT$(TIME$, 5)
   PRINT #2, "File dati: "; FI$
   PRINT #2, : PRINT #2,
   PRINT #2, Intest$
   PRINT #2, : PRINT #2, : PRINT #2,

END SUB

' ================================ OutputModElast ==============================
'   Procedura per la stampa dei moduli di elasticit
' ------------------------------------------------------------------------------
SUB OutputModElast (SM, DefTag, EL, GL)

   PRINT #2, : PRINT #2, : PRINT #2,
   IF SM = 0 THEN
      PRINT #2, "Modulo di elasticita' normale =     "; EL; " N/mm2"
   ELSE
      PRINT #2, "Modulo di elasticita' normale =     "; EL; " kg/cm2"
   END IF
   IF DefTag = 1 THEN
      IF SM = 0 THEN
         PRINT #2, "Modulo di elasticita' tangenziale = "; GL; " N/mm2"
      ELSE
         PRINT #2, "Modulo di elasticita' tangenziale = "; GL; " kg/cm2"
      END IF
   END IF
   
END SUB

' ================================ OutputSpoOriz ===============================
'  Stampa gli spostamenti orizzontali dei traversi
' ------------------------------------------------------------------------------
SUB OutputSpoOriz (KZ, SpoAssTel(), SpoRelTel())

   PRINT #2, : PRINT #2, : PRINT #2,
   PRINT #2, "SPOSTAMENTO ORIZZONTALE DEI TRAVERSI": PRINT #2,
   PRINT #2, " traverso      spostamento      spostamento"
   PRINT #2, "              assoluto (mm)    relativo (mm)"
   PRINT #2,
   FOR K = KZ TO 1 STEP -1
      PRINT #2, USING "   ##          ####.###         ####.###"; K; SpoAssTel(K) * 1000; SpoRelTel(K) * 1000
   NEXT K

END SUB

' ================================ Scala =======================================
'   Procedura per la definizione della scala per rappresentazione monometrica
'      (vedi: A.Ghersi, Introduzione alla programmazione, CUEN, Napoli, 1987)
' ------------------------------------------------------------------------------
SUB Scala (X1, Y1, X2, Y2)

   FX = .125
   FY = .075
   SC = (X2 - X1) / FX
   IF SC < (Y2 - Y1) / FY THEN SC = (Y2 - Y1) / FY
   LX = FX * SC
   LY = FY * SC
   XC = (X1 + X2) / 2
   YC = (Y1 + Y2) / 2
   X1 = XC - LX / 2
   X2 = XC + LX / 2
   Y1 = YC - LY / 2
   Y2 = YC + LY / 2

END SUB

' ================================ TermNoto ====================================
'   Procedura per la valutazione del vettore termine noto R(),
'   come somma dei carichi nodali esterni e delle azioni di incastro
'   trasmesse ai nodi dalle aste
' ------------------------------------------------------------------------------
SUB TermNoto (TrRig, IZ, JZ, X(), Y(), ZA(), ZI(), E, TA$(), Estr(), NT(), K(), B(), TrRig(), N(), P(), F(), R())

   DIM Ld(6, 6), LdT(6, 6), Kg(6, 6), S(6, 1), Sg(6, 1)

   CALL AzzeraMat(R())
   FOR I = 1 TO IZ
      FOR K = 1 TO 3
         P = N(I, K)
         IF P <> 0 THEN R(P) = R(P) + F(I, K)
      NEXT K
   NEXT I
   FOR J = 1 TO JZ
      IF TA$(J) <> "Q8" AND (P(J, 1) <> 0 OR P(J, 2) <> 0) THEN
         E1 = Estr(J, 1)
         E2 = Estr(J, 2)
         CALL Geometria(X(E1), X(E2), Y(E1), Y(E2), Lun, SnAl, CsAl, Ld())
         IF LEFT$(TA$(J), 1) = "W" THEN
            IF TA$(J) = "W" THEN
               CALL Kg.Wnk(K(J) * 1000, B(J), E, ZA(NT(J)), ZI(NT(J)), Lun, Ld(), Kg())
            ELSEIF TA$(J) = "WR" THEN
               TrRigS = TrRig(J, 1) / Lun
               TrRigT = TrRig(J, 2) / Lun
               CALL Kg.WnkRig(K(J) * 1000, B(J), E, ZA(NT(J)), ZI(NT(J)), Lun, TrRigS, TrRigT, Ld(), Kg())
            END IF
            CALL TrasponeMat(Ld(), LdT())
         END IF
         IF TrRig <> 0 THEN
            TrRig1 = TrRig(J, 1)
            TrRig2 = TrRig(J, 2)
         END IF
         CALL AzIncLocale(P(J, 1), P(J, 2), TA$(J), B(J), K(J) * 1000, TrRig1, TrRig2, Lun, SnAl, CsAl, Ld(), LdT(), Kg(), S())
         CALL ProdottoMat(Ld(), S(), Sg())
         FOR K = 1 TO 6
            IF K <= 3 THEN N = N(E1, K) ELSE N = N(E2, K - 3)
            IF N <> 0 THEN R(N) = R(N) - Sg(K, 1)
         NEXT K
      END IF
   NEXT J

END SUB

' ================================ VincoliCed ==================================
'   Procedura per l'aggiunta della rigidezza dei vincoli cedevoli
'   alla matrice di rigidezza
' ------------------------------------------------------------------------------
SUB VincoliCed (IZ, N(), KV(), NL, SB, KTT(), KTA(), KAA())

   FOR I = 1 TO IZ
      FOR K = 1 TO 3
         IF N(I, K) <> 0 THEN
            P = N(I, K)
            CALL Inserimento(P, P, KV(I, K), NL, SB, KTT(), KTA(), KAA())
         END IF
      NEXT K
   NEXT I

END SUB

