'                                  PRORET.BAS
'
'  procedure del programma TELRET, utilizzate anche nel programma TELSPA
'
'  ver. 2.5                                                     rev. 02.99


' ------------------ Dichiarazione delle procedure utilizzate ------------------

' procedure del file MATRIX.BAS
DECLARE SUB AzzeraMat (A!())

' procedure di questo stesso file
DECLARE SUB ControllaSezione (L!, ZA!(), ZI!())
DECLARE SUB PrintCampata (F!, RZ!, NP!())
DECLARE SUB PrintPiano (P!)
DECLARE SUB PrintPilastro (R!, RZ!, NP!())
DECLARE SUB PrintSez (L!, FS$(), ZB!(), ZH!(), ZI!(), TrRig!, TrRigSin!, TrRigDes!)

' ================================ ControllaSezione ============================
'
'   Procedura che controlla se a un tipo di sezione corrisponde
'   area e momento d'inerzia nullo
'
'   Variabili di ingresso:
'      L         numero d'ordine del tipo di sezione
'      ZA()      array che contiene le aree
'      ZI()      array che contiene i momenti di inerzia
'
'   Variabili di uscita:
'      L         se area e momento d'inerzia sono nulli fornisce L=0
'
' ------------------------------------------------------------------------------
SUB ControllaSezione (L, ZA(), ZI())

   IF L <> 0 THEN
      IF ZA(L) = 0 AND ZI(L) = 0 THEN L = 0
   END IF

END SUB

' ================================ ContrRitti ==================================
'   Procedura per il controllo che tra i ritti non vi siano sezioni
'   inammissibili (tipo 0)
' ------------------------------------------------------------------------------
SUB ContrRitti (PZ, RZ, NP(), NB(), NR(), ZA(), ZI(), ER)
 
   FOR P = 1 TO PZ
      FOR R = 1 TO RZ
         ERpil = 0
         IF NR(R, P) = 0 THEN
            FOR P1 = P + 1 TO PZ
               L = NR(R, P1)
               CALL ControllaSezione(L, ZA(), ZI())
               IF L <> 0 THEN ERpil = 1: EXIT FOR
            NEXT P1
            IF R > 1 AND ERpil = 0 THEN
               FOR P1 = P TO PZ
                  L = NB(R - 1, P1)
                  CALL ControllaSezione(L, ZA(), ZI())
                  IF L <> 0 THEN ERpil = 1: EXIT FOR
               NEXT P1
            END IF
            IF R < RZ AND ERpil = 0 THEN
               FOR P1 = P TO PZ
                  L = NB(R, P1)
                  CALL ControllaSezione(L, ZA(), ZI())
                  IF L <> 0 THEN ERpil = 1: EXIT FOR
               NEXT P1
            END IF
         END IF
         IF ERpil = 1 THEN
            PRINT "ritto"; NP(R), "piano"; P, "tipo di sezione non ammissibile"
            ER = 1
         END IF
      NEXT R
   NEXT P
   IF ER = 1 THEN
      PRINT "   Esecuzione interrotta:  sezione tipo 0 non ammissibile perch"
      PRINT "   interrompe la continuit verticale  ed  quindi incompatibile"
      PRINT "   con l'ipotesi di indeformabilit estensionale dei ritti"
   END IF

END SUB

' ================================ ConverteCarRet ==============================
'   Procedura per la conversione dei carichi sui nodi e sulle aste
' ------------------------------------------------------------------------------
SUB ConverteCarRet (RZ, PZ, Wnkl, NB(), NF(), ZA(), ZI(), IV, IM, IN, IO, Q(), QF(), MN(), MNF(), FYN(), FYF(), FX(), KZ, V$(), NI(), P(), F())

   CALL AzzeraMat(F())
   CALL AzzeraMat(P())
 
   ' carichi verticali distribuiti
   IF IV = 1 THEN
      J = 0
      'trave di fondazione
      IF Wnkl = 1 THEN
         FOR F = 0 TO RZ
            L = NF(F)
            CALL ControllaSezione(L, ZA(), ZI())
            IF L <> 0 THEN
               J = J + 1
               P(J, 2) = QF(F)
            END IF
         NEXT F
      END IF
      ' travi in elevazione
      FOR P = 1 TO PZ
         FOR R = 1 TO RZ - 1
            L = NB(R, P)
            CALL ControllaSezione(L, ZA(), ZI())
            IF L <> 0 THEN
               J = J + 1
               P(J, 2) = Q(R, P)
            END IF
         NEXT R
      NEXT P
   END IF

   ' momenti concentrati
   IF IM = 1 THEN
      ' nodi della trave di fondazione
      IF Wnkl = 1 THEN
         FOR F = 0 TO RZ + 1
            I = F + 1
            IF V$(I) <> "111" THEN F(I, 3) = MNF(F)
         NEXT F
      END IF
      ' nodi in elevazione
      FOR P = 1 TO PZ
         FOR R = 1 TO RZ
            I = RZ * P + 2 + R
            IF V$(I) <> "111" THEN F(I, 3) = MN(R, P)
         NEXT R
      NEXT P
   END IF
  
   ' forze verticali concentrate
   IF IN = 1 THEN
      ' nodi della trave di fondazione
      IF Wnkl = 1 THEN
         FOR F = 0 TO RZ + 1
            I = F + 1
            IF V$(I) <> "111" THEN F(I, 2) = FYF(F)
         NEXT F
      END IF
      ' nodi in elevazione
      FOR P = 1 TO PZ
         FOR R = 1 TO RZ
            I = RZ * P + 2 + R
            IF V$(I) <> "111" THEN F(I, 2) = FYN(R, P)
         NEXT R
      NEXT P
   END IF
 
   ' forze orizzontali
   IF IO = 1 THEN
      I = 0
      FOR K = 1 TO KZ
         I = I + 1
         F(NI(I), 1) = FX(K)
         DO
            I = I + 1
         LOOP UNTIL NI(I) = 0
      NEXT K
   END IF

END SUB

' ================================ ConverteGeomRet =============================
'   Procedura per la conversione dei dati geometrici
' ------------------------------------------------------------------------------
SUB ConverteGeomRet (PZ, RZ, Wnkl, TrRig, DefEst, DL(), H(), NB(), NR(), NF(), BF(), KF(), TrRigTra(), TrRigPil(), TrRigFon(), ZA(), ZI(), IZ, JZ, KZ, X(), Y(), V$(), NI(), TA$(), Estr(), NT(), B(), K(), TrRig())

   ' coordinate nodi in fondazione
   X(1) = 0
   Y(1) = H(1, 0)
   FOR R = 1 TO RZ + 1
      I = R + 1
      X(I) = X(I - 1) + DL(I - 2)
      IF R <= RZ THEN
         Y(I) = H(R, 0)
      ELSE
         Y(I) = H(RZ, 0)
      END IF
   NEXT R
   ' coordinate nodi in elevazione
   FOR P = 1 TO PZ
      FOR R = 1 TO RZ
         I = RZ * P + 2 + R
         IF P = 1 THEN Iinf = I - RZ - 1 ELSE Iinf = I - RZ
         X(I) = X(Iinf)
         Y(I) = Y(Iinf) + H(R, P)
      NEXT R
   NEXT P
  
   ' definizione aste - travi in fondazione
   J = 0
   IF Wnkl = 1 THEN
      FOR F = 0 TO RZ
         L = NF(F)
         CALL ControllaSezione(L, ZA(), ZI())
         IF L <> 0 THEN
            J = J + 1
            Estr(J, 1) = F + 1
            Estr(J, 2) = F + 2
            NT(J) = L
            B(J) = BF(F)
            K(J) = KF(F)
            IF TrRig = 2 THEN
               TA$(J) = "WR"
               TrRig(J, 1) = TrRigFon(F, 1)
               TrRig(J, 2) = TrRigFon(F, 2)
            ELSE
               TA$(J) = "W"
            END IF
         END IF
      NEXT F
   END IF
   JF = J
   ' definizione aste - travi in elevazione
   FOR P = 1 TO PZ
      FOR R = 1 TO RZ - 1
         L = NB(R, P)
         CALL ControllaSezione(L, ZA(), ZI())
         IF L <> 0 THEN
            J = J + 1
            Estr(J, 1) = RZ * P + 2 + R
            Estr(J, 2) = Estr(J, 1) + 1
            NT(J) = L
            IF TrRig = 0 THEN
               TA$(J) = "T"
            ELSE
               TA$(J) = "TR"
               TrRig(J, 1) = TrRigTra(R, P, 1)
               TrRig(J, 2) = TrRigTra(R, P, 2)
            END IF
         END IF
      NEXT R
   NEXT P
   ' definizione aste - pilastri
   FOR P = 1 TO PZ
      FOR R = 1 TO RZ
         L = NR(R, P)
         CALL ControllaSezione(L, ZA(), ZI())
         IF L <> 0 THEN
            J = J + 1
            I = RZ * P + 2 + R
            IF P = 1 THEN Iinf = I - RZ - 1 ELSE Iinf = I - RZ
            Estr(J, 1) = Iinf
            Estr(J, 2) = I
            NT(J) = L
            IF TrRig = 0 THEN
               TA$(J) = "T"
            ELSE
               TA$(J) = "TR"
               TrRig(J, 1) = TrRigPil(R, P, 1)
               TrRig(J, 2) = TrRigPil(R, P, 2)
            END IF
         END IF
      NEXT R
   NEXT P
   JZ = J
 
   ' definizione codici vincolo
   FOR J = JF + 1 TO JZ
      IF DefEst = 0 THEN
         V$(Estr(J, 1)) = "010"
         V$(Estr(J, 2)) = "010"
      ELSE
         V$(Estr(J, 1)) = "000"
         V$(Estr(J, 2)) = "000"
      END IF
   NEXT J
   IF Wnkl = 1 THEN
      FOR J = 1 TO JF
         V$(Estr(J, 1)) = "100"
         V$(Estr(J, 2)) = "100"
      NEXT J
   END IF
   FOR I = 1 TO IZ
      IF V$(I) = "" THEN V$(I) = "111"
   NEXT I
   IF Wnkl <> 1 THEN
      FOR I = 1 TO RZ + 2
         V$(I) = "111"
      NEXT I
   END IF

   ' definizione traversi
   EP = 0
   K = 0
   I = 0
   FOR P = 1 TO PZ
      FOR R = RZ - 1 TO 1 STEP -1
         L = NB(R, P)
         IF L <> 0 THEN
            E1 = RZ * P + 2 + R
            E2 = E1 + 1
            IF E2 <> EP THEN
               IF K > 0 THEN I = I + 1: NI(I) = 0
               K = K + 1
               IF V$(E2) <> "111" THEN I = I + 1: NI(I) = E2
            END IF
            IF V$(E1) <> "111" THEN I = I + 1: NI(I) = E1
            EP = E1
         END IF
      NEXT R
   NEXT P
   I = I + 1: NI(I) = 0
   KZ = K

END SUB

' ================================ ConverteRisRet ==============================
'   Procedura per la valutazione degli spostamenti dei traversi
'   e la conversione delle caratteristiche di sollecitazione
' ------------------------------------------------------------------------------
SUB ConverteRisRet (RZ, PZ, Wnkl, DefEst, NB(), NR(), NF(), ZA(), ZI(), D(), MS(), MD(), TS(), TD(), MP(), MT(), TR(), SN(), MFS(), MFD(), TFS(), TFD(), KZ, NI(), N(), R(), F(), M1(), M2(), T1(), T2(), N1())

   ' spostamenti traversi
   I = 0
   FOR K = 1 TO KZ
      I = I + 1
      IF N(NI(I), 1) = 0 THEN D(K) = 0 ELSE D(K) = R(N(NI(I), 1))
      DO
         I = I + 1
      LOOP UNTIL NI(I) = 0
   NEXT K
   ' caratteristiche di sollecitazione nella trave di fondazione
   J = 0
   IF Wnkl = 1 THEN
      FOR F = 0 TO RZ
         L = NF(F)
         CALL ControllaSezione(L, ZA(), ZI())
         IF L <> 0 THEN
            J = J + 1
            MFS(F) = M1(J)
            TFS(F) = T1(J)
            MFD(F) = M2(J)
            TFD(F) = T2(J)
         END IF
      NEXT F
   END IF
   ' caratteristiche di sollecitazione nelle travi in elevazione
   FOR P = 1 TO PZ
      FOR R = 1 TO RZ - 1
         L = NB(R, P)
         CALL ControllaSezione(L, ZA(), ZI())
         IF L <> 0 THEN
            J = J + 1
            MS(R, P) = M1(J)
            TS(R, P) = T1(J)
            MD(R, P) = M2(J)
            TD(R, P) = T2(J)
         END IF
      NEXT R
   NEXT P
   ' taglio, momento flettente e sforzo normale nei pilastri
   FOR P = 1 TO PZ
      FOR R = 1 TO RZ
         L = NR(R, P)
         CALL ControllaSezione(L, ZA(), ZI())
         IF L <> 0 THEN
            J = J + 1
            MP(R, P) = M1(J)
            MT(R, P) = M2(J)
            TR(R, P) = T1(J)
            IF DefEst = 1 THEN
               SN(R, P) = N1(J)
            ELSE
               I = RZ * P + 2 + R
               SN(R, P) = F(I, 2)
            END IF
         END IF
      NEXT R
   NEXT P
   IF DefEst = 0 THEN
      FOR R = 1 TO RZ
         FOR P = PZ - 1 TO 1 STEP -1
            SN(R, P) = SN(R, P) + SN(R, P + 1)
         NEXT P
      NEXT R
    END IF

END SUB

' ================================ InputAltezze ================================
'   Procedura per la lettura delle altezze dei ritti
'
'  Dalla versione 2.5  possibile definire un piano di posa non orizzontale;
'  l'array H() deve essere dimensionato per ammettere anche 0 come 2 indice
' ------------------------------------------------------------------------------
SUB InputAltezze (Contr, PZ, RZ, AltVar, HI(), H())

   AltVar = 0
   FOR P = PZ TO 1 STEP -1
      INPUT #1, HI(P)
      IF Contr = 1 THEN
         IF HI(P) > 0 THEN
            PRINT #2, "piano"; P, "altezza"; HI(P)
         ELSE
            PRINT #2, "piano"; P, "altezza variabile"
         END IF
      END IF
      IF HI(P) <= 0 THEN AltVar = 1
   NEXT P
   FOR P = PZ TO 1 STEP -1
      IF HI(P) > 0 THEN
         FOR R = 1 TO RZ
            H(R, P) = HI(P)
         NEXT R
      ELSE
         FOR R = 1 TO RZ
            INPUT #1, H(R, P)
            IF Contr = 1 THEN
               PRINT #2, "piano"; P, "ritto"; R, "altezza"; H(R, P)
            END IF
         NEXT R
      END IF
   NEXT P
   IF HI(1) >= 0 THEN
      FOR R = 1 TO RZ
         H(R, 0) = 0
      NEXT R
   ELSE
      FOR R = 1 TO RZ
         INPUT #1, H(R, 0)
         IF Contr = 1 THEN
            PRINT #2, "piano"; P, "ritto"; R, "altezza"; H(R, 0)
         END IF
      NEXT R
   END IF

END SUB

' ================================ InputAsteRet ================================
'   Procedura per la lettura dei dati relativi alle aste
' ------------------------------------------------------------------------------
SUB InputAsteRet (Contr, Wnkl, TrRig, PZ, RZ, LZ, NP(), NB(), NR(), NF(), BF(), KF(), TrRigTra(), TrRigPil(), TrRigFon(), ER)

   ' travi in elevazione
   FOR P = PZ TO 1 STEP -1
      FOR R = 1 TO RZ - 1
         INPUT #1, NB(R, P)
         IF TrRig <> 0 AND NB(R, P) <> 0 THEN
            INPUT #1, TrRigTra(R, P, 1), TrRigTra(R, P, 2)
         END IF
         IF Contr = 1 THEN
            PRINT #2, "campata"; NP(R); NP(R + 1), "piano"; P,
            PRINT #2, "sezione tipo"; NB(R, P),
            IF TrRig <> 0 AND NB(R, P) <> 0 THEN
               PRINT #2, "tratti rigidi"; TrRigTra(R, P, 1), TrRigTra(R, P, 2),
            END IF
            PRINT #2,
         END IF
         IF NB(R, P) < 0 OR NB(R, P) > LZ THEN
            PRINT "campata"; NP(R); NP(R + 1), "piano"; P,
            PRINT "sezione tipo"; NB(R, P); "  non accettabile"
            ER = 1
         END IF
      NEXT R
   NEXT P
   ' ritti
   FOR P = PZ TO 1 STEP -1
      FOR R = 1 TO RZ
         INPUT #1, NR(R, P)
         IF TrRig <> 0 AND NR(R, P) <> 0 THEN
            INPUT #1, TrRigPil(R, P, 1), TrRigPil(R, P, 2)
         END IF
         IF Contr = 1 THEN
            PRINT #2, "ritto"; NP(R), "piano"; P, "sezione tipo"; NR(R, P),
            IF TrRig <> 0 AND NR(R, P) <> 0 THEN
               PRINT #2, "tratti rigidi"; TrRigPil(R, P, 1), TrRigPil(R, P, 2),
            END IF
            PRINT #2,
         END IF
         IF NR(R, P) < 0 OR NR(R, P) > LZ THEN
            PRINT "ritto"; NP(R), "piano"; P,
            PRINT "sezione tipo"; NR(R, P); "  non accettabile"
            ER = 1
         END IF
      NEXT R
   NEXT P
   ' trave in fondazione
   IF Wnkl = 1 THEN
      FOR F = 0 TO RZ
         INPUT #1, NF(F), BF(F), KF(F)
         IF TrRig = 2 AND NF(F) <> 0 THEN
            INPUT #1, TrRigFon(F, 1), TrRigFon(F, 2)
         END IF
         IF Contr = 1 THEN
            PRINT #2, "trave di fondazione - tronco"; F,
            PRINT #2, "sezione tipo"; NF(F); "   B ="; BF(F); "   K ="; KF(F),
            IF TrRig = 2 AND NF(F) <> 0 THEN
               PRINT #2, "tratti rigidi"; TrRigFon(F, 1), TrRigFon(F, 2),
            END IF
            PRINT #2,
         END IF
         IF NF(F) < 0 OR NF(F) > LZ THEN
            PRINT "trave di fondazione - tronco"; F,
            PRINT "sezione tipo"; NF(F); "  non accettabile"
            ER = 1
         END IF
      NEXT F
   END IF

END SUB

' ================================ InputCarRet =================================
'   Procedura per la lettura dei carichi su aste, nodi e traversi
' ------------------------------------------------------------------------------
SUB InputCarRet (Contr, Wnkl, IV, IM, IN, IO, PZ, RZ, KZ, NP(), V$(), NB(), NF(), ZA(), ZI(), Q(), QF(), MN(), MNF(), FYN(), FYF(), FX())

   IV = 0: IM = 0: IN = 0: IO = 0
   INPUT #1, IV
   IF NOT EOF(1) THEN INPUT #1, IM
   IF NOT EOF(1) THEN INPUT #1, IN
   IF NOT EOF(1) THEN INPUT #1, IO
   IF IV = 1 THEN GOSUB CarVert
   IF IM = 1 THEN GOSUB MomNodali
   IF IN = 1 THEN GOSUB ForzeNodali
   IF IO = 1 THEN GOSUB ForzeOriz
EXIT SUB

CarVert:
   IF Contr = 1 THEN PRINT #2, "carichi verticali"
   FOR P = PZ TO 1 STEP -1
      FOR R = 1 TO RZ - 1
         L = NB(R, P)
         CALL ControllaSezione(L, ZA(), ZI())
         IF L <> 0 THEN
            INPUT #1, Q(R, P)
            IF Contr = 1 THEN
               PRINT #2, "campata"; NP(R); NP(R + 1), "piano"; P, "py ="; Q(R, P)
            END IF
         END IF
      NEXT R
   NEXT P
   IF Wnkl = 1 THEN
      FOR F = 0 TO RZ
         L = NF(F)
         CALL ControllaSezione(L, ZA(), ZI())
         IF L <> 0 THEN
            INPUT #1, QF(F)
            IF Contr = 1 THEN
               PRINT #2, "trave di fondazione, tratto"; F, "py ="; QF(F)
            END IF
         END IF
      NEXT F
   END IF
RETURN

MomNodali:
   IF Contr = 1 THEN PRINT #2, "momenti nodali"
   FOR P = PZ TO 1 STEP -1
      FOR R = 1 TO RZ
         I = RZ * P + 2 + R
         IF V$(I) <> "111" THEN
            INPUT #1, MN(R, P)
            IF Contr = 1 THEN
               PRINT #2, "pilastro"; NP(R), "piano"; P, "M ="; MN(R, P)
            END IF
         END IF
      NEXT R
   NEXT P
   IF Wnkl = 1 THEN
      FOR F = 0 TO RZ + 1
         I = F + 1
         IF V$(I) <> "111" THEN
            INPUT #1, MNF(F)
            IF Contr = 1 THEN
               PRINT #2, "trave di fondazione, nodo ";
               CALL PrintPilastro(F, RZ, NP())
               PRINT #2, "M ="; MNF(F)
            END IF
         END IF
      NEXT F
   END IF
RETURN

ForzeNodali:
   IF Contr = 1 THEN PRINT #2, "forze verticali nodali"
   FOR P = PZ TO 1 STEP -1
      FOR R = 1 TO RZ
         I = RZ * P + 2 + R
         IF V$(I) <> "111" THEN
            INPUT #1, FYN(R, P)
            IF Contr = 1 THEN
               PRINT #2, "pilastro"; NP(R), "piano"; P, "Fy ="; FYN(R, P)
            END IF
         END IF
      NEXT R
   NEXT P
   IF Wnkl = 1 THEN
      FOR F = 0 TO RZ + 1
         I = F + 1
         IF V$(I) <> "111" THEN
            INPUT #1, FYF(F)
            IF Contr = 1 THEN
               PRINT #2, "trave di fondazione, nodo ";
               CALL PrintPilastro(F, RZ, NP())
               PRINT #2, "Fy ="; FYF(F)
            END IF
         END IF
      NEXT F
   END IF
RETURN

ForzeOriz:
   IF Contr = 1 THEN PRINT #2, "forze orizzontali"
   FOR K = KZ TO 1 STEP -1
      INPUT #1, FX(K)
      IF Contr = 1 THEN
         PRINT #2, "traverso"; K, "F ="; FX(K)
      END IF
   NEXT K
RETURN

END SUB

' ================================ InputLuci ===================================
'   Procedura per la lettura delle luci delle campate
' ------------------------------------------------------------------------------
SUB InputLuci (Contr, Wnkl, RZ, DL())
 
   IF Wnkl = 1 THEN
      R1 = 0
      R2 = RZ
   ELSE
      R1 = 1
      R2 = RZ - 1
   END IF
   FOR R = R1 TO R2
      INPUT #1, DL(R)
      IF Contr = 1 THEN
         SELECT CASE R
            CASE 0
               PRINT #2, "sbalzo sinistro trave fondazione", "luce"; DL(R)
            CASE 1 TO RZ - 1
               PRINT #2, "campata"; R, "luce"; DL(R)
            CASE RZ
               PRINT #2, "sbalzo destro trave fondazione", " luce "; DL(R)
         END SELECT
      END IF
   NEXT R
  
END SUB

' ================================ InputMasseRet ===============================
'   Procedura per la lettura delle masse (per l'analisi modale)
' ------------------------------------------------------------------------------
SUB InputMasseRet (Contr, KZ, Mas())
  
   FOR K = KZ TO 1 STEP -1
      INPUT #1, Mas(K)
      IF Contr = 1 THEN
         PRINT #2, "traverso"; K, "m ="; Mas(K)
      END IF
   NEXT K

END SUB

' ================================ InputNumRitti ===============================
'   Procedura per la lettura della numerazione in carpenteria dei ritti
' ------------------------------------------------------------------------------
SUB InputNumRitti (Contr, RZ, NP())

   FOR R = 1 TO RZ
      INPUT #1, NP(R)
      IF Contr = 1 THEN
         PRINT #2, "ritto"; R, "numerazione in carpenteria"; NP(R)
      END IF
   NEXT R

END SUB

' ================================ OutputAsteRet ===============================
'   Procedura per la stampa dei dati relativi a travi in elevazione,
'   pilastri e trave di fondazione
' ------------------------------------------------------------------------------
SUB OutputAsteRet (SM, PZ, RZ, Wnkl, TrRig, NP(), DL(), NB(), NR(), NF(), BF(), KF(), TrRigTra(), TrRigPil(), TrRigFon(), FS$(), ZB(), ZH(), ZI())
  
   ' travi in elevazione
   PRINT #2, : PRINT #2, : PRINT #2,
   PRINT #2, "LUCI E SEZIONI DELLE TRAVI": PRINT #2,
   PRINT #2, " campata   luce  piano sez.  b     h    inerzia";
   IF TrRig <> 0 THEN
      PRINT #2, "    tratto rigido";
   ELSEIF PZ > 1 THEN
      PRINT #2, "  piano sez.  b     h    inerzia";
   END IF
   PRINT #2,
   PRINT #2, "             m               m     m       m4  ";
   IF TrRig <> 0 THEN
      PRINT #2, "  sin.(m)   des.(m)";
   ELSEIF PZ > 1 THEN
      PRINT #2, "              m     m       m4  ";
   END IF
   PRINT #2, : PRINT #2,
   FOR R = 1 TO RZ - 1
      PRINT #2, USING "### -###  ##.##"; NP(R); NP(R + 1); DL(R);
      FOR P = PZ TO 1 STEP -1
         L = NB(R, P)
         IF (TrRig <> 0 AND P < PZ) OR ((PZ - P) MOD 2 = 0 AND P < PZ) THEN
            PRINT #2, : PRINT #2, "               ";
         END IF
         PRINT #2, USING "   ##  "; P;
         IF TrRig <> 0 THEN
            TrRigSin = TrRigTra(R, P, 1)
            TrRigDes = TrRigTra(R, P, 2)
         END IF
         CALL PrintSez(L, FS$(), ZB(), ZH(), ZI(), TrRig, TrRigSin, TrRigDes)
      NEXT P
      PRINT #2, : PRINT #2,
   NEXT R
   ' pilastri
   PRINT #2, : PRINT #2, : PRINT #2,
   PRINT #2, "SEZIONI DEI PILASTRI": PRINT #2,
   PRINT #2, "pilastro         piano sez.  b     h    inerzia";
   IF TrRig <> 0 THEN
      PRINT #2, "    tratto rigido";
   ELSEIF PZ > 1 THEN
      PRINT #2, "  piano sez.  b     h    inerzia";
   END IF
   PRINT #2,
   PRINT #2, "                             m     m       m4  ";
   IF TrRig <> 0 THEN
      PRINT #2, "  inf.(m)   sup.(m)";
   ELSEIF PZ > 1 THEN
      PRINT #2, "              m     m       m4  ";
   END IF
   PRINT #2, : PRINT #2,
   FOR R = 1 TO RZ
      PRINT #2, USING "  ###          "; NP(R);
      FOR P = PZ TO 1 STEP -1
         L = NR(R, P)
         IF (TrRig <> 0 AND P < PZ) OR ((PZ - P) MOD 2 = 0 AND P < PZ) THEN
            PRINT #2, : PRINT #2, "               ";
         END IF
         PRINT #2, USING "   ##  "; P;
         IF TrRig <> 0 THEN
            TrRigInf = TrRigPil(R, P, 1)
            TrRigSup = TrRigPil(R, P, 2)
         END IF
         CALL PrintSez(L, FS$(), ZB(), ZH(), ZI(), TrRig, TrRigInf, TrRigSup)
      NEXT P
      PRINT #2, : PRINT #2,
   NEXT R
   ' trave di fondazione
   IF Wnkl = 1 THEN
      PRINT #2, : PRINT #2, : PRINT #2,
      PRINT #2, "CARATTERISTICHE DELLA TRAVE DI FONDAZIONE": PRINT #2,
      PRINT #2, " campata   luce        sez.  b     h    inerzia    B     K   ";
      IF TrRig = 2 THEN
         PRINT #2, "    tratto rigido";
      END IF
      PRINT #2,
      IF SM = 0 THEN
         PRINT #2, "             m               m     m       m4      m   N/cm3 ";
      ELSE
         PRINT #2, "             m               m     m       m4      m   kg/cm3";
      END IF
      IF TrRig = 2 THEN
         PRINT #2, "  inf.(m)   sup.(m)";
      END IF
      PRINT #2, : PRINT #2,
      FOR F = 0 TO RZ
         L = NF(F)
         CALL PrintCampata(F, RZ, NP())
         PRINT #2, USING "  ##.##       "; DL(F);
         CALL PrintSez(L, FS$(), ZB(), ZH(), ZI(), 0, 0, 0)
         IF SM = 0 THEN
            PRINT #2, USING " ##.## ####.#"; BF(F); KF(F);
         ELSE
            PRINT #2, USING " ##.## ###.##"; BF(F); KF(F);
         END IF
         IF TrRig = 2 THEN
            TrRigSin = TrRigFon(F, 1)
            TrRigDes = TrRigFon(F, 2)
            IF TrRigSin = 0 THEN
               PRINT #2, "    ----  ";
            ELSE
               PRINT #2, USING "   ##.##  "; TrRigSin;
            END IF
            IF TrRigDes = 0 THEN
               PRINT #2, "    ----  ";
            ELSE
               PRINT #2, USING "   ##.##  "; TrRigDes;
            END IF
         END IF
         PRINT #2,
      NEXT F
   END IF

END SUB

' ================================ OutputCarRet ================================
'   Procedura per la stampa dei carichi su aste, nodi e traversi
' ------------------------------------------------------------------------------
SUB OutputCarRet (SM, Wnkl, IV, IM, IN, IO, PZ, RZ, KZ, NP(), V$(), NB(), NF(), ZA(), ZI(), Q(), QF(), MN(), MNF(), FYN(), FYF(), FX())

   IF Wnkl = 1 THEN
      PV = PZ + 1
      R1 = 0
      R2 = RZ + 1
      P1 = 0
   ELSE
      PV = PZ
      R1 = 1
      R2 = RZ
      P1 = 1
   END IF
   IF PV > 4 THEN PV = 4
  
   IF IV = 1 THEN GOSUB OutCarVert
   IF IM = 1 THEN GOSUB OutMomNodali
   IF IN = 1 THEN GOSUB OutForzeNodali
   IF IO = 1 THEN GOSUB OutForzeOriz
EXIT SUB

OutCarVert:
   PRINT #2, : PRINT #2, : PRINT #2,
   PRINT #2, "CARICHI VERTICALI": PRINT #2,
   PRINT #2, " campata";
   FOR P = 1 TO PV
      PRINT #2, "   piano  carico";
   NEXT P
   PRINT #2,
   PRINT #2, "        ";
   FOR P = 1 TO PV
      IF SM = 0 THEN
         PRINT #2, "           kN/m ";
      ELSE
         PRINT #2, "            t/m ";
      END IF
   NEXT P
   PRINT #2, : PRINT #2,
   FOR R = R1 TO R2 - 1
      CALL PrintCampata(R, RZ, NP())
      FOR P = PZ TO P1 STEP -1
         IF P = 0 OR (R >= 1 AND R <= RZ - 1) THEN
            IF (PZ - P) MOD 4 = 0 AND P < PZ AND R <> 0 AND R <> RZ THEN
               PRINT #2, : PRINT #2, "        ";
            END IF
            IF P = 0 THEN
               L = NF(R)
               Q = QF(R)
            ELSE
               L = NB(R, P)
               Q = Q(R, P)
            END IF
            CALL ControllaSezione(L, ZA(), ZI())
            PRINT #2, "  ";
            CALL PrintPiano(P)
            PRINT #2, " ";
            IF L <> 0 THEN
               IF SM = 0 THEN
                  PRINT #2, USING "#####.# "; Q;
               ELSE
                  PRINT #2, USING "####.## "; Q;
               END IF
            ELSE
               PRINT #2, "   ---- ";
            END IF
         END IF
      NEXT P
      PRINT #2,
      IF PZ + 1 - P1 > 4 THEN PRINT #2,
   NEXT R
RETURN

OutMomNodali:
   PRINT #2, : PRINT #2, : PRINT #2,
   PRINT #2, "MOMENTI NEI NODI ": PRINT #2,
   PRINT #2, "pilastro";
   FOR P = 1 TO PV
      PRINT #2, "   piano momento";
   NEXT P
   PRINT #2,
   PRINT #2, "        ";
   FOR P = 1 TO PV
      IF SM = 0 THEN
         PRINT #2, "           kNm  ";
      ELSE
         PRINT #2, "            tm  ";
      END IF
   NEXT P
   PRINT #2, : PRINT #2,
   FOR R = R1 TO R2
      CALL PrintPilastro(R, RZ, NP())
      FOR P = PZ TO P1 STEP -1
         IF P = 0 OR (R >= 1 AND R <= RZ) THEN
            IF (PZ - P) MOD 4 = 0 AND P < PZ AND R <> 0 AND R <> RZ + 1 THEN
               PRINT #2, : PRINT #2, "        ";
            END IF
            IF P = 0 THEN
               I = R + 1
               Q = MNF(R)
            ELSE
               I = RZ * P + 2 + R
               Q = MN(R, P)
            END IF
            PRINT #2, "   ";
            CALL PrintPiano(P)
            IF V$(I) = "111" THEN
               PRINT #2, "   ---- ";
            ELSE
               IF SM = 0 THEN
                  PRINT #2, USING "#####.# "; Q;
               ELSE
                  PRINT #2, USING "####.## "; Q;
               END IF
            END IF
         END IF
      NEXT P
      PRINT #2,
      IF PZ + 1 - P1 > 4 THEN PRINT #2,
   NEXT R
RETURN

OutForzeNodali:
   PRINT #2, : PRINT #2, : PRINT #2,
   PRINT #2, "FORZE VERTICALI NEI NODI ": PRINT #2,
   PRINT #2, "pilastro";
   FOR P = 1 TO PV
      PRINT #2, "   piano  forza ";
   NEXT P
   PRINT #2,
   PRINT #2, "        ";
   FOR P = 1 TO PV
      IF SM = 0 THEN
         PRINT #2, "            kN  ";
      ELSE
         PRINT #2, "             t  ";
      END IF
   NEXT P
   PRINT #2, : PRINT #2,
   FOR R = R1 TO R2
      CALL PrintPilastro(R, RZ, NP())
      FOR P = PZ TO P1 STEP -1
         IF P = 0 OR (R >= 1 AND R <= RZ) THEN
            IF (PZ - P) MOD 4 = 0 AND P < PZ AND R <> 0 AND R <> RZ + 1 THEN
               PRINT #2, : PRINT #2, "        ";
            END IF
            IF P = 0 THEN
               I = R + 1
               Q = FYF(R)
            ELSE
               I = RZ * P + 2 + R
               Q = FYN(R, P)
            END IF
            PRINT #2, "   ";
            CALL PrintPiano(P)
            IF V$(I) = "111" THEN
               PRINT #2, "   ---- ";
            ELSE
               IF SM = 0 THEN
                  PRINT #2, USING "#####.# "; Q;
               ELSE
                  PRINT #2, USING "####.## "; Q;
               END IF
            END IF
         END IF
      NEXT P
      PRINT #2,
      IF PZ + 1 - P1 > 4 THEN PRINT #2,
   NEXT R
RETURN

OutForzeOriz:
   PRINT #2, : PRINT #2, : PRINT #2,
   PRINT #2, "FORZE ORIZZONTALI": PRINT #2,
   PRINT #2, "  traverso    forza"
   IF SM = 0 THEN
      PRINT #2, "                kN": PRINT #2,
   ELSE
      PRINT #2, "                 t": PRINT #2,
   END IF
   FOR K = KZ TO 1 STEP -1
      IF SM = 0 THEN
         PRINT #2, USING "     ##    ######.#"; K; FX(K)
      ELSE
         PRINT #2, USING "     ##    #####.##"; K; FX(K)
      END IF
   NEXT K
RETURN

END SUB

' ================================ OutputMasseRet ==============================
'   Procedura per la stampa delle masse
' ------------------------------------------------------------------------------
SUB OutputMasseRet (KZ, Mas())

   PRINT #2, : PRINT #2, : PRINT #2,
   PRINT #2, "MASSE DI PIANO   ": PRINT #2,
   PRINT #2, "  traverso    massa (t)": PRINT #2,
   FOR K = KZ TO 1 STEP -1
      PRINT #2, USING "     ##    ######.###"; K; Mas(K)
   NEXT K

END SUB

' ================================ OutputPiani =================================
'   Procedura per la stampa delle altezze dei piani
' ------------------------------------------------------------------------------
SUB OutputPiani (AltVar, PZ, RZ, NP(), HI(), H())

   PRINT #2, "ALTEZZA DEI PIANI": PRINT #2,
   PRINT #2, "piano     altezza (m)": PRINT #2,
   FOR P = PZ TO 1 STEP -1
      PRINT #2, USING "###     "; P;
      IF HI(P) > 0 THEN
         PRINT #2, USING "#####.##"; HI(P)
      ELSE
         PRINT #2, "  variabile"
      END IF
   NEXT P
   IF AltVar = 1 THEN
      IF RZ > 4 THEN RV = 4 ELSE RV = RZ
      PRINT #2, : PRINT #2,
      PRINT #2, "piano   ";
      FOR R = 1 TO RV
         PRINT #2, "    pil.  altezza";
      NEXT R
      PRINT #2,
      PRINT #2, "        ";
      FOR R = 1 TO RV
         PRINT #2, "             m   ";
      NEXT R
      PRINT #2, : PRINT #2,
      FOR P = PZ TO 0 STEP -1
         IF P > 0 THEN
            V = HI(P)
         ELSE
            IF HI(1) < 0 THEN V = 0 ELSE V = 1
         END IF
         IF V = 0 THEN
            PRINT #2, USING " ##     "; P;
            FOR R = 1 TO RZ
               IF R MOD 4 = 1 AND R > 1 THEN PRINT #2, : PRINT #2, "        ";
               PRINT #2, USING "    ###   ###.## "; NP(R); H(R, P);
            NEXT R
            PRINT #2, : PRINT #2,
         END IF
      NEXT P
   END IF

END SUB

' ================================ OutputRisRet ================================
'   Procedura per la stampa dei risultati
'
'   N.B.    Outp indica cosa stampare
'              0 oppure j ( > 0 ) = tutto
'              -1 = inviluppo modale (esclude spost.vert., rotaz., squilibrio)
' ------------------------------------------------------------------------------
SUB OutputRisRet (SM, Outp, RZ, PZ, Wnkl, NP(), NB(), NR(), NF(), ZA(), ZI(), MS(), MD(), TS(), TD(), MP(), MT(), TR(), SN(), MFS(), MFD(), TFS(), TFD(), IZ, KZ, NL, V$(), N(), R(), F())
  
   DIM V(3)
   
   ' gli spostamenti orizzontali sono stampati da OutputSpoOriz di PROCOM.BAS
   IF Outp >= 0 THEN GOSUB SpostamentiVer
   GOSUB RisTravi
   GOSUB RisPilastri
   IF Wnkl = 1 THEN GOSUB RisFond
   IF Outp >= 0 THEN GOSUB Squilibrio
EXIT SUB

SpostamentiVer:
   ' stampa spostamenti verticali e rotazioni dei nodi
   PRINT #2, : PRINT #2, : PRINT #2,
   PRINT #2, "SPOSTAMENTO VERTICALE E ROTAZIONE DEI NODI": PRINT #2,
   PRINT #2, "pilastro    piano    vy (mm)   rotaz.x1000";
   IF PZ > 1 THEN PRINT #2, "    piano    vy (mm)   rotaz.x1000";
   PRINT #2, : PRINT #2,
   IF Wnkl = 1 THEN
      PV = PZ + 1
      R1 = 0
      R2 = RZ + 1
      P1 = 0
   ELSE
      PV = PZ
      R1 = 1
      R2 = RZ
      P1 = 1
   END IF
   IF PV > 2 THEN PV = 2
   FOR R = R1 TO R2
      CALL PrintPilastro(R, RZ, NP())
      FOR P = PZ TO P1 STEP -1
         IF P = 0 OR (R >= 1 AND R <= RZ) THEN
            IF P = 0 THEN I = R + 1 ELSE I = RZ * P + 2 + R
            IF N(I, 2) <> 0 THEN VY = R(N(I, 2)) * 1000 ELSE VY = 0
            IF N(I, 3) <> 0 THEN ROT = R(N(I, 3)) * 1000 ELSE ROT = 0
            IF (PZ - P) MOD 2 = 0 AND P < PZ AND R <> 0 AND R <> RZ + 1 THEN
               PRINT #2, : PRINT #2, "        ";
            END IF
            PRINT #2, "    ";
            CALL PrintPiano(P)
            PRINT #2, USING "   ####.###    ####.###   "; VY; ROT;
         END IF
      NEXT P
      PRINT #2, : PRINT #2,
   NEXT R
RETURN

RisTravi:
   PRINT #2, : PRINT #2, : PRINT #2,
   PRINT #2, "CARATTERISTICHE DI SOLLECITAZIONE NELLE TRAVI": PRINT #2,
   PRINT #2, " campata  piano    momento a     momento a     taglio a      taglio a"
   IF SM = 0 THEN
      PRINT #2, "                 sinistra (kNm) destra (kNm) sinistra (kN)  destra (kN)"
   ELSE
      PRINT #2, "                 sinistra (tm)  destra (tm)  sinistra (t)   destra (t)"
   END IF
   FOR R = 1 TO RZ - 1
      PRINT #2,
      FOR P = PZ TO 1 STEP -1
         PRINT #2, USING "### -###   ##"; NP(R); NP(R + 1); P;
         L = NB(R, P)
         CALL ControllaSezione(L, ZA(), ZI())
         IF L <> 0 THEN
            IF SM = 0 THEN
               PRINT #2, USING "   #######.## "; MS(R, P); MD(R, P); TS(R, P); TD(R, P)
            ELSE
               PRINT #2, USING "   ######.### "; MS(R, P); MD(R, P); TS(R, P); TD(R, P)
            END IF
         ELSE
            PRINT #2, USING "        \   \ "; "-----"; "-----"; "-----"; "-----"
         END IF
      NEXT P
   NEXT R
RETURN

RisPilastri:
   PRINT #2, : PRINT #2, : PRINT #2,
   PRINT #2, "CARATTERISTICHE DI SOLLECITAZIONE NEI PILASTRI": PRINT #2,
   PRINT #2, " pilastro piano    momento       momento        taglio       sforzo"
   IF SM = 0 THEN
      PRINT #2, "                  sup. (kNm)    inf. (kNm)       (kN)     normale (kN)"
   ELSE
      PRINT #2, "                  sup. (tm)     inf. (tm)        (t)      normale (t)"
   END IF
   FOR R = 1 TO RZ
      PRINT #2,
      FOR P = PZ TO 1 STEP -1
         PRINT #2, USING "   ###     ##"; NP(R); P;
         L = NR(R, P)
         CALL ControllaSezione(L, ZA(), ZI())
         IF L <> 0 THEN
            IF SM = 0 THEN
               PRINT #2, USING "   #######.## "; MT(R, P); MP(R, P); TR(R, P); SN(R, P)
            ELSE
               PRINT #2, USING "   ######.### "; MT(R, P); MP(R, P); TR(R, P); SN(R, P)
            END IF
         ELSE
            PRINT #2, USING "        \   \ "; "-----"; "-----"; "-----"; "-----"
         END IF
      NEXT P
   NEXT R
RETURN

RisFond:
   PRINT #2, : PRINT #2, : PRINT #2,
   PRINT #2, "CARATTERISTICHE DI SOLLECITAZIONE NELLA TRAVE DI FONDAZIONE": PRINT #2,
   PRINT #2, " campata           momento a     momento a     taglio a      taglio a"
   IF SM = 0 THEN
      PRINT #2, "                 sinistra (kNm) destra (kNm) sinistra (kN)  destra (kN)"
   ELSE
      PRINT #2, "                 sinistra (tm)  destra (tm)  sinistra (t)   destra (t)"
   END IF
   FOR F = 0 TO RZ
      SELECT CASE F
         CASE 0
            PRINT #2, " sb.sin      ";
         CASE 1 TO RZ - 1
            PRINT #2, USING "### -###     "; NP(F); NP(F + 1);
         CASE RZ
            PRINT #2, " sb.des      ";
      END SELECT
      L = NF(F)
      CALL ControllaSezione(L, ZA(), ZI())
      IF L <> 0 THEN
         IF SM = 0 THEN
            PRINT #2, USING "   #######.## "; MFS(F); MFD(F); TFS(F); TFD(F)
         ELSE
            PRINT #2, USING "   ######.### "; MFS(F); MFD(F); TFS(F); TFD(F)
         END IF
      ELSE
         PRINT #2, USING "        \   \ "; "-----"; "-----"; "-----"; "-----"
      END IF
   NEXT F
RETURN

Squilibrio:
   ' controlla se vi  squilibrio nei nodi
   Squil = 0
   FOR I = 1 TO IZ
      FOR K = 1 TO 3
         IF MID$(V$(I), K, 1) = "0" THEN
            IF N(I, K) > NL AND ABS(F(I, K)) > .01 THEN Squil = 1: EXIT FOR
         END IF
      NEXT K
      IF Squil = 1 THEN EXIT FOR
   NEXT I
   ' stampa l'eventuale squilibrio nei nodi
   IF Squil = 1 THEN
      PRINT #2, : PRINT #2,
      PRINT #2, "SQUILIBRIO NEI NODI": PRINT #2,
      IF SM = 0 THEN
         PRINT #2, "pilastro  piano        Fx (kN)      Fy (kN)      M (kNm)"
      ELSE
         PRINT #2, "pilastro  piano        Fx (t)       Fy (t)       M (tm)"
      END IF
      PRINT #2,
      FOR I = 1 TO IZ
         V = 0
         FOR K = 1 TO 3
            IF N(I, K) > NL AND ABS(F(I, K)) > .01 THEN V(K) = 1 ELSE V(K) = 0
            V = V + V(K)
         NEXT K
         IF V <> 0 THEN
            P = INT((I - 3) / RZ)
            IF P = 0 THEN R = I - 1 ELSE R = I - RZ * P - 2
            CALL PrintPilastro(R, RZ, NP())
            PRINT "  ";
            CALL PrintPiano(P)
            FOR K = 1 TO 3
               IF V(K) = 0 THEN
                  PRINT #2, "         ----";
               ELSE
                  IF SM = 0 THEN
                     PRINT #2, USING "    ######.##"; F(I, K);
                  ELSE
                     PRINT #2, USING "    #####.###"; F(I, K);
                  END IF
               END IF
            NEXT K
            PRINT #2,
         END IF
      NEXT I
   END IF
   ' controlla se vi  squilibrio nei traversi
   Squil = 0
   FOR K = 1 TO KZ
      F = 0
      FOR I = 1 TO IZ
         IF N(I, 1) = K THEN
            F = F + F(I, 1)
         END IF
      NEXT I
      IF ABS(F) > .01 THEN
         Squil = 1
         EXIT FOR
      END IF
   NEXT K
   ' stampa l'eventuale squilibrio nei traversi
   IF Squil = 1 THEN
      PRINT #2, : PRINT #2,
      PRINT #2, "SQUILIBRIO NEI TRAVERSI ORIZZONTALI": PRINT #2,
      IF SM = 0 THEN
         PRINT #2, "traverso    squilibrio (kN)"
      ELSE
         PRINT #2, "traverso    squilibrio (t)"
      END IF
      PRINT #2,
      FOR K = KZ TO 1 STEP -1
         F = 0
         FOR I = 1 TO IZ
            IF N(I, 1) = K THEN
               F = F + F(I, 1)
            END IF
         NEXT I
         IF ABS(F) > .01 THEN
            IF SM = 0 THEN
               PRINT #2, USING "  ###       ######.##"; K; F
            ELSE
               PRINT #2, USING "  ###       #####.###"; K; F
            END IF
         END IF
      NEXT K
   END IF
RETURN

END SUB

' ================================ OutputTabSez ================================
'   Procedura per la stampa della tabella dei tipi di sezione
' ------------------------------------------------------------------------------
SUB OutputTabSez (DefTag, LZ, FS$(), ZB(), ZH(), ZA(), ZAT(), ZI())

   ZImax = 0
   FOR L = 1 TO LZ
      IF ZI(L) > ZImax THEN ZImax = ZI(L)
   NEXT L
   IF ZImax > .0005 THEN
      F1$ = " ###.####"
      F2$ = " ###.######"
   ELSE
      F1$ = " ##.#####"
      F2$ = " ##.########"
   END IF
   PRINT #2, : PRINT #2, : PRINT #2,
   PRINT #2, "TABELLA DEI TIPI DI SEZIONE": PRINT #2,
   PRINT #2, "                   b     h     area ";
   IF DefTag = 1 THEN PRINT #2, "   A/Chi ";
   PRINT #2, "   inerzia "
   PRINT #2, "sezione  forma     m     m      m2  ";
   IF DefTag = 1 THEN PRINT #2, "     m2  ";
   PRINT #2, "      m4   ": PRINT #2,
   FOR L = 1 TO LZ
      PRINT #2, USING " ###       !   "; L; FS$(L);
      IF FS$(L) = "R" THEN
         PRINT #2, USING " ##.## ##.##"; ZB(L); ZH(L);
      ELSE
         PRINT #2, "  ----  ----";
      END IF
      PRINT #2, USING F1$; ZA(L);
      IF DefTag = 1 THEN
         PRINT #2, USING F1$; ZAT(L);
      END IF
      PRINT #2, USING F2$; ZI(L)
   NEXT L

END SUB

' ================================ PrintCampata ================================
'
'   Procedura per la stampa dell'indicazione della campata (o sbalzo)
'
'   Variabili di ingresso:
'      F         numero d'ordine della campata
'      RZ        numero totale di pilastri
'      NP()      array che contiene la numerazione dei pilastri in carpenteria
'
' ------------------------------------------------------------------------------
SUB PrintCampata (F, RZ, NP())

   SELECT CASE F
      CASE 0
         PRINT #2, "  sb.sin";
      CASE 1 TO RZ - 1
         PRINT #2, USING "### -###"; NP(F); NP(F + 1);
      CASE RZ
         PRINT #2, "  sb.des";
   END SELECT

END SUB

' ================================ PrintPiano ==================================
'
'   Procedura per la stampa dell'indicazione del piano (o fondazione)
'
'   Variabili di ingresso:
'      P         numero d'ordine del piano (0=fondazione)
'
' ------------------------------------------------------------------------------
SUB PrintPiano (P)

   IF P = 0 THEN
      PRINT #2, "fond.";
   ELSE
      PRINT #2, USING " ##  "; P;
   END IF

END SUB

' ================================ PrintPilastro ===============================
'
'   Procedura per la stampa dell'indicazione del pilastro
'
'   Variabili di ingresso:
'      R         numero d'ordine del pilastro
'      RZ        numero totale di pilastri
'      NP()      array che contiene la numerazione dei pilastri in carpenteria
'
' ------------------------------------------------------------------------------
SUB PrintPilastro (R, RZ, NP())

   SELECT CASE R
      CASE 0
         PRINT #2, "estr.sin";
      CASE 1 TO RZ
         PRINT #2, USING " ####   "; NP(R);
      CASE RZ + 1
         PRINT #2, "estr.des";
   END SELECT

END SUB

' ================================ PrintSez ====================================
'
'  Procedura per la stampa di base, altezza ed inerzia della sezione L
'  e degli eventuali tratti rigidi
'
'  Variabili di ingresso:
'     L         numero d'ordine del tipo di sezione
'     FS$()     array che contiene il codice di forma delle sezioni
'     ZB()      array che contiene le basi
'     ZH()      array che contiene le altezze
'     ZI()      array che contiene i momenti di inerzia
'     TrRig, TrRigSin, TrRigDes
'
' ------------------------------------------------------------------------------
SUB PrintSez (L, FS$(), ZB(), ZH(), ZI(), TrRig, TrRigSin, TrRigDes)

   IF L = 0 THEN
      PRINT #2, USING "###  ----  ----  --------"; L;
   ELSEIF FS$(L) = "R" THEN
      PRINT #2, USING "### ##.## ##.## ##.######"; L; ZB(L); ZH(L); ZI(L);
   ELSEIF FS$(L) = "G" THEN
      PRINT #2, USING "###  ----  ---- ##.######"; L; ZI(L);
   END IF
   IF TrRig <> 0 THEN
      IF TrRigSin = 0 THEN
         PRINT #2, "    ----  ";
      ELSE
         PRINT #2, USING "   ##.##  "; TrRigSin;
      END IF
      IF TrRigDes = 0 THEN
         PRINT #2, "    ----  ";
      ELSE
         PRINT #2, USING "   ##.##  "; TrRigDes;
      END IF
   END IF

END SUB

' ================================ SommaModiRet ================================
'
'  Effettua la somma dei quadrati di valori relativi ai singoli modi.
'  Successivamente ne calcola la radice quadrata.
'
'  Variabili di ingresso:
'     Oper      operazione da effettuare:
'                    0 = dimensiona array inviluppo e somma quadrati
'                    1 = somma quadrati
'                    2 = effettua radice quadrata e la dispone negli array base
'     Wnkl      indica se  presente la trave di fondazione
'     RZ        numero di pilastri
'     PZ        numero di piani
'     KZ        numero di traversi
'     D()       spostamento orizzontale dei traversi per un modo
'     MS(), MD()  momento agli estremi delle travi per un modo
'     TS(), TD()  taglio agli estremi delle travi per un modo
'     MT(), MP()  momento agli estremi dei pilastri per un modo
'     TR(), SN()  taglio e sforzo normale nei pilastri per un modo
'     MFS(), MFD()  momento agli estremi delle travi di fond. per un modo
'     TFS(), TFD()  taglio agli estremi delle travi di fond. per un modo
'
'  Variabili interne:
'     InvD()              spostamento orizzontale dei traversi per tutti i modi
'     InvMS(), InvMD()    momento agli estremi delle travi per tutti i modi
'     InvTS(), InvTD()    taglio agli estremi delle travi per tutti i modi
'     InvMT(), InvMP()    momento agli estremi dei pilastri per tutti i modi
'     InvTR(), InvSN()    taglio e sforzo normale nei pilastri per tutti i modi
'     InvMFS(), InvMFD()  momento agli estremi travi fond. per tutti i modi
'     InvTFS(), InvTFD()  taglio agli estremi travi fond. per tutti i modi
'
' ------------------------------------------------------------------------------
SUB SommaModiRet (Oper, Wnkl, RZ, PZ, KZ, D(), MS(), MD(), TS(), TD(), MT(), MP(), TR(), SN(), MFS(), MFD(), TFS(), TFD())
  
   STATIC InvD(), InvMD(), InvMS()
   STATIC InvTD(), InvTS()
   STATIC InvMP(), InvMT(), InvTR(), InvSN()
   STATIC InvMFD(), InvMFS()
   STATIC InvTFD(), InvTFS()
   IF Oper = 0 THEN
      REDIM InvD(KZ), InvMD(RZ - 1, PZ), InvMS(RZ - 1, PZ)
      REDIM InvTD(RZ - 1, PZ), InvTS(RZ - 1, PZ)
      REDIM InvMP(RZ, PZ), InvMT(RZ, PZ), InvTR(RZ, PZ), InvSN(RZ, PZ)
      REDIM InvMFD(0 TO RZ), InvMFS(0 TO RZ)
      REDIM InvTFD(0 TO RZ), InvTFS(0 TO RZ)
   END IF
   IF Oper <= 1 THEN
      FOR K = 1 TO KZ
         InvD(K) = InvD(K) + D(K) ^ 2
      NEXT K
      FOR P = 1 TO PZ
         FOR R = 1 TO RZ - 1
            InvMS(R, P) = InvMS(R, P) + MS(R, P) ^ 2
            InvMD(R, P) = InvMD(R, P) + MD(R, P) ^ 2
            InvTS(R, P) = InvTS(R, P) + TS(R, P) ^ 2
            InvTD(R, P) = InvTD(R, P) + TD(R, P) ^ 2
         NEXT R
      NEXT P
      FOR P = 1 TO PZ
         FOR R = 1 TO RZ
            InvMT(R, P) = InvMT(R, P) + MT(R, P) ^ 2
            InvMP(R, P) = InvMP(R, P) + MP(R, P) ^ 2
            InvTR(R, P) = InvTR(R, P) + TR(R, P) ^ 2
            InvSN(R, P) = InvSN(R, P) + SN(R, P) ^ 2
         NEXT R
      NEXT P
      IF Wnkl = 1 THEN
         FOR R = 0 TO RZ
            InvMFS(R) = InvMFS(R) + MFS(R) ^ 2
            InvMFD(R) = InvMFD(R) + MFD(R) ^ 2
            InvTFS(R) = InvTFS(R) + TFS(R) ^ 2
            InvTFD(R) = InvTFD(R) + TFD(R) ^ 2
         NEXT R
      END IF
   ELSE
      FOR K = 1 TO KZ
         InvD(K) = SQR(InvD(K))
         D(K) = InvD(K)
      NEXT K
      FOR P = 1 TO PZ
         FOR R = 1 TO RZ - 1
            InvMS(R, P) = SQR(InvMS(R, P))
            InvMD(R, P) = SQR(InvMD(R, P))
            InvTS(R, P) = SQR(InvTS(R, P))
            InvTD(R, P) = SQR(InvTD(R, P))
            MS(R, P) = InvMS(R, P)
            MD(R, P) = InvMD(R, P)
            TS(R, P) = InvTS(R, P)
            TD(R, P) = InvTD(R, P)
         NEXT R
      NEXT P
      FOR P = 1 TO PZ
         FOR R = 1 TO RZ
            InvMT(R, P) = SQR(InvMT(R, P))
            InvMP(R, P) = SQR(InvMP(R, P))
            InvTR(R, P) = SQR(InvTR(R, P))
            InvSN(R, P) = SQR(InvSN(R, P))
            MT(R, P) = InvMT(R, P)
            MP(R, P) = InvMP(R, P)
            TR(R, P) = InvTR(R, P)
            SN(R, P) = InvSN(R, P)
         NEXT R
      NEXT P
      IF Wnkl = 1 THEN
         FOR R = 0 TO RZ
            InvMFS(R) = SQR(InvMFS(R))
            InvMFD(R) = SQR(InvMFD(R))
            InvTFS(R) = SQR(InvTFS(R))
            InvTFD(R) = SQR(InvTFD(R))
            MFS(R) = InvMFS(R)
            MFD(R) = InvMFD(R)
            TFS(R) = InvTFS(R)
            TFD(R) = InvTFD(R)
         NEXT R
      END IF
   END IF

END SUB

