'                                  SOLSIST.BAS
'
'          procedure per la soluzione dei sistemi di equazioni lineari
'
'  ver. 1.1                                                       rev. 09.93

' ================================ Risolve =====================================
'
'   Risoluzione di un sistema di equazioni col metodo di riduzione di Gauss
'       Seconda fase - risoluzione mediante sostituzione all'indietro
'
'       NOTA: la matrice dei coefficienti e` stata resa triangolare superiore
'             effettuando un pivotaggio parziale,  cioe` uno scambio di righe
'             per rendere massimo il termine sulla diagonale principale
'
'   Variabili di ingresso:
'      A(,)      matrice dei coefficienti delle incognite, triangolarizzata
'      B()       vettore termini noti
'      IPVT()    indice della riga del pivot K-esimo
'
'   Variabili di uscita:
'      B()       vettore soluzione
'
'   Variabili interne principali:
'      N         ordine della matrice dei coefficienti
'
' ------------------------------------------------------------------------------
'
SUB Risolve (A(), B(), IPVT())

    ' determina il numero di elementi degli array A, B ed IPVT
    LRA = LBOUND(A, 1)
    LCA = LBOUND(A, 2)
    NRA = UBOUND(A, 1)
    NCA = UBOUND(A, 2)
    LRB = LBOUND(B, 1)
    NRB = UBOUND(B, 1)
    LRI = LBOUND(IPVT, 1)
    NRI = UBOUND(IPVT, 1)

    ' controlla che le dimensioni degli array A, B ed IPVT siano corrette
    IF LRA > 1 OR LCA > 1 OR LRB > 1 OR LRI > 1 THEN
       PRINT "ERRORE 1 in Risolve - limiti inferiori di A, B o IPVT maggiori di 1"
       END
    END IF
    IF NRA <> NCA THEN
       PRINT "ERRORE 2 in Risolve - matrice dei coefficienti A non quadrata"
       END
    END IF
    IF NRB < NRA OR NRI < NRA - 1 THEN
       PRINT "ERRORE 3 in Risolve - array B o IPVT insufficiente"
       END
    END IF

    ' definisce N, ordine della matrice dei coefficienti
    N = NRA

    ' ripete su B le operazioni fatte per triangolarizzare A
    ' NOTA: il ciclo non viene effettuato nel caso che N = 1
    FOR K = 1 TO N - 1
       SWAP B(K), B(IPVT(K))
       FOR I = K + 1 TO N
          B(I) = B(I) - A(I, K) * B(K)
       NEXT I
    NEXT K
    ' effettua la sostituzione all'indietro per tutte le incognite
    FOR K = N TO 1 STEP -1
       B(K) = B(K) / A(K, K)
       ' NOTA: il ciclo non viene effettuato nel caso che K = 1
       FOR I = 1 TO K - 1
          B(I) = B(I) - A(I, K) * B(K)
       NEXT I
    NEXT K

END SUB

' ================================ Risolve.K1 ==================================
'
'   Risoluzione di un sistema di equazioni col metodo di riduzione di Gauss
'       Seconda fase - risoluzione mediante sostituzione all'indietro
'
'       NOTE: la matrice dei coefficienti K e` divisa in 4: KTT, KTA, KAT, KAA
'             la matrice dei coefficienti e` simmetrica
'             la matrice dei coefficienti e` a banda (in array monodimensionale)
'             la matrice K e` stata resa triangolare inferiore, senza pivotaggio
'
'   Variabili di ingresso:
'      KTT(,)    matrice dei coefficienti, triangolarizzata superiormente
'      KTA(,)
'      KAA()        "     "       "      , a banda (dimensione NAxSB-SBx(SB-1)/2)
'      SB        ampiezza della semibanda (compreso elemento diagonale)
'      RID       variabile di controllo del procedimento di risoluzione:
'                   RID = 0     triangolarizz. termini noti e sostituz. indietro
'                   RID = 1     triangolarizz. termini noti, esclusi i primi NT
'                   RID = 2     sostituzione indietro esclusi i primi NT termini
'      B()       vettore termini noti
'
'   Variabili di uscita:
'      B()       vettore soluzione
'
'   Variabili interne principali:
'      NT        ordine della matrice KTT
'      NA        ordine della matrice KAA
'      N         ordine della matrice dei coefficienti: N=NT+NA
'
' ------------------------------------------------------------------------------
'
SUB Risolve.K1 (KTT(), KTA(), KAA(), SB, RID, B())

    ' determina il numero di elementi degli array KTT, KTA, KAA, B
    LRTT = LBOUND(KTT, 1)
    LCTT = LBOUND(KTT, 2)
    NRTT = UBOUND(KTT, 1)
    NCTT = UBOUND(KTT, 2)
    LRTA = LBOUND(KTA, 1)
    LCTA = LBOUND(KTA, 2)
    NRTA = UBOUND(KTA, 1)
    NCTA = UBOUND(KTA, 2)
    LRAA = LBOUND(KAA, 1)
    NRAA = UBOUND(KAA, 1)
    LRB = LBOUND(B, 1)
    NRB = UBOUND(B, 1)

    ' controlla che le dimensioni degli array KTT, KTA, KAA siano corrette
    ' definisce NT, NA, N, ordine delle matrici dei coefficienti
    IF LRTT > 1 OR LCTT > 1 OR LRTA > 1 OR LCTA > 1 OR LRAA > 1 THEN
       PRINT "ERRORE 1 in Risolve.K1 - limiti inferiori di KTT, KTA o KAA maggiori di 1"
       END
    END IF
    IF NRTT <> NCTT THEN
       PRINT "ERRORE 2 in Risolve.K1 - matrice KTT non quadrata"
       END
    END IF
    NT = NRTT
    NA = NCTA
    N = NT + NA
    IF NRAA <> NA * SB - SB * (SB - 1) / 2 THEN
       PRINT "ERRORE 3 in Risolve.K1 - dimensioni di KAA non corrette"
       END
    END IF
    IF NRTA <> NT THEN
       PRINT "ERRORE 4 in Risolve.K1 - dimensioni di KTA non corrette"
       END
    END IF
    IF LRB > 1 THEN
       PRINT "ERRORE 5 in Risolve.K1 - limiti inferiori di B maggiori di 1"
       END
    END IF
    IF NRB < N THEN
       PRINT "ERRORE 6 in Risolve.K1 - array B insufficiente"
       END
    END IF

    SELECT CASE RID
       CASE 0
          GOSUB TriangBa.K1
          GOSUB TriangBt.K1
          GOSUB DeterminaBt.K1
          GOSUB DeterminaBa.K1
       CASE 1
          GOSUB TriangBa.K1
       CASE 2
          GOSUB DeterminaBa.K1
    END SELECT
EXIT SUB

TriangBa.K1:
    ' ripete sulla seconda parte di B le operazioni fatte per triang. KAA e KTT
    FOR K = NA TO 1 STEP -1
       NK = NT + K
       C = B(NK)
       B(NK) = B(NK) / KAA(K)
       ' per l'azzeramento dei termini in KAA
       IF K < SB THEN LB = K ELSE LB = SB
       II = K - LB + 1
       DP = NA
       P = K
       FOR I = K - 1 TO II STEP -1
          NI = NT + I
          DP = DP - 1
          P = P + DP
          B(NI) = B(NI) - KAA(P) * C
       NEXT I
       ' per l'azzeramento dei termini in KTA
       FOR I = 1 TO NT
          B(I) = B(I) - KTA(I, K) * C
       NEXT I
    NEXT K
RETURN

TriangBt.K1: 
    ' ripete sulla prima parte di B le operazioni fatte per triang. KAA e KTT
    FOR K = NT TO 1 STEP -1
       C = B(K)
       B(K) = B(K) / KTT(K, K)
       ' per l'azzeramento dei termini in KTT
       FOR I = 1 TO K - 1
          B(I) = B(I) - KTT(I, K) * C
       NEXT I
    NEXT K
RETURN

DeterminaBt.K1:
    ' effettua la sostituzione all'indietro per le prime NT incognite
    FOR K = 1 TO NT
       FOR I = K + 1 TO NT
          B(I) = B(I) - KTT(K, I) * B(K)
       NEXT I
    NEXT K
RETURN

DeterminaBa.K1:
    ' effettua la sostituzione all'indietro per le altre NA incognite
    FOR K = 1 TO NT
       FOR I = 1 TO NA
          NI = NT + I
          B(NI) = B(NI) - KTA(K, I) * B(K)
       NEXT I
    NEXT K
    FOR K = 1 TO NA
       NK = NT + K
       IF NA - K + 1 < SB THEN LB = NA - K + 1 ELSE LB = SB
       DP = NA
       P = K
       FOR I = K + 1 TO K + LB - 1
          NI = NT + I
          P = P + DP
          DP = DP - 1
          B(NI) = B(NI) - KAA(P) * B(NK)
       NEXT I
    NEXT K
RETURN

END SUB

' ================================ Risolve.K2 ==================================
'
'   Risoluzione di un sistema di equazioni col metodo di riduzione di Gauss
'       Seconda fase - risoluzione mediante sostituzione all'indietro
'
'       NOTE: la matrice dei coefficienti K e` divisa in 4: KTT, KTA, KAT, KAA
'             la matrice dei coefficienti e` simmetrica
'             la matrice dei coefficienti e` a banda (in array bidimensionale)
'             la matrice K e` stata resa triangolare inferiore, senza pivotaggio
'
'   Variabili di ingresso:
'      KTT(,)    matrice dei coefficienti, triangolarizzata superiormente
'      KTA(,)
'      KAA(,)       "     "       "      , a banda (dimensioni NA,SB)
'      SB        ampiezza della semibanda (compreso elemento diagonale)
'      RID       variabile di controllo del procedimento di risoluzione:
'                   RID = 0     triangolarizz. termini noti e sostituz. indietro
'                   RID = 1     triangolarizz. termini noti, esclusi i primi NT
'                   RID = 2     sostituzione indietro esclusi i primi NT termini
'      B()       vettore termini noti
'
'   Variabili di uscita:
'      B()       vettore soluzione
'
'   Variabili interne principali:
'      NT        ordine della matrice KTT
'      NA        ordine della matrice KAA
'      N         ordine della matrice dei coefficienti: N=NT+NA
'
' ------------------------------------------------------------------------------
'
SUB Risolve.K2 (KTT(), KTA(), KAA(), SB, RID, B())

    ' determina il numero di elementi degli array KTT, KTA, KAA, B
    LRTT = LBOUND(KTT, 1)
    LCTT = LBOUND(KTT, 2)
    NRTT = UBOUND(KTT, 1)
    NCTT = UBOUND(KTT, 2)
    LRTA = LBOUND(KTA, 1)
    LCTA = LBOUND(KTA, 2)
    NRTA = UBOUND(KTA, 1)
    NCTA = UBOUND(KTA, 2)
    LRAA = LBOUND(KAA, 1)
    LCAA = LBOUND(KAA, 2)
    NRAA = UBOUND(KAA, 1)
    NCAA = UBOUND(KAA, 2)
    LRB = LBOUND(B, 1)
    NRB = UBOUND(B, 1)

    ' controlla che le dimensioni degli array KTT, KTA, KAA siano corrette
    ' definisce NT, NA, N, ordine delle matrici dei coefficienti
    IF LRTT > 1 OR LCTT > 1 OR LRTA > 1 OR LCTA > 1 OR LRAA > 1 OR LCAA > 1 THEN
       PRINT "ERRORE 1 in Risolve.K2 - limiti inferiori di KTT, KTA o KAA maggiori di 1"
       END
    END IF
    IF NRTT <> NCTT THEN
       PRINT "ERRORE 2 in Risolve.K2 - matrice KTT non quadrata"
       END
    END IF
    NT = NRTT
    NA = NCTA
    N = NT + NA
    IF NRAA <> NA OR NCAA <> SB THEN
       PRINT "ERRORE 3 in Risolve.K2 - dimensioni di KAA non corrette"
       END
    END IF
    IF NRTA <> NT THEN
       PRINT "ERRORE 4 in Risolve.K2 - dimensioni di KTA non corrette"
       END
    END IF
    IF LRB > 1 THEN
       PRINT "ERRORE 5 in Risolve.K2 - limiti inferiori di B maggiori di 1"
       END
    END IF
    IF NRB < N THEN
       PRINT "ERRORE 6 in Risolve.K2 - array B insufficiente"
       END
    END IF

    SELECT CASE RID
       CASE 0
          GOSUB TriangBa.K2
          GOSUB TriangBt.K2
          GOSUB DeterminaBt.K2
          GOSUB DeterminaBa.K2
       CASE 1
          GOSUB TriangBa.K2
       CASE 2
          GOSUB DeterminaBa.K2
    END SELECT
EXIT SUB

TriangBa.K2:
    ' ripete sulla seconda parte di B le operazioni fatte per triang. KAA e KTT
    FOR K = NA TO 1 STEP -1
       NK = NT + K
       C = B(NK)
       B(NK) = B(NK) / KAA(K, 1)
       ' per l'azzeramento dei termini in KAA
       IF K < SB THEN LB = K ELSE LB = SB
       II = K - LB + 1
       FOR I = K - 1 TO II STEP -1
          D = K - I
          NI = NT + I
          B(NI) = B(NI) - KAA(I, D + 1) * C
       NEXT I
       ' per l'azzeramento dei termini in KTA
       FOR I = 1 TO NT
          B(I) = B(I) - KTA(I, K) * C
       NEXT I
    NEXT K
RETURN

TriangBt.K2:
    ' ripete sulla prima parte di B le operazioni fatte per triang. KAA e KTT
    FOR K = NT TO 1 STEP -1
       C = B(K)
       B(K) = B(K) / KTT(K, K)
       ' per l'azzeramento dei termini in KTT
       FOR I = 1 TO K - 1
          B(I) = B(I) - KTT(I, K) * C
       NEXT I
    NEXT K
RETURN

DeterminaBt.K2:
    ' effettua la sostituzione all'indietro per le prime NT incognite
    FOR K = 1 TO NT
       FOR I = K + 1 TO NT
          B(I) = B(I) - KTT(K, I) * B(K)
       NEXT I
    NEXT K
RETURN

DeterminaBa.K2:
    ' effettua la sostituzione all'indietro per le altre NA incognite
    FOR K = 1 TO NT
       FOR I = 1 TO NA
          NI = NT + I
          B(NI) = B(NI) - KTA(K, I) * B(K)
       NEXT I
    NEXT K
    FOR K = 1 TO NA
       NK = NT + K
       IF NA - K + 1 < SB THEN LB = NA - K + 1 ELSE LB = SB
       FOR I = K + 1 TO K + LB - 1
          D = K - I
          NI = NT + I
          B(NI) = B(NI) - KAA(K, 1 - D) * B(NK)
       NEXT I
    NEXT K
RETURN

END SUB

' ================================ Triang ======================================
'
'   Risoluzione di un sistema di equazioni col metodo di riduzione di Gauss
'       Prima fase - triangolarizzazione della matrice dei coefficienti
'
'       NOTA: la matrice dei coefficienti  viene resa  triangolare superiore
'             effettuando un pivotaggio parziale, cioe` uno scambio di righe
'             per rendere massimo il termine sulla diagonale principale
'
'   Variabili di ingresso:
'      A(,)      matrice dei coefficienti delle incognite, da triangolarizzare
'
'   Variabili di uscita:
'      A(,)      matrice dei coefficienti, triangolarizzata superiormente
'                NOTA: nel triangolo inferiore sono conservati i fattori
'                      usati per l'azzeramento
'      IPVT()    indice della riga del pivot K-esimo
'
'   Variabili interne principali:
'      N         ordine della matrice dei coefficienti
'
' ------------------------------------------------------------------------------
'
SUB Triang (A(), IPVT())
 
    ' determina il numero di elementi degli array A ed IPVT
    LRA = LBOUND(A, 1)
    LCA = LBOUND(A, 2)
    NRA = UBOUND(A, 1)
    NCA = UBOUND(A, 2)
    LRI = LBOUND(IPVT, 1)
    NRI = UBOUND(IPVT, 1)

    ' controlla che le dimensioni degli array A ed IPVT siano corrette
    IF LRA > 1 OR LCA > 1 OR LRI > 1 THEN
       PRINT "ERRORE 1 in Triang - limiti inferiori di A o IPVT maggiori di 1"
       END
    END IF
    IF NRA <> NCA THEN
       PRINT "ERRORE 2 in Triang - matrice dei coefficienti A non quadrata"
       END
    END IF
    IF NRI < NRA - 1 THEN
       PRINT "ERRORE 3 in Triang - array IPVT insufficiente"
       END
    END IF
 
    ' definisce N, ordine della matrice dei coefficienti
    N = NRA
 
    ' triangolarizzazione della matrice A
    ' NOTA: il ciclo non viene effettuato nel caso che N = 1
    FOR K = 1 TO N - 1
       ' cerca il pivot per la riga K
       M = K
       FOR I = K + 1 TO N
          IF ABS(A(I, K)) > ABS(A(M, K)) THEN M = I
       NEXT I
       IPVT(K) = M
       SWAP A(M, K), A(K, K)
       IF A(K, K) = 0 THEN
          PRINT "ERRORE 4 in Triang - matrice dei coefficienti A singolare"
          END
       END IF
       ' determina i coefficienti moltiplicativi delle righe
       FOR I = K + 1 TO N
          A(I, K) = A(I, K) / A(K, K)
       NEXT I
       ' scambia le righe ed azzera tutti gli elementi della colonna
       FOR J = K + 1 TO N
          SWAP A(M, J), A(K, J)
          IF A(K, J) <> 0 THEN
             FOR I = K + 1 TO N
                A(I, J) = A(I, J) - A(I, K) * A(K, J)
             NEXT I
          END IF
       NEXT J
    NEXT K

END SUB

' ================================ Triang.K1 ===================================
'
'   Risoluzione di un sistema di equazioni col metodo di riduzione di Gauss
'       Prima fase - triangolarizzazione della matrice dei coefficienti
'
'       NOTE: la matrice dei coefficienti K e` divisa in 4: KTT, KTA, KAT, KAA
'             la matrice dei coefficienti e` simmetrica
'             la matrice dei coefficienti e` a banda (in array monodimensionale)
'             la matrice K viene resa triangolare inferiore, senza pivotaggio
'
'             Il procedimento di riduzione puo` essere limitato alla KAA e KTA,
'             per ottenere in KTT la matrice di rigidezza laterale
'
'   Variabili di ingresso:
'      KTT(,)    matrice dei coefficienti delle incognite, da triangolarizzare
'      KTA(,)       "     "       "
'      KAA()        "     "       "     , a banda (dimensione NAxSB-SBx(SB-1)/2)
'      SB        ampiezza della semibanda (compreso elemento diagonale)
'      RID       variabile di controllo del procedimento di riduzione:
'                   RID = 0     vengono triangolarizzate KTT e KAA
'                   RID = 1     viene triangolarizzata solo KAA
'
'   Variabili di uscita:
'      KTT(,)    matrice dei coefficienti, triangolarizzata inferiormente
'      KTA(,)       "     "       "
'      KAA()        "     "       "
'
'   Variabili interne principali:
'      NT        ordine della matrice KTT
'      NA        ordine della matrice KAA
'
' ---------------------------------------------------------------------------
'
SUB Triang.K1 (KTT(), KTA(), KAA(), SB, RID)

    ' determina il numero di elementi degli array KTT, KTA, KAA
    LRTT = LBOUND(KTT, 1)
    LCTT = LBOUND(KTT, 2)
    NRTT = UBOUND(KTT, 1)
    NCTT = UBOUND(KTT, 2)
    LRTA = LBOUND(KTA, 1)
    LCTA = LBOUND(KTA, 2)
    NRTA = UBOUND(KTA, 1)
    NCTA = UBOUND(KTA, 2)
    LRAA = LBOUND(KAA, 1)
    NRAA = UBOUND(KAA, 1)

    ' controlla che le dimensioni degli array KTT, KTA, KAA siano corrette
    ' definisce NT, NA, ordine delle matrici dei coefficienti
    IF LRTT > 1 OR LCTT > 1 OR LRTA > 1 OR LCTA > 1 OR LRAA > 1 THEN
       PRINT "ERRORE 1 in Triang.K1 - limiti inferiori di KTT, KTA o KAA maggiori di 1"
       END
    END IF
    IF NRTT <> NCTT THEN
       PRINT "ERRORE 2 in Triang.K1 - matrice KTT non quadrata"
       END
    END IF
    NT = NRTT
    NA = NCTA
    IF NRAA <> NA * SB - SB * (SB - 1) / 2 THEN
       PRINT "ERRORE 3 in Triang.K1 - dimensioni di KAA non corrette"
       END
    END IF
    IF NRTA <> NT THEN
       PRINT "ERRORE 4 in Triang.K1 - dimensioni di KTA non corrette"
       END
    END IF
 
    ' triangolarizzazione della matrice dei coefficienti
    ' prima fase: triangolarizzazione di KAA
    FOR K = NA TO 1 STEP -1
       IF KAA(K) = 0 THEN
          PRINT "ERRORE 5 in Triang.K1 - valore 0 nella diagonale principale di KAA"
          END
       END IF
       ' ciclo di azzeramento per le righe di KAT e KAA
       IF K < SB THEN LB = K ELSE LB = SB
       II = K - LB + 1
       DP = NA
       P = K
       FOR I = K - 1 TO II STEP -1
          DP = DP - 1
          P = P + DP
          IF KAA(P) <> 0 THEN
             ' determina il coefficiente moltiplicativo relativo alla riga I
             C = KAA(P) / KAA(K)
             ' modifica tutti gli elementi della riga I
             FOR J = 1 TO NT
                KTA(J, I) = KTA(J, I) - KTA(J, K) * C
             NEXT J
             DM = NA
             M = I
             DQ = DP
             Q = P
             FOR J = I TO II STEP -1
                KAA(M) = KAA(M) - KAA(Q) * C
                DM = DM - 1
                M = M + DM
                DQ = DQ - 1
                Q = Q + DQ
             NEXT J
             KAA(P) = C
          END IF
       NEXT I
       ' ciclo di azzeramento per le righe di KTT e KTA
       FOR I = NT TO 1 STEP -1
          IF KTA(I, K) <> 0 THEN
             ' determina il coefficiente moltiplicativo relativo alla riga I
             C = KTA(I, K) / KAA(K)
             ' modifica tutti gli elementi della riga I
             FOR J = 1 TO I
                KTT(J, I) = KTT(J, I) - KTA(J, K) * C
             NEXT J
             KTA(I, K) = C
          END IF
       NEXT I
    NEXT K
              
    ' triangolarizzazione della matrice dei coefficienti
    ' seconda fase: triangolarizzazione di KTT
    IF RID <> 1 THEN
       FOR K = NT TO 1 STEP -1
          IF KTT(K, K) = 0 THEN
             PRINT "ERRORE 6 in Triang.K1 - valore 0 nella diagonale principale di KTT"
             END
          END IF
          ' ciclo di azzeramento per le righe di KTT
          FOR I = K - 1 TO 1 STEP -1
             IF KTT(I, K) <> 0 THEN
                ' determina il coefficiente moltiplicativo relativo alla riga I
                C = KTT(I, K) / KTT(K, K)
                ' modifica tutti gli elementi della riga I
                FOR J = 1 TO I
                   KTT(J, I) = KTT(J, I) - KTT(J, K) * C
                NEXT J
                KTT(I, K) = C
             END IF
          NEXT I
       NEXT K
    END IF
               
END SUB

' ================================ Triang.K2 ===================================
'
'   Risoluzione di un sistema di equazioni col metodo di riduzione di Gauss
'       Prima fase - triangolarizzazione della matrice dei coefficienti
'
'       NOTE: la matrice dei coefficienti K e` divisa in 4: KTT, KTA, KAT, KAA
'             la matrice dei coefficienti e` simmetrica
'             la matrice dei coefficienti e` a banda (in array bidimensionale)
'             la matrice K viene resa triangolare inferiore, senza pivotaggio
'
'             Il procedimento di riduzione pu essere limitato alla KAA e KTA,
'             per ottenere in KTT la matrice di rigidezza laterale; oppure, in
'             una fase successiva, pu essere effettuato solo su KTT
'
'   Variabili di ingresso:
'      KTT(,)    matrice dei coefficienti delle incognite, da triangolarizzare
'      KTA(,)       "     "       "
'      KAA(,)       "     "       "      , a banda (dimensioni NA,SB)
'      SB        ampiezza della semibanda (compreso elemento diagonale)
'      RID       variabile di controllo del procedimento di riduzione:
'                   RID = 0     vengono triangolarizzate KTT e KAA
'                   RID = 1     viene triangolarizzata solo KAA
'                   RID = 2     viene triangolarizzata solo KTT
'
'   Variabili di uscita:
'      KTT(,)    matrice dei coefficienti, triangolarizzata inferiormente
'      KTA(,)       "     "       "
'      KAA(,)       "     "       "
'
'   Variabili interne principali:
'      NT        ordine della matrice KTT
'      NA        ordine della matrice KAA
'
' ------------------------------------------------------------------------------
'
SUB Triang.K2 (KTT(), KTA(), KAA(), SB, RID)

   ' determina il numero di elementi degli array KTT, KTA, KAA
   LRTT = LBOUND(KTT, 1)
   LCTT = LBOUND(KTT, 2)
   NRTT = UBOUND(KTT, 1)
   NCTT = UBOUND(KTT, 2)
   LRTA = LBOUND(KTA, 1)
   LCTA = LBOUND(KTA, 2)
   NRTA = UBOUND(KTA, 1)
   NCTA = UBOUND(KTA, 2)
   LRAA = LBOUND(KAA, 1)
   LCAA = LBOUND(KAA, 2)
   NRAA = UBOUND(KAA, 1)
   NCAA = UBOUND(KAA, 2)
  
   ' controlla che le dimensioni degli array KTT, KTA, KAA siano corrette
   ' definisce NT, NA, ordine delle matrici dei coefficienti
   IF LRTT > 1 OR LCTT > 1 OR LRTA > 1 OR LCTA > 1 OR LRAA > 1 OR LCAA > 1 THEN
      PRINT "ERRORE 1 in Triang.K2 - limiti inferiori di KTT, KTA o KAA maggiori di 1"
      END
   END IF
   IF NRTT <> NCTT THEN
      PRINT "ERRORE 2 in Triang.K2 - matrice KTT non quadrata"
      END
   END IF
   NT = NRTT
   NA = NCTA
   IF NRAA <> NA OR NCAA <> SB THEN
      PRINT "ERRORE 3 in Triang.K2 - dimensioni di KAA non corrette"
      END
   END IF
   IF NRTA <> NT THEN
      PRINT "ERRORE 4 in Triang.K2 - dimensioni di KTA non corrette"
      END
   END IF

   ' triangolarizzazione della matrice dei coefficienti
   ' prima fase: triangolarizzazione di KAA
   IF RID <> 2 THEN
      FOR K = NA TO 1 STEP -1
         IF KAA(K, 1) = 0 THEN
            PRINT "ERRORE 5 in Triang.K2 - valore 0 nella diagonale principale di KAA"
            END
         END IF
         ' ciclo di azzeramento per le righe di KAT e KAA
         IF K < SB THEN LB = K ELSE LB = SB
         II = K - LB + 1
         FOR I = K - 1 TO II STEP -1
            D = K - I
            IF KAA(I, D + 1) <> 0 THEN
               ' determina il coefficiente moltiplicativo relativo alla riga I
               C = KAA(I, D + 1) / KAA(K, 1)
               ' modifica tutti gli elementi della riga I
               FOR J = 1 TO NT
                  KTA(J, I) = KTA(J, I) - KTA(J, K) * C
               NEXT J
               FOR J = II TO I
                  M = I - J + 1
                  KAA(J, M) = KAA(J, M) - KAA(J, M + D) * C
               NEXT J
               KAA(I, D + 1) = C
            END IF
         NEXT I
         ' ciclo di azzeramento per le righe di KTT e KTA
         FOR I = NT TO 1 STEP -1
            IF KTA(I, K) <> 0 THEN
               ' determina il coefficiente moltiplicativo relativo alla riga I
               C = KTA(I, K) / KAA(K, 1)
               ' modifica tutti gli elementi della riga I
               FOR J = 1 TO I
                  KTT(J, I) = KTT(J, I) - KTA(J, K) * C
               NEXT J
               KTA(I, K) = C
            END IF
         NEXT I
      NEXT K
   END IF
               
   ' triangolarizzazione della matrice dei coefficienti
   ' seconda fase: triangolarizzazione di KTT
   IF RID <> 1 THEN
      FOR K = NT TO 1 STEP -1
         IF KTT(K, K) = 0 THEN
            PRINT "ERRORE 6 in Triang.K2 - valore 0 nella diagonale principale di KTT"
            END
         END IF
         ' ciclo di azzeramento per le righe di KTT
         FOR I = K - 1 TO 1 STEP -1
            IF KTT(I, K) <> 0 THEN
               ' determina il coefficiente moltiplicativo relativo alla riga I
               C = KTT(I, K) / KTT(K, K)
               ' modifica tutti gli elementi della riga I
               FOR J = 1 TO I
                  KTT(J, I) = KTT(J, I) - KTT(J, K) * C
               NEXT J
               KTT(I, K) = C
            END IF
         NEXT I
      NEXT K
   END IF
               
END SUB

