'                                  TELGEN.BAS
'
'  programma per il calcolo di strutture piane intelaiate di forma qualsiasi;
'  consente anche:
'  - analisi modale
'  - analisi con carico incrementale, aggiornando la matrice di rigidezza
'    (facendo variare E) alla fine di ogni passo di carico
'
'  derivato da TELGEN ver 1.5 rev. 12.95
'
'  tipi di aste:        T  =  trave alla De Saint Venant, incastrata ai nodi
'                       C1 =  trave con cerniera al primo estremo
'                       C2 =  trave con cerniera al secondo estremo
'                       P  =  pendolo
'                       TR =  (o R) trave con tratti rigidi all'estemit
'                       W  =  trave su suolo elastico alla Winkler
'                       WR =  trave su suolo elastico con tratti rigidi
'
'  vincoli esterni:     vincoli rigidi o elasticamente cedevoli
'                       vincoli mutui allo spostamento orizzontale
'
'  carichi:             azioni concentrate sui nodi
'                       carichi uniformemente distribuiti sulle aste
'
'  ver. 2.0                                                       rev. 02.98


' ------------------ Dichiarazione delle procedure utilizzate ------------------

' procedure del file SOLSIST.BAS
DECLARE SUB Triang.K2 (KTT!(), KTA!(), KAA!(), SB!, RID!)
DECLARE SUB Risolve.K2 (KTT!(), KTA!(), KAA!(), SB!, RID!, B!())

' procedure del file MATRIX.BAS
DECLARE SUB SottraeMat (A!(), B!(), C!())

' procedure del file PROCOM.BAS
DECLARE SUB Aspetta (Sec!)
DECLARE SUB CarSoll (DefTag!, TrRig!, JZ!, X!(), Y!(), ZA!(), ZAT!(), ZI!(), Easta!(), Gasta!(), TA$(), Estr!(), NT!(), K!(), B!(), TrRig!(), N!(), P!(), F!(), R!(), N1!(), T1!(), M1!(), N2!(), T2!(), M2!())
DECLARE SUB DefinisceIO (Prog$, FI$, FO$, FT$)
DECLARE SUB DefinisceMO (Prog$, DefEst!, DefTag!, Contr!)
DECLARE SUB DefinisceSM (SM!)
DECLARE SUB DisegnaSchema (IZ!, JZ!, X!(), Y!(), TA$(), Estr!())
DECLARE SUB FormaMat (NumNodi!, NumEstr!(), N!(), Kg!(), NL!, SB!, KTT!(), KTA!(), KAA!())
DECLARE SUB Incognite (IZ!, JZ!, KZ!, TA$(), Estr!(), V$(), NI!(), N!(), NL!, NZ!, SB!)
DECLARE SUB InputIntestaz (Contr!, Intest$)
DECLARE SUB InputModElast (Contr!, SM!, EL!, GL!, E!, G!)
DECLARE SUB InputSezioni (Contr!, LZ!, FS$(), ZB!(), ZH!(), ZA!(), ZI!(), ZAT!(), ER!)
DECLARE SUB MatAstaGlobale (TA$, DefTag!, X1!, X2!, Y1!, Y2!, A!, AT!, Iner!, K!, B!, TrRigSin!, TrRigDes!, E!, G!, Lun!, SnAl!, CsAl!, Ld!(), Kg!())
DECLARE SUB OutputIntestaz (NomeProg$, FI$, Intest$)
DECLARE SUB OutputModElast (SM!, DefTag!, EL!, GL!)
DECLARE SUB TermNoto (TrRig!, IZ!, JZ!, X!(), Y!(), ZA!(), ZI!(), E!, TA$(), Estr!(), NT!(), K!(), B!(), TrRig!(), N!(), P!(), F!(), R!())
DECLARE SUB VincoliCed (IZ!, N!(), KV!(), NL!, SB!, KTT!(), KTA!(), KAA!())

' procedure del file PROGEN.BAS
DECLARE SUB InputAsteGen (Incr!, Contr!, JZ!, LZ!, Estr!(), NT!(), TA$(), Wnkl!, B!(), K!(), TrRig!, TrRig!(), Ates!(), Acom!(), ER!)
DECLARE SUB InputCarGen (Incr!, MoAn!, Contr!, IZ!, JZ!, QN!, QA!, nMod!, F!(), P!(), nPassi!, PassoOut!, ER!)
DECLARE SUB InputIndGen (IZ!, JZ!, KZ!, LZ!)
DECLARE SUB InputMasseGen (Contr!, KZ!, Mas!())
DECLARE SUB InputNodiGen (Contr!, IZ!, X!(), Y!(), V$(), VCed!, KV!(), ER!)
DECLARE SUB InputTraversiGen (Contr!, KZ!, NI!())
DECLARE SUB OutputAsteGen (Incr!, SM!, DefTag!, Wnkl!, TrRig!, JZ!, NT!(), TA$(), Estr!(), FS$(), ZB!(), ZH!(), ZA!(), ZAT!(), ZI!(), B!(), K!(), TrRig!(), Ates!(), Acom!())
DECLARE SUB OutputCarGen (SM!, CAR!, IZ!, JZ!, QN!, QA!, F!(), P!())
DECLARE SUB OutputMasseGen (KZ!, Mas!())
DECLARE SUB OutputNodiGen (IZ!, X!(), Y!(), V$(), VCed!, KV!())
DECLARE SUB OutputRisGen (SM!, Outp!, IZ!, JZ!, KZ!, NL!, V$(), KV!(), NI!(), N!(), R!(), M1!(), M2!(), T1!(), T2!(), N1!(), N2!(), F!())
DECLARE SUB SommaModiGen (Oper!, JZ!, NZ!, R!(), M1!(), M2!(), T1!(), T2!(), N1!(), N2!())

' procedure del file MNLIMITE.BAS
DECLARE SUB CalcolaSquilibrio (JZ!, X!(), Y!(), TA$(), Estr!(), F!(), N1!(), T1!(), M1!(), N2!(), T2!(), M2!())
DECLARE SUB Curv (B!, H!, Ates!, Acom!, E!, ec1!, ecu!, sigmac!, Es!, Ey!, esy!, esu!, Nmed!, Mlim1!, Mlim2!, Rid1!, Rid2!)
DECLARE SUB InputMateriale (ec1!, ecu!, sigmac!, Es!, Ey!, esy!, esu!)
DECLARE SUB OutputMateriale (ec1!, ecu!, sigmac!, Es!, Ey!, esy!, esu!)

' procedure del file MOANA.BAS
DECLARE SUB AnalisiModale (Klat!(), MA!(), KZ!, nMod!, EigVal!(), EigVet!(), T!(), Part!(), Forze!(), Energia!())
DECLARE SUB InputSpettro (Contr!, SpePar!(), SpeT!(), SpeR!())
DECLARE SUB OutputModi (SM!, KZ!, j!, EigVal!(), EigVet!(), T!(), Beta!(), Part!(), Forza!(), R!(), EnIso!, EnTot!, Smorz!)
DECLARE SUB OutputSpettro (SpePar!(), SpeT!(), SpeR!())
DECLARE FUNCTION Risposta! (T!, SpePar!())
DECLARE FUNCTION Risposta2! (T!, nCop!, SpeT!(), SpeR!())

CLEAR , , 2000

' ------------------ Dimensionamento delle variabili con indice ----------------

OPTION BASE 1            ' definisce l'estremo inferiore dell'indice degli array
DIM Ld(6, 6), Kg(6, 6)   ' array statici
                         ' array dinamici: sono dimensionati in apposite routine


' --------------------------- Programma principale -----------------------------

GOSUB Presentazione
GOSUB Opzioni
CALL DefinisceIO("GEN", FI$, FO$, FT$)
CALL DefinisceMO("GEN", DefEst, DefTag, Contr)
CALL DefinisceSM(SM)
OPEN FI$ FOR INPUT AS #1
OPEN FO$ FOR OUTPUT AS #2
GOSUB LeggeGeomGen
GOSUB StampaGeomGen
GOSUB Schema
GOSUB DefinisceIncognite
IF Incr = 0 THEN
   ' analisi standard
   GOSUB MatRigStruttura
   GOSUB Triangolarizza
   CAR = 0
   DO WHILE NOT EOF(1)
      CAR = CAR + 1
      GOSUB LeggeCarGen
      IF MoAn THEN
         GOSUB AnalisiModale
      END IF
      IF QN > 0 OR QA > 0 THEN
         GOSUB StampaCarGen
         GOSUB TerminiNoti
         GOSUB Risolvi
         GOSUB CaratSollec
         Outp = 0:   GOSUB StampaRisultati
      END IF
   LOOP
ELSE
   ' analisi incrementale
   CAR = 0
   DO WHILE NOT EOF(1)
      CAR = CAR + 1
      GOSUB LeggeCarGen
      GOSUB StampaCarGen
      FOR Passo = 1 TO nPassi
         PRINT "--- Passo"; Passo
         GOSUB MatRigStruttura
         GOSUB Triangolarizza
         GOSUB TerminiNoti
         GOSUB Risolvi
         GOSUB CaratSollec
         IF Passo MOD PassoOut = 0 OR Passo = nPassi THEN
            Outp = 0:   GOSUB StampaRisultati
         END IF
         GOSUB AggiornaModuloElastico
      NEXT Passo
   LOOP
END IF
CLOSE #1
CLOSE #2
PRINT
PRINT "elaborazione terminata"
PRINT
END


' -------------------------------- DimXXXXXX -----------------------------------
'  Gruppo di routine per il dimensionamento degli array dinamici
' ------------------------------------------------------------------------------

' ................................ DimCarGen ...................................
'  Routine per il dimensionamento degli array relativi a carichi e risultati
'
DimCarGen:
   DIM F(IZ, 3), P(JZ, 2)
   DIM DeltaF(IZ, 3), DeltaP(JZ, 2)
   DIM M1(JZ), M2(JZ), T1(JZ), T2(JZ), N1(JZ), N2(JZ)
   DIM DeltaM1(JZ), DeltaM2(JZ), DeltaT1(JZ), DeltaT2(JZ), DeltaN1(JZ), DeltaN2(JZ)
RETURN

' ................................ DimGeomGen ..................................
'  Routine per il dimensionamento degli array relativi alla geometria
'
DimGeomGen:
   DIM X(IZ), Y(IZ), V$(IZ), KV(IZ, 3), NI(IZ + KZ)
   DIM Estr(JZ, 8)
   DIM E1(JZ), E2(JZ), TA$(JZ), NT(JZ), B(JZ), K(JZ), TrRig(JZ, 2)
   DIM Eprec(JZ), Gprec(JZ), Easta(JZ), Gasta(JZ)
   DIM Ates(JZ), Acom(JZ)
   DIM N(IZ, 3)
RETURN

' ................................ DimMatRig ...................................
'  Routine per il dimensionamento di matrice di rigidezza e vettore carico
'
DimMatRig:
   IF NL = 0 THEN
      REDIM KTT(0 TO 0, 0 TO 0), KTA(0 TO 0, NZ - NL)
   ELSE
      REDIM KTT(NL, NL), KTA(NL, NZ - NL), Klat(NL, NL)
   END IF
   REDIM KAA(NZ - NL, SB), R(NZ), DeltaR(NZ)
RETURN

' ................................ DimTabSez ...................................
'  Routine per il dimensionamento degli array relativi ai tipi di sezione
'
DimTabSez:
   DIM FS$(LZ), ZB(LZ), ZH(LZ), ZA(LZ), ZI(LZ), ZAT(LZ)
RETURN


' -------------------------------- Presentazione -------------------------------
'  Routine per la visualizzazione della schermata di presentazione del
'  programma
' ------------------------------------------------------------------------------
Presentazione:
   CLS
   PRINT TAB(31); "TELGEN 2.0 - rev. 02.98"
   PRINT
   PRINT "       Le ipotesi di calcolo e le modalit operative del programma TELGEN"
   PRINT "       sono riportate nella terza parte, capitolo 5, del libro:"
   PRINT "       A.Ghersi, R.Coraggio, Il personal computer nel calcolo di edifici."
   PRINT "       Analisi matriciale di strutture intelaiate, CUEN, Napoli, 1990."
   PRINT
   PRINT "       La presente versione consente anche di effettuare un'analisi modale"
   PRINT "       e di prendere in considerazione tratti rigidi agli estremi di tutte"
   PRINT "       le aste. Informazioni dettagliate sono fornite nel file TELGEN.DOC"
   PRINT
   PRINT
   PRINT
RETURN


' -------------------------------- Opzioni -------------------------------------
'  Routine che individua l'opziono output ridotto e l'opzione calcolo
'  rigidezze definita nel mandare in esecuzione il programma
' ------------------------------------------------------------------------------
Opzioni:
   C$ = UCASE$(COMMAND$)
   ' output ridotto
   IF INSTR(C$, "/OR") <> 0 THEN
      OutRid = 1
   ELSE
      OutRid = 0
   END IF
   '* opzioni in fase di attivazione (analisi incrementale)
   IF INSTR(C$, "/+Z") > 0 THEN
      Incr = 1
   ELSEIF INSTR(C$, "/+R") > 0 THEN
      Incr = 2
   ELSE
      Incr = 0
   END IF
RETURN

' -------------------------------- Completata ----------------------------------
'  Routine per l'indicazione della fine di una fase
' ------------------------------------------------------------------------------
Completata:
   LOCATE CSRLIN - 1, 44
   PRINT "completata            "
RETURN


' -------------------------------- InCorso -------------------------------------
'  Routine per l'indicazione dell'inizio di una fase
' ------------------------------------------------------------------------------
InCorso:
   LOCATE CSRLIN - 1, 44
   PRINT "in corso"
RETURN


' -------------------------------- NomeProgramma -------------------------------
'  Routine per la visualizzazione dell'intestazione del programma
' ------------------------------------------------------------------------------
NomeProgramma:
   CLS
   PRINT "TELGEN 2.0 - rev. 02.98"
   PRINT
RETURN


' -------------------------------- LeggeGeomGen --------------------------------
'  Routine per la lettura dei dati geometrici ed elastici
' ------------------------------------------------------------------------------
LeggeGeomGen:
   GOSUB NomeProgramma
   PRINT "lettura dati geometrici": GOSUB InCorso
  
   ER = 0
   CALL InputIntestaz(Contr, IT$)
   CALL InputIndGen(IZ, JZ, KZ, LZ)
   GOSUB DimGeomGen
   GOSUB DimTabSez
   GOSUB DimCarGen
   CALL InputNodiGen(Contr, IZ, X(), Y(), V$(), VCed, KV(), ER)
   CALL InputTraversiGen(Contr, KZ, NI())
   CALL InputModElast(Contr, SM, EL, GL, E, G)
   ' dati che servono per variare il modulo elastico per carico incrementale
   IF Incr = 1 THEN
      ' caso 1 - E variabile per fessurazione (Zila Rinaldi)
      CALL InputMateriale(ec1, ecu, sigmac, Es, Ey, esy, esu)
   ELSEIF Incr = 2 THEN
      ' caso 2 - E variabile con sigma (Rossella Massimino)
      '* --- da definire
   END IF
   CALL InputSezioni(Contr, LZ, FS$(), ZB(), ZH(), ZA(), ZI(), ZAT(), ER)
   CALL InputAsteGen(Incr, Contr, JZ, LZ, Estr(), NT(), TA$(), Wnkl, B(), K(), TrRig, TrRig(), Ates(), Acom(), ER)
   IF ER = 1 THEN END
   FOR j = 1 TO JZ
      Eprec(j) = 0
      Gprec(j) = 0
      Easta(j) = E
      Gasta(j) = G
   NEXT j
  
   IF NOT (Contr = 1 AND FO$ = "CON") THEN GOSUB Completata
RETURN


' -------------------------------- StampaGeomGen -------------------------------
'  Routine per la stampa di dati geometrici ed elastici
' ------------------------------------------------------------------------------
StampaGeomGen:
   PRINT "stampa dati geometrici"
   IF FO$ = "CON" THEN CLS  ELSE GOSUB InCorso
  
   CALL OutputIntestaz("TELGEN 2.0", FI$, IT$)
   CALL OutputNodiGen(IZ, X(), Y(), V$(), VCed, KV())
   CALL OutputAsteGen(Incr, SM, DefTag, Wnkl, TrRig, JZ, NT(), TA$(), Estr(), FS$(), ZB(), ZH(), ZA(), ZAT(), ZI(), B(), K(), TrRig(), Ates(), Acom())
   CALL OutputModElast(SM, DefTag, EL, GL)
   IF Incr = 1 THEN
      CALL OutputMateriale(ec1, ecu, sigmac, Es, Ey, esy, esu)
   ELSEIF Incr = 2 THEN
      '* --- da definire
   END IF
  
   IF FO$ = "CON" THEN
      CALL Aspetta(2): GOSUB NomeProgramma
   ELSE
      GOSUB Completata
   END IF
RETURN


' -------------------------------- Schema --------------------------------------
'  Routine che visualizza lo schema geometrico
' ------------------------------------------------------------------------------
Schema:
   CALL DisegnaSchema(IZ, JZ, X(), Y(), TA$(), Estr())
   GOSUB NomeProgramma
RETURN


' -------------------------------- DefinisceIncognite --------------------------
'  Routine per la numerazione delle incognite  e la valutazione della semibanda
'  (gli array che costituiscono la matrice di rigidezza vengono dimensionati)
' ------------------------------------------------------------------------------
DefinisceIncognite:
   PRINT "numerazione incognite": GOSUB InCorso

   CALL Incognite(IZ, JZ, KZ, TA$(), Estr(), V$(), NI(), N(), NL, NZ, SB)
   GOSUB DimMatRig
 
   GOSUB Completata
   PRINT "- numero totale di incognite"; NZ;
   IF NL <> 0 THEN PRINT " (tra cui"; NL; "spostamenti di traversi)";
   PRINT
   PRINT "- ampiezza della semibanda  "; SB
RETURN


' -------------------------------- MatRigStruttura -----------------------------
'  Routine per la costruzione della matrice di rigidezza della struttura
' ------------------------------------------------------------------------------
MatRigStruttura:
   PRINT "costruzione matrice rigidezza": GOSUB InCorso
  
   IF NL = 0 THEN
      REDIM KTT(0 TO 0, 0 TO 0), KTA(0 TO 0, NZ - NL)
   ELSE
      REDIM KTT(NL, NL), KTA(NL, NZ - NL)
   END IF
   REDIM KAA(NZ - NL, SB)
   FOR j = 1 TO JZ
      L = NT(j)
      IF TA$(j) = "Q8" THEN
         '* --- da preparare
         NumNodi = 8
         STOP        '* provvisorio
      ELSE
         NumNodi = 2
         E1 = Estr(j, 1)
         E2 = Estr(j, 2)
         CALL MatAstaGlobale(TA$(j), DefTag, X(E1), X(E2), Y(E1), Y(E2), ZA(L), ZAT(L), ZI(L), K(j) * 1000, B(j), TrRig(j, 1), TrRig(j, 2), Easta(j), Gasta(j), Lun, SnAl, CsAl, Ld(), Kg())
      END IF
      REDIM NumEstr(NumNodi)
      FOR nodo = 1 TO NumNodi
         NumEstr(nodo) = Estr(j, nodo)
      NEXT nodo
      CALL FormaMat(NumNodi, NumEstr(), N(), Kg(), NL, SB, KTT(), KTA(), KAA())
      LOCATE CSRLIN - 1, 44
      PRINT USING "eseguita per il ### %"; j / JZ * 100
   NEXT j
   IF VCed = 1 THEN CALL VincoliCed(IZ, N(), KV(), NL, SB, KTT(), KTA(), KAA())
  
   GOSUB Completata
RETURN


' -------------------------------- Triangolarizza ------------------------------
'  Routine che triangolarizza la matrice di rigidezza della struttura
' ------------------------------------------------------------------------------
Triangolarizza:
   PRINT "triangolarizz. matrice rigidezza": GOSUB InCorso
  
   CALL Triang.K2(KTT(), KTA(), KAA(), SB, 1)
   IF NL > 0 THEN
      Klat(1, 1) = KTT(1, 1)
      FOR R = 2 TO NL
         Klat(R, R) = KTT(R, R)
         FOR C = 1 TO R - 1
            Klat(C, R) = KTT(C, R)
            Klat(R, C) = KTT(C, R)
         NEXT C
      NEXT R
      CALL Triang.K2(KTT(), KTA(), KAA(), SB, 2)
   END IF
  
   GOSUB Completata
RETURN

' -------------------------------- LeggeCarGen ---------------------------------
'  Routine per la lettura dei carichi nodali e sulle aste
' ------------------------------------------------------------------------------
LeggeCarGen:
   PRINT
   PRINT "    Schema di carico n."; CAR
   PRINT "lettura dati di carico": GOSUB InCorso
 
   ER = 0
   CALL InputCarGen(Incr, MoAn, Contr, IZ, JZ, QN, QA, nMod, F(), P(), nPassi, PassoOut, ER)
   IF ER = 1 THEN END
   '
   IF MoAn THEN
      REDIM Mas(KZ), MA(KZ), SpePar(12), SpeT(1), SpeR(1)
      CALL InputMasseGen(Contr, KZ, Mas())
      FOR K = 1 TO KZ
         IF SM = 0 THEN MA(K) = Mas(K) ELSE MA(K) = Mas(K) / 10
      NEXT K
      CALL InputSpettro(Contr, SpePar(), SpeT(), SpeR())
   END IF

   IF NOT (Contr = 1 AND FO$ = "CON") THEN GOSUB Completata
RETURN

' -------------------------------- AnalisiModale -------------------------------
'  Routine per l'analisi modale
' ------------------------------------------------------------------------------
AnalisiModale:
   PRINT "determinazione modi": GOSUB InCorso

   REDIM Pnull(JZ, 2), Fmod(IZ, 3)
   REDIM EigVal(KZ), EigVet(KZ, KZ), T(nMod), Beta(nMod), Part(nMod)
   REDIM Forze(KZ, nMod), Energia(nMod)

   PRINT #2, : PRINT #2, : PRINT #2,
   PRINT #2, "SCHEMA DI CARICO N."; CAR; " - ANALISI MODALE"
   CALL OutputMasseGen(KZ, Mas())
   CALL OutputSpettro(SpePar(), SpeT(), SpeR())
   CALL AnalisiModale(Klat(), MA(), KZ, nMod, EigVal(), EigVet(), T(), Part(), Forze(), Energia())

   GOSUB Completata

   FOR j = 1 TO nMod
      IF FO$ = "CON" THEN CLS
      PRINT "risoluzione modo"; j: GOSUB InCorso
      IF SpePar(1) = 1 THEN
         Beta(j) = Risposta2(T(j), SpePar(2), SpeT(), SpeR())
      ELSE
         Beta(j) = Risposta(T(j), SpePar())
      END IF
      Molt0 = Beta(j) * Part(j) * 9.81
      Molt1 = Molt0 / EigVal(j)
      I = 0
      FOR K = 1 TO KZ
         FX(K) = Molt0 * Forze(K, j)
         R(K) = Molt1 * EigVet(K, j)
         I = I + 1
         Fmod(NI(I), 1) = FX(K)
         DO
            I = I + 1
         LOOP UNTIL NI(I) = 0
      NEXT K
      CALL Risolve.K2(Klat(), KTA(), KAA(), SB, 2, R())
      CALL CarSoll(DefTag, TrRig, JZ, X(), Y(), ZA(), ZAT(), ZI(), Easta(), Gasta(), TA$(), Estr(), NT(), K(), B(), TrRig(), N(), Pnull(), Fmod(), R(), N1(), T1(), M1(), N2(), T2(), M2())
      Smorz = 0
      CALL OutputModi(SM, KZ, j, EigVal(), EigVet(), T(), Beta(), Part(), FX(), R(), EnIso, EnTot, Smorz)
      IF OutRid = 0 THEN
         CALL OutputRisGen(SM, 0, IZ, JZ, KZ, NL, V$(), KV(), NI(), N(), R(), M1(), M2(), T1(), T2(), N1(), N2(), Fmod())
      END IF
      IF j = 1 THEN Oper = 0 ELSE Oper = 1
      CALL SommaModiGen(Oper, JZ, NZ, R(), M1(), M2(), T1(), T2(), N1(), N2())
      REDIM R(NZ), Fmod(IZ, 3)
      IF FO$ = "CON" THEN LOCATE 24, 1 ELSE GOSUB Completata
   NEXT j
   Oper = 2
   CALL SommaModiGen(Oper, JZ, NZ, R(), M1(), M2(), T1(), T2(), N1(), N2())
   Outp = -1:  GOSUB StampaRisultati

RETURN


' -------------------------------- StampaCarGen --------------------------------
'  Routine per la stampa dei carichi nodali e sulle aste
' ------------------------------------------------------------------------------
StampaCarGen:
   PRINT "stampa dati di carico"
   IF FO$ = "CON" THEN CLS  ELSE GOSUB InCorso
  
   PRINT #2, : PRINT #2, : PRINT #2,
   PRINT #2, "SCHEMA DI CARICO N."; CAR: PRINT #2,
   CALL OutputCarGen(SM!, CAR, IZ, JZ, QN, QA, F(), P())
  
   IF FO$ = "CON" THEN
      CALL Aspetta(2): GOSUB NomeProgramma
   ELSE
      GOSUB Completata
   END IF
RETURN


' -------------------------------- TerminiNoti ---------------------------------
'  Routine per il calcolo dei carichi nodali   
' ------------------------------------------------------------------------------
TerminiNoti:
   PRINT "preparazione termini noti": GOSUB InCorso
 
   '* per il momento non si tiene conto di eventuali carichi negli elementi Q8
   IF Incr = 0 THEN
      CALL TermNoto(TrRig, IZ, JZ, X(), Y(), ZA(), ZI(), E, TA$(), Estr(), NT(), K(), B(), TrRig(), N(), P(), F(), R())
   ELSE
      FOR I = 1 TO IZ
         DeltaF(I, 1) = F(I, 1) / nPassi
         DeltaF(I, 2) = F(I, 2) / nPassi
         DeltaF(I, 3) = F(I, 3) / nPassi
      NEXT I
      FOR j = 1 TO JZ
         DeltaP(j, 1) = P(j, 1) / nPassi
         DeltaP(j, 2) = P(j, 2) / nPassi
      NEXT j
      CALL TermNoto(TrRig, IZ, JZ, X(), Y(), ZA(), ZI(), E, TA$(), Estr(), NT(), K(), B(), TrRig(), N(), DeltaP(), DeltaF(), DeltaR())
   END IF

   GOSUB Completata
RETURN


' -------------------------------- Risolvi -------------------------------------
'  Routine per la risoluzione mediante sostituzione all'indietro
' ------------------------------------------------------------------------------
Risolvi:
   PRINT "risoluzione": GOSUB InCorso
  
   IF Incr = 0 THEN
      CALL Risolve.K2(KTT(), KTA(), KAA(), SB, 0, R())
   ELSE
      CALL Risolve.K2(KTT(), KTA(), KAA(), SB, 0, DeltaR())
      FOR N = 1 TO NZ
         R(N) = R(N) + DeltaR(N)
      NEXT N
   END IF
  
   GOSUB Completata
RETURN


' -------------------------------- CaratSollec ---------------------------------
'  Routine per il calcolo delle azioni sulle aste
' ------------------------------------------------------------------------------
CaratSollec:
   PRINT "calcolo caratt. sollecitazione": GOSUB InCorso
  
   '* per il momento non considera gli elementi Q8
   IF Incr = 0 THEN
      CALL CarSoll(DefTag, TrRig, JZ, X(), Y(), ZA(), ZAT(), ZI(), Easta(), Gasta(), TA$(), Estr(), NT(), K(), B(), TrRig(), N(), P(), F(), R(), N1(), T1(), M1(), N2(), T2(), M2())
   ELSE
      CALL CarSoll(DefTag, TrRig, JZ, X(), Y(), ZA(), ZAT(), ZI(), Easta(), Gasta(), TA$(), Estr(), NT(), K(), B(), TrRig(), N(), DeltaP(), DeltaF(), DeltaR(), DeltaN1(), DeltaT1(), DeltaM1(), DeltaN2(), DeltaT2(), DeltaM2())
      FOR j = 1 TO JZ
         M1(j) = M1(j) + DeltaM1(j)
         T1(j) = T1(j) + DeltaT1(j)
         N1(j) = N1(j) + DeltaN1(j)
         M2(j) = M2(j) + DeltaM2(j)
         T2(j) = T2(j) + DeltaT2(j)
         N2(j) = N2(j) + DeltaN2(j)
      NEXT j
   END IF
 
   GOSUB Completata
RETURN


' -------------------------------- StampaRisultati -----------------------------
'  Routine per la stampa dei risultati
' ------------------------------------------------------------------------------
StampaRisultati:
   IF Outp = -1 THEN
      PRINT "stampa inviluppo modale"
   ELSE
      PRINT "stampa risultati"
   END IF
   IF FO$ = "CON" THEN CLS  ELSE GOSUB InCorso

   REDIM SquilF(IZ, 3)
   IF Icr = 0 THEN
      FOR I = 1 TO IZ
         SquilF(I, 1) = F(I, 1)
         SquilF(I, 2) = F(I, 2)
         SquilF(I, 3) = F(I, 3)
      NEXT I
   ELSE
      FOR I = 1 TO IZ
         SquilF(I, 1) = F(I, 1) * Passo / nPassi
         SquilF(I, 2) = F(I, 2) * Passo / nPassi
         SquilF(I, 3) = F(I, 3) * Passo / nPassi
      NEXT I
      CALL CalcolaSquilibrio(JZ, X(), Y(), TA$(), Estr(), SquilF(), N1(), T1(), M1(), N2(), T2(), M2())
   END IF
  
   IF Outp = -1 THEN
      PRINT #2, : PRINT #2, : PRINT #2,
      PRINT #2, "ANALISI MODALE - inviluppo dei modi"
   END IF
   IF Incr <> 0 THEN
      PRINT #2, : PRINT #2,
      PRINT #2, "Risultati del passo"; Passo; " su"; nPassi
   END IF
   CALL OutputRisGen(SM, Outp, IZ, JZ, KZ, NL, V$(), KV(), NI(), N(), R(), M1(), M2(), T1(), T2(), N1(), N2(), SquilF())
   GOSUB OutputIpotesiGen

   IF FO$ = "CON" THEN
      LOCATE 24, 1
   ELSE
      GOSUB Completata
   END IF
RETURN


' -------------------------------- AggiornaModuloElastico ----------------------
'  Routine per l'aggiornamento del modulo elastico (per carico incrementale)
' ------------------------------------------------------------------------------
AggiornaModuloElastico:
   FOR j = 1 TO JZ
      Mmed = (M1(j) + M2(j)) / 2
      Nmed = (N1(j) + N2(j)) / 2
      L = NT(j)
      CALL Curv(ZB(L), ZH(L), Ates(j), Acom(j), E, ec1, ecu, sigmac, Es, Ey, esy, esu, Nmed, Mlim1, Mlim2, Rid1, Rid2)
      Eprec(j) = Easta(j)
      Gprec(j) = Gasta(j)
      IF Mmed <= Mlim1 THEN
         Easta(j) = E
         Gasta(j) = G
      ELSEIF Mmed <= Mlim2 THEN
         Easta(j) = E * Rid1
         Gasta(j) = G * Rid1
      ELSE
         Easta(j) = E * Rid2
         Gasta(j) = G * Rid2
      END IF
   NEXT j
RETURN


' ................................ OutputIpotesiGen ............................
'  Routine per la stampa delle note sulle ipotesi di calcolo
'
OutputIpotesiGen:
   PRINT #2, : PRINT #2,
   PRINT #2, "Il calcolo e' stato eseguito :"
   PRINT #2, "- tenendo conto della deformazione estensionale delle aste"
   IF DefTag = 1 THEN
      PRINT #2, "- tenendo conto della deformazione a taglio delle aste"
      IF Wnkl = 1 THEN
         PRINT #2, "- trascurando la deformazione a taglio delle aste su suolo alla Winkler"
      END IF
   ELSE
      PRINT #2, "- trascurando la deformazione a taglio delle aste"
   END IF
   PRINT #2, : PRINT #2,
RETURN

