'                                  TELRET.BAS
'
'  Programma per la risoluzione statica o l'analisi modale di strutture piane
'           intelaiate con maglie di forma rettangolare o trapezia
'
'  tipi di aste:        travi alla De Saint Venant, incastrate ai nodi
'                          (senza o con tratto rigido agli estremi)
'                       travi di fondazione su suolo elastico alla Winkler
'
'  vincoli esterni:     vincoli di incastro perfetto alla base
'                       vincoli mutui allo spostamento orizzontale (traversi)
'
'  carichi:             carichi uniformemente distribuiti sulle travi
'                       momenti concentrati sui nodi
'                       forze verticali concentrate sui nodi
'                       forze orizzontali applicate ai traversi
'
'  ver. 2.5                                                       rev. 02.99


' ------------------ Dichiarazione delle procedure utilizzate ------------------

' procedure del file MOANA.BAS
DECLARE SUB AnalisiModale (Klat!(), MA!(), KZ!, nMod!, EigVal!(), EigVet!(), T!(), Part!(), Forze!(), Energia!())
DECLARE SUB OutputModi (SM!, KZ!, J!, EigVal!(), EigVet!(), T!(), Beta!(), Part!(), Forza!(), R!(), EnIso!, EnTot!, Smorz!)
DECLARE FUNCTION Risposta! (T!, SpePar!())
DECLARE FUNCTION Risposta2! (T!, nCop!, SpeT!(), SpeR!())

' procedure del file SOLSIST.BAS
DECLARE SUB Triang.K2 (KTT!(), KTA!(), KAA!(), SB!, RID!)
DECLARE SUB Risolve.K2 (KTT!(), KTA!(), KAA!(), SB!, RID!, B!())

' procedure del file PROCOM.BAS
DECLARE SUB Aspetta (Sec!)
DECLARE SUB CarSoll (DefTag!, TrRig!, JZ!, X!(), Y!(), ZA!(), ZAT!(), ZI!(), Easta!(), Gasta!(), TA$(), Estr!(), NT!(), K!(), B!(), TrRig!(), N!(), P!(), F!(), R!(), N1!(), T1!(), M1!(), N2!(), T2!(), M2!())
DECLARE SUB DefinisceIO (Prog$, FI$, FO$, FT$)
DECLARE SUB DefinisceMO (Prog$, DefEst!, DefTag!, Contr!)
DECLARE SUB DefinisceSM (SM!)
DECLARE SUB DisegnaSchema (IZ!, JZ!, X!(), Y!(), TA$(), Estr!())
DECLARE SUB FormaMat (NumNodi!, NumEstr!(), N!(), Kg!(), NL!, SB!, KTT!(), KTA!(), KAA!())
DECLARE SUB Incognite (IZ!, JZ!, KZ!, TA$(), Estr!(), V$(), NI!(), N!(), NL!, NZ!, SB!)
DECLARE SUB InputIntestaz (Contr!, Intest$)
DECLARE SUB InputModElast (Contr!, SM!, EL!, GL!, E!, G!)
DECLARE SUB InputSezioni (Contr!, LZ!, FS$(), ZB!(), ZH!(), ZA!(), ZI!(), ZAT!(), ER!)
DECLARE SUB InputSpettro (Contr!, SpePar!(), SpeT!(), SpeR!())
DECLARE SUB MatAstaGlobale (TA$, DefTag!, X1!, X2!, Y1!, Y2!, A!, AT!, Iner!, K!, B!, TrRigSin!, TrRigDes!, E!, G!, Lun!, SnAl!, CsAl!, Ld!(), Kg!())
DECLARE SUB OutputIntestaz (NomeProg$, FI$, Intest$)
DECLARE SUB OutputModElast (SM!, DefTag!, EL!, GL!)
DECLARE SUB OutputSpettro (SpePar!(), SpeT!(), SpeR!())
DECLARE SUB OutputSpoOriz (KZ!, SpoAssTel!(), SpoRelTel!())
DECLARE SUB TermNoto (TrRig!, IZ!, JZ!, X!(), Y!(), ZA!(), ZI!(), E!, TA$(), Estr!(), NT!(), K!(), B!(), TrRig!(), N!(), P!(), F!(), R!())
DECLARE SUB VincoliCed (IZ!, N!(), KV!(), NL!, SB!, KTT!(), KTA!(), KAA!())

' procedure del file PRORET.BAS
DECLARE SUB ContrRitti (PZ!, RZ!, NP!(), NB!(), NR!(), ZA!(), ZI!(), ER!)
DECLARE SUB ConverteCarRet (RZ!, PZ!, Wnkl!, NB!(), NF!(), ZA!(), ZI!(), IV!, IM!, IN!, IO!, Q!(), QF!(), MN!(), MNF!(), FYN!(), FYF!(), FX!(), KZ!, V$(), NI!(), P!(), F!())
DECLARE SUB ConverteGeomRet (PZ!, RZ!, Wnkl!, TrRig!, DefEst!, DL!(), H!(), NB!(), NR!(), NF!(), BF!(), KF!(), TrRigTra!(), TrRigPil!(), TrRigFon!(), ZA!(), ZI!(), IZ!, JZ!, KZ!, X!(), Y!(), V$(), NI!(), TA$(), Estr!(), NT!(), B!(), K!(), TrRig!())
DECLARE SUB ConverteRisRet (RZ!, PZ!, Wnkl!, DefEst!, NB!(), NR!(), NF!(), ZA!(), ZI!(), D!(), MS!(), MD!(), TS!(), TD!(), MP!(), MT!(), TR!(), SN!(), MFS!(), MFD!(), TFS!(), TFD!(), KZ!, NI!(), N!(), R!(), F!(), M1!(), M2!(), T1!(), T2!(), N1!())
DECLARE SUB InputAltezze (Contr!, PZ!, RZ!, AltVar!, HI!(), H!())
DECLARE SUB InputAsteRet (Contr!, Wnkl!, TrRig!, PZ!, RZ!, LZ!, NP!(), NB!(), NR!(), NF!(), BF!(), KF!(), TrRigTra!(), TrRigPil!(), TrRigFon!(), ER!)
DECLARE SUB InputCarRet (Contr!, Wnkl!, IV!, IM!, IN!, IO!, PZ!, RZ!, KZ!, NP!(), V$(), NB!(), NF!(), ZA!(), ZI!(), Q!(), QF!(), MN!(), MNF!(), FYN!(), FYF!(), FX!())
DECLARE SUB InputLuci (Contr!, Wnkl!, RZ!, DL!())
DECLARE SUB InputMasseRet (Contr!, KZ!, Mas!())
DECLARE SUB InputNumRitti (Contr!, RZ!, NP!())
DECLARE SUB OutputAsteRet (SM!, PZ!, RZ!, Wnkl!, TrRig!, NP!(), DL!(), NB!(), NR!(), NF!(), BF!(), KF!(), TrRigTra!(), TrRigPil!(), TrRigFon!(), FS$(), ZB!(), ZH!(), ZI!())
DECLARE SUB OutputCarRet (SM!, Wnkl!, IV!, IM!, IN!, IO!, PZ!, RZ!, KZ!, NP!(), V$(), NB!(), NF!(), ZA!(), ZI!(), Q!(), QF!(), MN!(), MNF!(), FYN!(), FYF!(), FX!())
DECLARE SUB OutputMasseRet (KZ!, Mas!())
DECLARE SUB OutputPiani (AltVar!, PZ!, RZ!, NP!(), HI!(), H!())
DECLARE SUB OutputRisRet (SM!, Outp!, RZ!, PZ!, Wnkl!, NP!(), NB!(), NR!(), NF!(), ZA!(), ZI!(), MS!(), MD!(), TS!(), TD!(), MP!(), MT!(), TR!(), SN!(), MFS!(), MFD!(), TFS!(), TFD!(), IZ!, KZ!, NL!, V$(), N!(), R!(), F!())
DECLARE SUB OutputTabSez (DefTag!, LZ!, FS$(), ZB!(), ZH!(), ZA!(), ZAT!(), ZI!())
DECLARE SUB SommaModiRet (Oper!, Wnkl!, RZ!, PZ!, KZ!, D!(), MS!(), MD!(), TS!(), TD!(), MT!(), MP!(), TR!(), SN!(), MFS!(), MFD!(), TFS!(), TFD!())

CLEAR , , 2000

' ------------------ Dimensionamento delle variabili con indice ----------------

OPTION BASE 1            ' definisce l'estremo inferiore dell'indice degli array
DIM Ld(6, 6), Kg(6, 6)   ' array statici
                         ' array dinamici: sono dimensionati in apposite routine


' --------------------------- Programma principale -----------------------------

GOSUB Presentazione
GOSUB Opzioni
CALL DefinisceIO("RET", FI$, FO$, FT$)
CALL DefinisceMO("RET", DefEst, DefTag, Contr)
CALL DefinisceSM(SM)
OPEN FI$ FOR INPUT AS #1
OPEN FO$ FOR OUTPUT AS #2
GOSUB LeggeGeomRet
GOSUB StampaGeomRet
IF CalcRig = 1 THEN GOSUB DistinguiTelai
GOSUB ConvGeomRet
GOSUB Schema
GOSUB DefinisceIncognite
GOSUB MatRigStruttura
GOSUB Triangolarizza
CAR = 0
DO WHILE NOT EOF(1)
   CAR = CAR + 1
   GOSUB LeggeCarRet
   IF IO > 1 THEN
      GOSUB AnalisiModale
   END IF
   IF IO = 1 OR IV + IM + IN > 0 THEN
      GOSUB StampaCarRet
      GOSUB ConvCarRet
      GOSUB TerminiNoti
      GOSUB Risolvi
      GOSUB CaratSollec
      GOSUB ConvRisultRet
      Outp = 0:   GOSUB StampaRisultRet
      IF CalcRig = 1 THEN GOSUB CalcolaRigidezze
   END IF
LOOP
CLOSE #1
CLOSE #2
PRINT
PRINT "elaborazione terminata"
PRINT
END


' -------------------------------- DimXXXXXX -----------------------------------
'  Gruppo di routine per il dimensionamento degli array dinamici
' ------------------------------------------------------------------------------

' ................................ DimCarGen ...................................
'  Routine per il dimensionamento degli array relativi a carichi e risultati
'
DimCarGen:
   DIM F(IZ, 3), P(JZ, 2)
   DIM M1(JZ), M2(JZ), T1(JZ), T2(JZ), N1(JZ), N2(JZ)
RETURN

' ................................ DimCarRet ...................................
'  Routine per il dimensionamento degli array relativi a carichi e risultati
'
DimCarRet:
   DIM Q(RZ - 1, PZ), MN(RZ, PZ), FYN(RZ, PZ), FX(KZ)
   REDIM Mas(KZ), MA(KZ), SpePar(12), SpeT(1), SpeR(1)
   DIM D(KZ), MD(RZ - 1, PZ), MS(RZ - 1, PZ), TD(RZ - 1, PZ), TS(RZ - 1, PZ)
   DIM MP(RZ, PZ), MT(RZ, PZ), TR(RZ, PZ), SN(RZ, PZ)
   IF Wnkl = 1 THEN
      DIM QF(0 TO RZ), MNF(0 TO RZ + 1), FYF(0 TO RZ + 1)
      DIM MFD(0 TO RZ), MFS(0 TO RZ), TFD(0 TO RZ), TFS(0 TO RZ)
   END IF
RETURN

' ................................ DimGeomGen ..................................
'  Routine per il dimensionamento degli array relativi alla geometria
'
DimGeomGen:
   DIM X(IZ), Y(IZ), V$(IZ), KV(1, 1), NI(IZ + KZ)
   DIM Estr(JZ, 2), TA$(JZ), NT(JZ), B(JZ), K(JZ)
   DIM Easta(JZ), Gasta(JZ)
   DIM N(IZ, 3)
   IF TrRig <> 0 THEN DIM TrRig(JZ, 2)
RETURN

' ................................ DimGeomRet ..................................
'  Routine per il dimensionamento degli array relativi alla geometria
'
DimGeomRet:
   DIM NP(RZ), HI(PZ), H(RZ, 0 TO PZ), DL(0 TO RZ)
   DIM NB(RZ - 1, PZ), NR(RZ, PZ)
   IF Wnkl = 1 THEN DIM NF(0 TO RZ), BF(0 TO RZ), KF(0 TO RZ)
   IF TrRig <> 0 THEN DIM TrRigTra(RZ - 1, PZ, 2), TrRigPil(RZ, PZ, 2)
   IF TrRig = 2 THEN DIM TrRigFon(0 TO RZ, 2)
RETURN

' ................................ DimMatRig ...................................
'  Routine per il dimensionamento di matrice di rigidezza e vettore carico
'
DimMatRig:
   IF NL = 0 THEN
      REDIM KTT(0 TO 0, 0 TO 0), KTA(0 TO 0, NZ - NL)
   ELSE
      REDIM KTT(NL, NL), KTA(NL, NZ - NL), Klat(NL, NL)
   END IF
   DIM KAA(NZ - NL, SB), R(NZ)
RETURN

' ................................ DimTabSez ...................................
'  Routine per il dimensionamento degli array relativi ai tipi di sezione
'
DimTabSez:
   DIM FS$(LZ), ZB(LZ), ZH(LZ), ZA(LZ), ZI(LZ), ZAT(LZ)
RETURN

' -------------------------------- Presentazione -------------------------------
'  Routine per la visualizzazione della schermata di presentazione del
'  programma
' ------------------------------------------------------------------------------
Presentazione:
   CLS
   PRINT TAB(31); "TELRET 2.4 - rev. 02.98"
   PRINT
   PRINT "       Le ipotesi di calcolo e le modalit operative  del programma TELRET"
   PRINT "       (versione base) sono riportate nella terza parte, cap.6, del libro:"
   PRINT "       A.Ghersi, R.Coraggio,  Il personal computer nel calcolo di edifici."
   PRINT "       Analisi matriciale di strutture intelaiate, CUEN, Napoli, 1990."
   PRINT
   PRINT "       La presente versione consente anche di effettuare un'analisi modale"
   PRINT "       e di prendere in considerazione tratti rigidi agli estremi di tutte"
   PRINT "       le aste. Informazioni dettagliate sono fornite nel file TELRET.DOC"
   PRINT
   PRINT
RETURN

' -------------------------------- Opzioni -------------------------------------
'  Routine che individua l'opziono output ridotto e l'opzione calcolo
'  rigidezze definita nel mandare in esecuzione il programma
' ------------------------------------------------------------------------------
Opzioni:
   C$ = UCASE$(COMMAND$)
   ' output ridotto
   IF INSTR(C$, "/OR") <> 0 THEN
      OutRid = 1
   ELSE
      OutRid = 0
   END IF
   IF INSTR(C$, "/R") = 0 THEN CalcRig = 0 ELSE CalcRig = 1
RETURN

' -------------------------------- Completata ----------------------------------
'  Routine per l'indicazione della fine di una fase
' ------------------------------------------------------------------------------
Completata:
   LOCATE CSRLIN - 1, 44
   PRINT "completata            "
RETURN


' -------------------------------- InCorso -------------------------------------
'  Routine per l'indicazione dell'inizio di una fase
' ------------------------------------------------------------------------------
InCorso:
   LOCATE CSRLIN - 1, 44
   PRINT "in corso"
RETURN


' -------------------------------- NomeProgramma -------------------------------
'  Routine per la visualizzazione dell'intestazione del programma
' ------------------------------------------------------------------------------
NomeProgramma:
   CLS
   PRINT "TELRET 2.4 - rev. 02.98"
   PRINT
RETURN


' -------------------------------- LeggeGeomRet --------------------------------
'  Routine per la lettura dei dati geometrici ed elastici
' ------------------------------------------------------------------------------
LeggeGeomRet:
   GOSUB NomeProgramma
   PRINT "lettura dati geometrici": GOSUB InCorso
  
   ER = 0
   CALL InputIntestaz(Contr, IT$)
   GOSUB InputIndRet
   GOSUB DimGeomRet
   GOSUB DimTabSez
   CALL InputNumRitti(Contr, RZ, NP())
   CALL InputAltezze(Contr, PZ, RZ, AltVar, HI(), H())
   CALL InputLuci(Contr, Wnkl, RZ, DL())
   CALL InputModElast(Contr, SM, EL, GL, E, G)
   CALL InputSezioni(Contr, LZ, FS$(), ZB(), ZH(), ZA(), ZI(), ZAT(), ER)
   CALL InputAsteRet(Contr, Wnkl, TrRig, PZ, RZ, LZ, NP(), NB(), NR(), NF(), BF(), KF(), TrRigTra(), TrRigPil(), TrRigFon(), ER)
   IF ER = 1 THEN END
   IF DefEst = 0 THEN CALL ContrRitti(PZ, RZ, NP(), NB(), NR(), ZA(), ZI(), ER)
   IF ER = 1 THEN END
  
   IF NOT (Contr = 1 AND FO$ = "CON") THEN GOSUB Completata
RETURN

' ................................ InputIndRet .................................
'  Routine per la lettura delle indicazioni generali
'
InputIndRet:
   INPUT #1, RZ, PZ, LZ, Wnkl
   SELECT CASE Wnkl
      CASE 0
         TrRig = 0:  Wnkl = 0
      CASE 1
         TrRig = 0:  Wnkl = 1
      CASE 10
         TrRig = 1:  Wnkl = 0
      CASE 11
         TrRig = 1:  Wnkl = 1
      CASE 21
         TrRig = 2:  Wnkl = 1
   END SELECT
   IF Contr = 1 THEN
      PRINT #2, "numero di pilastri: "; RZ
      PRINT #2, "numero di piani: "; PZ
      PRINT #2, "numero di tipi di sezione: "; LZ
      IF Wnkl = 1 THEN PRINT #2, " presente la trave di fondazione"
   END IF
   IF DefEst = 0 AND Wnkl = 1 THEN
      PRINT "Esecuzione interrotta:  il programma non consente l'ipotesi"
      PRINT "di aste inestensibili quando esiste una trave di fondazione"
      END
   END IF
RETURN

' -------------------------------- StampaGeomRet -------------------------------
'  Routine per la stampa di dati geometrici ed elastici
' ------------------------------------------------------------------------------
StampaGeomRet:
   PRINT "stampa dati geometrici"
   IF FO$ = "CON" THEN CLS  ELSE GOSUB InCorso
  
   CALL OutputIntestaz("TELRET 2.4", FI$, IT$)
   CALL OutputPiani(AltVar, PZ, RZ, NP(), HI(), H())
   CALL OutputAsteRet(SM, PZ, RZ, Wnkl, TrRig, NP(), DL(), NB(), NR(), NF(), BF(), KF(), TrRigTra(), TrRigPil(), TrRigFon(), FS$(), ZB(), ZH(), ZI())
   CALL OutputTabSez(DefTag, LZ, FS$(), ZB(), ZH(), ZA(), ZAT(), ZI())
   CALL OutputModElast(SM, DefTag, EL, GL)
  
   IF FO$ = "CON" THEN
      CALL Aspetta(2): GOSUB NomeProgramma
   ELSE
      GOSUB Completata
   END IF
RETURN

' -------------------------------- DistinguiTelai ------------------------------
'  Routine che distingue i singoli telai (in caso di opzione /R)
' ------------------------------------------------------------------------------
DistinguiTelai:
   DIM Npil(20)
   Ntel = 1
   FOR R = 1 TO RZ - 1
      Pend = 0
      FOR P = 1 TO PZ
         L = NB(R, P)
         IF ZI(L) <> 0 THEN Pend = 1
      NEXT P
      IF Pend = 0 THEN
         Npil(Ntel) = R
         Ntel = Ntel + 1
      END IF
   NEXT R
   Npil(Ntel) = RZ
RETURN

' -------------------------------- ConvGeomRet ---------------------------------
'   Routine per la conversione dei dati geometrici per telai a maglie
'   rettangolari
' ------------------------------------------------------------------------------
ConvGeomRet:
   PRINT "conversione dati geometrici": GOSUB InCorso
   
   IZ = RZ * (PZ + 1) + 2
   JZ = (2 * RZ - 1) * PZ + RZ + 1     ' massimo numero di aste possibile
   KZ = INT(RZ * PZ / 2)               ' massimo numero di traversi possibile
   GOSUB DimGeomGen
   CALL ConverteGeomRet(PZ, RZ, Wnkl, TrRig, DefEst, DL(), H(), NB(), NR(), NF(), BF(), KF(), TrRigTra(), TrRigPil(), TrRigFon(), ZA(), ZI(), IZ, JZ, KZ, X(), Y(), V$(), NI(), TA$(), Estr(), NT(), B(), K(), TrRig())
   FOR J = 1 TO JZ
      Easta(J) = E
      Gasta(J) = G
   NEXT J
   VCed = 0
   GOSUB DimCarRet
   GOSUB DimCarGen
  
   GOSUB Completata
RETURN

' -------------------------------- Schema --------------------------------------
'  Routine che visualizza lo schema geometrico
' ------------------------------------------------------------------------------
Schema:
   CALL DisegnaSchema(IZ, JZ, X(), Y(), TA$(), Estr())
   GOSUB NomeProgramma
RETURN

' -------------------------------- DefinisceIncognite --------------------------
'  Routine per la numerazione delle incognite  e la valutazione della semibanda
'  (gli array che costituiscono la matrice di rigidezza vengono dimensionati)
' ------------------------------------------------------------------------------
DefinisceIncognite:
   PRINT "numerazione incognite": GOSUB InCorso

   CALL Incognite(IZ, JZ, KZ, TA$(), Estr(), V$(), NI(), N(), NL, NZ, SB)
   GOSUB DimMatRig

   GOSUB Completata
   PRINT "- numero totale di incognite"; NZ;
   IF NL <> 0 THEN PRINT " (tra cui"; NL; "spostamenti di traversi)";
   PRINT
   PRINT "- ampiezza della semibanda  "; SB
RETURN

' -------------------------------- MatRigStruttura -----------------------------
'  Routine per la costruzione della matrice di rigidezza della struttura
' ------------------------------------------------------------------------------
MatRigStruttura:
   PRINT "costruzione matrice rigidezza": GOSUB InCorso
 
   FOR J = 1 TO JZ
      L = NT(J)
      IF TrRig <> 0 THEN
         TrRigSin = TrRig(J, 1)
         TrRigDes = TrRig(J, 2)
      END IF
      E1 = Estr(J, 1)
      E2 = Estr(J, 2)
      CALL MatAstaGlobale(TA$(J), DefTag, X(E1), X(E2), Y(E1), Y(E2), ZA(L), ZAT(L), ZI(L), K(J) * 1000, B(J), TrRigSin, TrRigDes, E, G, Lun, SnAl, CsAl, Ld(), Kg())
      NumNodi = 2
      REDIM NumEstr(2)
      NumEstr(1) = Estr(J, 1)
      NumEstr(2) = Estr(J, 2)
      CALL FormaMat(NumNodi, NumEstr(), N(), Kg(), NL, SB, KTT(), KTA(), KAA())
      LOCATE CSRLIN - 1, 44
      PRINT USING "eseguita per il ### %"; J / JZ * 100
   NEXT J
   IF VCed = 1 THEN CALL VincoliCed(IZ, N(), KV(), NL, SB, KTT(), KTA(), KAA())
 
   GOSUB Completata
RETURN

' -------------------------------- Triangolarizza ------------------------------
'  Routine che triangolarizza la matrice di rigidezza della struttura
' ------------------------------------------------------------------------------
Triangolarizza:
   PRINT "triangolarizz. matrice rigidezza": GOSUB InCorso
  
   CALL Triang.K2(KTT(), KTA(), KAA(), SB, 1)
   IF NL > 0 THEN
      Klat(1, 1) = KTT(1, 1)
      FOR R = 2 TO NL
         Klat(R, R) = KTT(R, R)
         FOR C = 1 TO R - 1
            Klat(C, R) = KTT(C, R)
            Klat(R, C) = KTT(C, R)
         NEXT C
      NEXT R
      CALL Triang.K2(KTT(), KTA(), KAA(), SB, 2)
   END IF
  
   GOSUB Completata
RETURN

' -------------------------------- LeggeCarRet ---------------------------------
'  Routine per la lettura dei carichi nodali e sulle aste
' ------------------------------------------------------------------------------
LeggeCarRet:
   PRINT
   PRINT "    Schema di carico n."; CAR
   PRINT "lettura dati di carico": GOSUB InCorso
  
   IF Contr = 1 THEN PRINT #2, "    Schema di carico n."; CAR
   CALL InputCarRet(Contr, Wnkl, IV, IM, IN, IO, PZ, RZ, KZ, NP(), V$(), NB(), NF(), ZA(), ZI(), Q(), QF(), MN(), MNF(), FYN(), FYF(), FX())
   IF IO > 1 THEN
      IF Contr = 1 THEN PRINT #2, "richiesta analisi modale, con uso di"; IO; "modi"
      CALL InputMasseRet(Contr, KZ, Mas())
      FOR K = 1 TO KZ
         IF SM = 0 THEN MA(K) = Mas(K) ELSE MA(K) = Mas(K) / 10
      NEXT K
      CALL InputSpettro(Contr, SpePar(), SpeT(), SpeR())
   END IF
   
   IF NOT (Contr = 1 AND FO$ = "CON") THEN GOSUB Completata
RETURN

' -------------------------------- AnalisiModale -------------------------------
'  Routine per l'analisi modale
' ------------------------------------------------------------------------------
AnalisiModale:
   PRINT "determinazione modi": GOSUB InCorso

   nMod = IO
   REDIM EigVal(KZ), EigVet(KZ, KZ), T(nMod), Beta(nMod), Part(nMod)
   REDIM Forze(KZ, nMod), Energia(nMod)

   PRINT #2, : PRINT #2, : PRINT #2,
   PRINT #2, "SCHEMA DI CARICO N."; CAR; " - ANALISI MODALE"
   CALL OutputMasseRet(KZ, Mas())
   CALL OutputSpettro(SpePar(), SpeT(), SpeR())
   CALL AnalisiModale(Klat(), MA(), KZ, nMod, EigVal(), EigVet(), T(), Part(), Forze(), Energia())

   GOSUB Completata

   FOR J = 1 TO nMod
      IF FO$ = "CON" THEN CLS
      PRINT "risoluzione modo"; J: GOSUB InCorso
      IF SpePar(1) = 1 THEN
         Beta(J) = Risposta2(T(J), SpePar(2), SpeT(), SpeR())
      ELSE
         Beta(J) = Risposta(T(J), SpePar())
      END IF
      Molt0 = Beta(J) * Part(J) * 9.81
      Molt1 = Molt0 / EigVal(J)
      I = 0
      FOR K = 1 TO KZ
         FX(K) = Molt0 * Forze(K, J)
         R(K) = Molt1 * EigVet(K, J)
         I = I + 1
         F(NI(I), 1) = FX(K)
         DO
            I = I + 1
         LOOP UNTIL NI(I) = 0
      NEXT K
      CALL Risolve.K2(Klat(), KTA(), KAA(), SB, 2, R())
      CALL CarSoll(DefTag, TrRig, JZ, X(), Y(), ZA(), ZAT(), ZI(), Easta(), Gasta(), TA$(), Estr(), NT(), K(), B(), TrRig(), N(), P(), F(), R(), N1(), T1(), M1(), N2(), T2(), M2())
      CALL ConverteRisRet(RZ, PZ, Wnkl, DefEst, NB(), NR(), NF(), ZA(), ZI(), D(), MS(), MD(), TS(), TD(), MP(), MT(), TR(), SN(), MFS(), MFD(), TFS(), TFD(), KZ, NI(), N(), R(), F(), M1(), M2(), T1(), T2(), N1())
      Smorz = 0
      CALL OutputModi(SM, KZ, J, EigVal(), EigVet(), T(), Beta(), Part(), FX(), R(), EnIso, EnTot, Smorz)
      IF OutRid = 0 THEN
         REDIM SpoRelTel(KZ)
         SpoRelTel(1) = D(1)
         FOR K = 2 TO KZ
            SpoRelTel(K) = D(K) - D(K - 1)
         NEXT K
         CALL OutputSpoOriz(KZ, D(), SpoRelTel())
         CALL OutputRisRet(SM, 0, RZ, PZ, Wnkl, NP(), NB(), NR(), NF(), ZA(), ZI(), MS(), MD(), TS(), TD(), MP(), MT(), TR(), SN(), MFS(), MFD(), TFS(), TFD(), IZ, KZ, NL, V$(), N(), R(), F())
      END IF
      IF J = 1 THEN Oper = 0 ELSE Oper = 1
      CALL SommaModiRet(Oper, Wnkl, RZ, PZ, KZ, D(), MS(), MD(), TS(), TD(), MT(), MP(), TR(), SN(), MFS(), MFD(), TFS(), TFD())
      REDIM R(NZ), F(IZ, 3)
      IF FO$ = "CON" THEN LOCATE 24, 1 ELSE GOSUB Completata
   NEXT J
   Oper = 2
   CALL SommaModiRet(Oper, Wnkl, RZ, PZ, KZ, D(), MS(), MD(), TS(), TD(), MT(), MP(), TR(), SN(), MFS(), MFD(), TFS(), TFD())
   Outp = -1:  GOSUB StampaRisultRet

RETURN

' -------------------------------- StampaCarRet --------------------------------
'  Routine per la stampa dei carichi nodali e sulle aste
' ------------------------------------------------------------------------------
StampaCarRet:
   PRINT "stampa dati di carico"
   IF FO$ = "CON" THEN CLS  ELSE GOSUB InCorso
   
   PRINT #2, : PRINT #2, : PRINT #2,
   PRINT #2, "SCHEMA DI CARICO N."; CAR
   CALL OutputCarRet(SM, Wnkl, IV, IM, IN, IO, PZ, RZ, KZ, NP(), V$(), NB(), NF(), ZA(), ZI(), Q(), QF(), MN(), MNF(), FYN(), FYF(), FX())
   
   IF FO$ = "CON" THEN
      CALL Aspetta(2): GOSUB NomeProgramma
   ELSE
      GOSUB Completata
   END IF
RETURN

' -------------------------------- ConvCarRet ----------------------------------
'  Routine per la conversione dei carichi sui nodi e sulle aste
' ------------------------------------------------------------------------------
ConvCarRet:
   PRINT "conversione dati di carico": GOSUB InCorso
  
   CALL ConverteCarRet(RZ, PZ, Wnkl, NB(), NF(), ZA(), ZI(), IV, IM, IN, IO, Q(), QF(), MN(), MNF(), FYN(), FYF(), FX(), KZ, V$(), NI(), P(), F())
  
   GOSUB Completata
RETURN

' -------------------------------- TerminiNoti ---------------------------------
'  Routine per il calcolo dei carichi nodali
' ------------------------------------------------------------------------------
TerminiNoti:
   PRINT "preparazione termini noti": GOSUB InCorso

   CALL TermNoto(TrRig, IZ, JZ, X(), Y(), ZA(), ZI(), E, TA$(), Estr(), NT(), K(), B(), TrRig(), N(), P(), F(), R())

   GOSUB Completata
RETURN

' -------------------------------- Risolvi -------------------------------------
'  Routine per la risoluzione mediante sostituzione all'indietro
' ------------------------------------------------------------------------------
Risolvi:
   PRINT "risoluzione": GOSUB InCorso
  
   CALL Risolve.K2(KTT(), KTA(), KAA(), SB, 0, R())
  
   GOSUB Completata
RETURN

' -------------------------------- CaratSollec ---------------------------------
'  Routine per il calcolo delle azioni sulle aste
' ------------------------------------------------------------------------------
CaratSollec:
   PRINT "calcolo caratt. sollecitazione": GOSUB InCorso
 
   CALL CarSoll(DefTag, TrRig, JZ, X(), Y(), ZA(), ZAT(), ZI(), Easta(), Gasta(), TA$(), Estr(), NT(), K(), B(), TrRig(), N(), P(), F(), R(), N1(), T1(), M1(), N2(), T2(), M2())

   GOSUB Completata
RETURN

' -------------------------------- ConvRisultRet -------------------------------
'  Routine per la conversione delle caratteristiche di sollecitazione
' ------------------------------------------------------------------------------
ConvRisultRet:
   PRINT "conversione caratt. sollecitaz.": GOSUB InCorso
 
   CALL ConverteRisRet(RZ, PZ, Wnkl, DefEst, NB(), NR(), NF(), ZA(), ZI(), D(), MS(), MD(), TS(), TD(), MP(), MT(), TR(), SN(), MFS(), MFD(), TFS(), TFD(), KZ, NI(), N(), R(), F(), M1(), M2(), T1(), T2(), N1())
   
   GOSUB Completata
RETURN

' -------------------------------- StampaRisultRet -----------------------------
'  Routine per la stampa dei risultati
' ------------------------------------------------------------------------------
StampaRisultRet:
   IF Outp = -1 THEN
      PRINT "stampa inviluppo modale"
   ELSE
      PRINT "stampa risultati"
   END IF
   IF FO$ = "CON" THEN CLS  ELSE GOSUB InCorso
 
   IF Outp = -1 THEN
      PRINT #2, : PRINT #2, : PRINT #2,
      PRINT #2, "ANALISI MODALE - inviluppo dei modi"
   END IF
   REDIM SpoRelTel(KZ)
   SpoRelTel(1) = D(1)
   FOR K = 2 TO KZ
      SpoRelTel(K) = D(K) - D(K - 1)
   NEXT K
   CALL OutputSpoOriz(KZ, D(), SpoRelTel())
   CALL OutputRisRet(SM, Outp, RZ, PZ, Wnkl, NP(), NB(), NR(), NF(), ZA(), ZI(), MS(), MD(), TS(), TD(), MP(), MT(), TR(), SN(), MFS(), MFD(), TFS(), TFD(), IZ, KZ, NL, V$(), N(), R(), F())
   GOSUB OutputIpotesiRet
  
   IF FO$ = "CON" THEN
      LOCATE 24, 1
   ELSE
      GOSUB Completata
   END IF
RETURN

' ................................ OutputIpotesiRet ............................
'  Routine per la stampa delle note sulle ipotesi di calcolo
'
OutputIpotesiRet:
   PRINT #2, : PRINT #2,
   PRINT #2, "Il calcolo e' stato eseguito :"
   IF DefEst = 1 THEN
      PRINT #2, "- tenendo conto della deformazione estensionale dei pilastri"
   ELSE
      PRINT #2, "- trascurando la deformazione estensionale dei pilastri"
   END IF
   IF DefTag = 1 THEN
      PRINT #2, "- tenendo conto della deformazione a taglio di travi e pilastri"
      IF Wnkl = 1 THEN
         PRINT #2, "- trascurando la deformazione a taglio della trave di fondazione"
      END IF
   ELSE
      PRINT #2, "- trascurando la deformazione a taglio delle aste"
   END IF
   PRINT #2, : PRINT #2,
RETURN

' -------------------------------- CalcolaRigidezze ----------------------------
'  Routine per il calcolo delle rigidezze (nel caso di opzione /R)
' ------------------------------------------------------------------------------
CalcolaRigidezze:
   IF PZ <> KZ THEN
      PRINT #2, "Impossibile effettuare il calcolo delle rigidezze perch il numero di piani"
      PRINT #2, " diverso dal numero di traversi inestensibili"
      RETURN
   END IF
   DIM Taglio(Ntel, PZ), Spost(PZ)
   FOR P = 1 TO PZ
      PriPil = 1
      FOR T = 1 TO Ntel
         Taglio(T, P) = 0
         FOR R = PriPil TO Npil(T)
            Taglio(T, P) = Taglio(T, P) + TR(R, P)
         NEXT R
         PriPil = Npil(T) + 1
      NEXT T
   NEXT P
   Spost(1) = D(1)
   FOR P = 2 TO PZ
      Spost(P) = D(P) - D(P - 1)
   NEXT P
   PRINT #2, "    RIGIDEZZE DEI TELAI:"
   PRINT #2,
   IF SM <> 0 THEN
      PRINT #2, "    telaio  piano       Taglio (t)     Spost.rel. (mm)     Rigidezza (t/mm)"
   ELSE
      PRINT #2, "    telaio  piano       Taglio (kN)    Spost.rel. (mm)     Rigidezza (kN/mm)"
   END IF
   PRINT #2,
   IF SM <> 0 THEN
      A$ = "      ##     ##        ######.##          "
      B$ = "             ##        ######.##          "
      C$ = "###.###             ####.###"
   ELSE
      A$ = "      ##     ##        #######.#          "
      B$ = "             ##        #######.#          "
      C$ = "###.###             #####.##"
   END IF
   FOR T = 1 TO Ntel
      FOR P = PZ TO 1 STEP -1
         Rigid = Taglio(T, P) / Spost(P) / 1000
         IF P = PZ THEN
            PRINT #2, USING A$; T; P; Taglio(T, P);
         ELSE
            PRINT #2, USING B$; P; Taglio(T, P);
         END IF
         PRINT #2, USING C$; Spost(P) * 1000; Rigid
      NEXT P
      PRINT #2,
   NEXT T
   PRINT #2, : PRINT #2,
RETURN

