'                                  TELSPA.BAS
'
'  Programma per la risoluzione statica o l'analisi modale di insiemi spaziali
'  di telai piani generici oppure con maglie di forma rettangolare o trapezia
'
'  tipi di telai:       G  =  generici
'                       R  =  a maglie rettangolari o trapezie
'
'  tipi di aste:        T  =  trave alla De Saint Venant, incastrata ai nodi
'                       altri tipi - vedi TELGEN
'
'  vincoli esterni:     vincoli rigidi o elasticamente cedevoli
'                       vincoli mutui allo spostamento orizzontale
'
'  carichi:             sul complesso spaziale:
'                          forza comunque orientata e momento torcente
'
'                       sulle strutture piane componenti:
'                          azioni concentrate sui nodi
'                          carichi uniformemente distribuiti sulle aste
'
'  tipo di analisi      statica oppure modale (con SRSS o con CQC)
'
'  ver. 2.3                                                       rev. 02.99

' ------------------ Dichiarazione delle procedure utilizzate ------------------

' procedure del file MATRIX.BAS
DECLARE SUB AzzeraMat (A!())
DECLARE SUB DuplicaMat (A!(), B!())
DECLARE SUB InputMat (A!(), F!)
DECLARE SUB InserMat (A!(), B!(), RI!, CI!)
DECLARE SUB ProdottoMat (A!(), B!(), C!())
DECLARE SUB WriteMat (A!(), F!)

' procedure del file SOLSIST.BAS
DECLARE SUB Triang (A!(), IPVT!())
DECLARE SUB Triang.K2 (KTT!(), KTA!(), KAA!(), SB!, RID!)
DECLARE SUB Risolve (A!(), B!(), IPVT!())
DECLARE SUB Risolve.K2 (KTT!(), KTA!(), KAA!(), SB!, RID!, B!())

' procedure del file PROCOM.BAS
DECLARE SUB Aspetta (Sec!)
DECLARE SUB CarSoll (DefTag!, TrRig!, JZ!, X!(), Y!(), ZA!(), ZAT!(), ZI!(), Easta!(), Gasta!(), TA$(), Estr!(), NT!(), K!(), B!(), TrRig!(), N!(), P!(), F!(), R!(), N1!(), T1!(), M1!(), N2!(), T2!(), M2!())
DECLARE SUB DefinisceIO (Prog$, FI$, FO$, FT$)
DECLARE SUB DefinisceMO (Prog$, DefEst!, DefTag!, Contr!)
DECLARE SUB DefinisceSM (SM!)
DECLARE SUB DisegnaSchema (IZ!, JZ!, X!(), Y!(), TA$(), Estr!())
DECLARE SUB FormaMat (NumNodi!, NumEstr!(), N!(), Kg!(), NL!, SB!, KTT!(), KTA!(), KAA!())
DECLARE SUB Incognite (IZ!, JZ!, KZ!, TA$(), Estr!(), V$(), NI!(), N!(), NL!, NZ!, SB!)
DECLARE SUB InputIntestaz (Contr!, Intest$)
DECLARE SUB InputSezioni (Contr!, LZ!, FS$(), ZB!(), ZH!(), ZA!(), ZI!(), ZAT!(), ER!)
DECLARE SUB InputModElast (Contr!, SM!, EL!, GL!, E!, G!)
DECLARE SUB MatAstaGlobale (TA$, DefTag!, X1!, X2!, Y1!, Y2!, A!, AT!, Iner!, K!, B!, TrRigSin!, TrRigDes!, E!, G!, Lun!, SnAl!, CsAl!, Ld!(), Kg!())
DECLARE SUB OutputIntestaz (NomeProg$, FI$, Intest$)
DECLARE SUB OutputModElast (SM!, DefTag!, EL!, GL!)
DECLARE SUB OutputSpoOriz (KZ!, SpoAss!(), SpoRel!())
DECLARE SUB TermNoto (TrRig!, IZ!, JZ!, X!(), Y!(), ZA!(), ZI!(), E!, TA$(), Estr!(), NT!(), K!(), B!(), TrRig!(), N!(), P!(), F!(), R!())
DECLARE SUB VincoliCed (IZ!, N!(), KV!(), NL!, SB!, KTT!(), KTA!(), KAA!())

' procedure del file PROGEN.BAS
DECLARE SUB InputAsteGen (Incr!, Contr!, JZ!, LZ!, Estr!(), NT!(), TA$(), Wnkl!, B!(), K!(), TrRig!, TrRig!(), Ates!(), Acom!(), ER!)
DECLARE SUB InputCarGen (Incr!, MoAn!, Contr!, IZ!, JZ!, QN!, QA!, nMod!, F!(), P!(), nPassi!, PassoOut!, ER!)
DECLARE SUB InputNodiGen (Contr!, IZ!, X!(), Y!(), V$(), VCed!, KV!(), ER!)
DECLARE SUB InputTraversiGen (Contr!, KZ!, NI!())
DECLARE SUB OutputAsteGen (Incr!, SM!, DefTag!, Wnkl!, TrRig!, JZ!, NT!(), TA$(), Estr!(), FS$(), ZB!(), ZH!(), ZA!(), ZAT!(), ZI!(), B!(), K!(), TrRig!(), Ates!(), Acom!())
DECLARE SUB OutputCarGen (SM!, CAR!, IZ!, JZ!, QN!, QA!, F!(), P!())
DECLARE SUB OutputNodiGen (IZ!, X!(), Y!(), V$(), VCed!, KV!())
DECLARE SUB OutputRisGen (SM!, Outp!, IZ!, JZ!, KZ!, NL!, V$(), KV!(), NI!(), N!(), R!(), M1!(), M2!(), T1!(), T2!(), N1!(), N2!(), F!())

' procedure del file PRORET.BAS
DECLARE SUB ContrRitti (PZ!, RZ!, NP!(), NB!(), NR!(), ZA!(), ZI!(), ER!)
DECLARE SUB ConverteCarRet (RZ!, PZ!, Wnkl!, NB!(), NF!(), ZA!(), ZI!(), IV!, IM!, IN!, IO!, Q!(), QF!(), MN!(), MNF!(), FYN!(), FYF!(), FX!(), KZ!, V$(), NI!(), P!(), F!())
DECLARE SUB ConverteGeomRet (PZ!, RZ!, Wnkl!, TrRig!, DefEst!, DL!(), H!(), NB!(), NR!(), NF!(), BF!(), KF!(), TrRigTra!(), TrRigPil!(), TrRigFon!(), ZA!(), ZI!(), IZ!, JZ!, KZ!, X!(), Y!(), V$(), NI!(), TA$(), Estr!(), NT!(), B!(), K!(), TrRig!())
DECLARE SUB ConverteRisRet (RZ!, PZ!, Wnkl!, DefEst!, NB!(), NR!(), NF!(), ZA!(), ZI!(), D!(), MS!(), MD!(), TS!(), TD!(), MP!(), MT!(), TR!(), SN!(), MFS!(), MFD!(), TFS!(), TFD!(), KZ!, NI!(), N!(), R!(), F!(), M1!(), M2!(), T1!(), T2!(), N1!())
DECLARE SUB InputAltezze (Contr!, PZ!, RZ!, AltVar!, HI!(), H!())
DECLARE SUB InputAsteRet (Contr!, Wnkl!, TrRig!, PZ!, RZ!, LZ!, NP!(), NB!(), NR!(), NF!(), BF!(), KF!(), TrRigTra!(), TrRigPil!(), TrRigFon!(), ER!)
DECLARE SUB InputCarRet (Contr!, Wnkl!, IV!, IM!, IN!, IO!, PZ!, RZ!, KZ!, NP!(), V$(), NB!(), NF!(), ZA!(), ZI!(), Q!(), QF!(), MN!(), MNF!(), FYN!(), FYF!(), FX!())
DECLARE SUB InputLuci (Contr!, Wnkl!, RZ!, DL!())
DECLARE SUB InputNumRitti (Contr!, RZ!, NP!())
DECLARE SUB OutputAsteRet (SM!, PZ!, RZ!, Wnkl!, TrRig!, NP!(), DL!(), NB!(), NR!(), NF!(), BF!(), KF!(), TrRigTra!(), TrRigPil!(), TrRigFon!(), FS$(), ZB!(), ZH!(), ZI!())
DECLARE SUB OutputCarRet (SM!, Wnkl!, IV!, IM!, IN!, IO!, PZ!, RZ!, KZ!, NP!(), V$(), NB!(), NF!(), ZA!(), ZI!(), Q!(), QF!(), MN!(), MNF!(), FYN!(), FYF!(), FX!())
DECLARE SUB OutputRisRet (SM!, Outp!, RZ!, PZ!, Wnkl!, NP!(), NB!(), NR!(), NF!(), ZA!(), ZI!(), MS!(), MD!(), TS!(), TD!(), MP!(), MT!(), TR!(), SN!(), MFS!(), MFD!(), TFS!(), TFD!(), IZ!, KZ!, NL!, V$(), N!(), R!(), F!())
DECLARE SUB OutputPiani (AltVar!, PZ!, RZ!, NP!(), HI!(), H!())
DECLARE SUB OutputTabSez (DefTag!, LZ!, FS$(), ZB!(), ZH!(), ZA!(), ZAT!(), ZI!())

' procedure del file MOANA.BAS
DECLARE SUB AnalisiModale3D (Klat!(), MA!(), PSZ!, nMod!, AngSisma!, EigVal!(), EigVet!(), T!(), Part!(), Forze!(), Energia!())
DECLARE SUB InputSpettro (Contr!, SpePar!(), SpeT!(), SpeR!())
DECLARE SUB OutputModi3D (SM!, PSZ!, J!, EigVal!(), EigVet!(), T!(), Beta!(), Part!(), EnIso!, EnTot!, Smorz!)
DECLARE SUB OutputSpettro (SpePar!(), SpeT!(), SpeR!())
DECLARE FUNCTION Risposta! (T!, SpePar!())
DECLARE FUNCTION Risposta2! (T!, nCop!, SpeT!(), SpeR!())

' procedure di questo file
DECLARE SUB CarSollTrascurG (SM!, JZ!, N1!(), T1!(), M1!(), N2!(), T2!(), M2!(), ValTrasc!)
DECLARE SUB CarSollTrascurR (SM!, RZ!, PZ!, Wnkl!, MS!(), MD!(), TS!(), TD!(), MP!(), MT!(), TR!(), SN!(), MFS!(), MFD!(), TFS!(), TFD!(), ValTrasc!)
DECLARE SUB CoeffInvMod (nMod!, Omega!(), zita!, Rgrec!())
DECLARE SUB InviluppoMod (SovrCQC!, nMod!, Rgrec!(), R0!)
DECLARE SUB InvModCom (SovrCQC!, nMod!, Rgrec!(), KZ!, SpoAss!(), SpoRel!())
DECLARE SUB InvModGen (SovrCQC!, nMod!, Rgrec!(), NZ!, JZ!, R!(), M1!(), M2!(), T1!(), T2!(), N1!(), N2!())
DECLARE SUB InvModRet (SovrCQC!, nMod!, Rgrec!(), KZ!, RZ!, PZ!, Wnkl!, D!(), MS!(), MD!(), TS!(), TD!(), MP!(), MT!(), TR!(), SN!(), MFS!(), MFD!(), TFS!(), TFD!())
DECLARE SUB OutputForzeSpa (SM!, PSZ!, Xm!(), Ym!(), Fsx!(), Fsy!(), Msz!())
DECLARE SUB OutputMasseSpa (PSZ!, Xm!(), Ym!(), Mas!(), Mro!())
DECLARE SUB OutputSpostSpa (PSZ!, Rs!())
DECLARE SUB WriteRisCom (Buff!, KZ!, SpoAss!(), SpoRel!())
DECLARE SUB WriteRisGen (Buff!, NZ!, JZ!, R!(), M1!(), M2!(), T1!(), T2!(), N1!(), N2!())
DECLARE SUB WriteRisRet (Buff!, KZ!, RZ!, PZ!, Wnkl!, D!(), MS!(), MD!(), TS!(), TD!(), MP!(), MT!(), TR!(), SN!(), MFS!(), MFD!(), TFS!(), TFD!())

CLEAR , , 2000
' ------------------ Dimensionamento delle variabili con indice ----------------

OPTION BASE 1            ' definisce l'estremo inferiore dell'indice degli array
DIM Ld(6, 6), Kg(6, 6)   ' array statici
                         ' array dinamici: sono dimensionati in apposite routine


' --------------------------- Programma principale -----------------------------

' definizione di costanti
CONST PiGr = 3.141592654#
CONST nMaxEstr = 2      ' numero massimo di estremit degi elementi
                        '* per ora ipotizzo solo aste a 2 nodi
CONST Incr = 0          ' non abilito analisi incrementale

GOSUB Presentazione
GOSUB Opzioni
CALL DefinisceIO("SPA", FI$, FO$, FT$)
CALL DefinisceMO("SPA", DefEst, DefTag, Contr)
CALL DefinisceSM(SM)
OPEN FI$ FOR INPUT AS #1
OPEN FO$ FOR OUTPUT AS #2
GOSUB LeggeGeomSpa
GOSUB StampaGeomSpa
OPEN FT$ + "MATR.SPA" FOR OUTPUT AS #3
OPEN FT$ + "GEOM.SPA" FOR OUTPUT AS #4
FOR T = 1 TO TZ
   GOSUB LeggeGeomTel
   GOSUB ConverteGeometria
   GOSUB LeggeCorrispTravImp
   GOSUB StampaGeomTel
   GOSUB Schema
   GOSUB DefinisceIncognite
   GOSUB MatRigTel
   GOSUB TriangTel
   GOSUB FormaMatSpa
   GOSUB WriteGeomTel
NEXT T
CLOSE #3
CLOSE #4
GOSUB TriangSpa

CAR = 0
DO WHILE NOT EOF(1)
   CAR = CAR + 1
   ICS = 0: ICT = 0
   INPUT #1, ICS
   IF NOT (EOF(1)) THEN INPUT #1, ICT
   IF ICS <> 0 THEN
      GOSUB LeggeCarSpa
      GOSUB StampaCarSpa
   END IF
   IF ICS > 1 THEN
      GOSUB AnalisiModale
      GOSUB OutputIpotesi
   END IF
   IF ICS = 1 OR ICT = 1 THEN GOSUB SommaCarSpa
   IF ICT = 1 THEN
      IF ICS <> 1 THEN
         PRINT #2, : PRINT #2, : PRINT #2,
         PRINT #2, "SCHEMA DI CARICO N."; CAR
      END IF
      OPEN FT$ + "MATR.SPA" FOR INPUT AS #3
      OPEN FT$ + "GEOM.SPA" FOR INPUT AS #4
      OPEN FT$ + "CAR.SPA" FOR OUTPUT AS #5
      FOR T = 1 TO TZ
         GOSUB ReadGeomTel
         GOSUB LeggeCarTel
         GOSUB ConverteCarichi
         GOSUB StampaCarTel
         GOSUB TerminiNoti
         GOSUB TriangR
         GOSUB WriteCar
         GOSUB SommaCarTel
      NEXT T
      CLOSE #3
      CLOSE #4
      CLOSE #5
   END IF
   IF ICS = 1 OR ICT = 1 THEN
      GOSUB RisolveSistema
      GOSUB StampaRisultSpa
      OPEN FT$ + "MATR.SPA" FOR INPUT AS #3
      OPEN FT$ + "GEOM.SPA" FOR INPUT AS #4
      IF ICT = 1 THEN OPEN FT$ + "CAR.SPA" FOR INPUT AS #5
      FOR T = 1 TO TZ
         GOSUB ReadGeomTel
         Izer = ICT: GOSUB ReadCar
         GOSUB Spostamenti
         GOSUB CaratSollec
         GOSUB ConverteRisult
         Outp = 0:   GOSUB StampaRisultTel
      NEXT T
      GOSUB OutputIpotesi
      CLOSE #3
      CLOSE #4
      IF ICT = 1 THEN
         CLOSE #5
         KILL FT$ + "CAR.SPA"
      END IF
   END IF
LOOP
GOSUB EraseFiles
PRINT
PRINT "elaborazione terminata"
PRINT
END


' -------------------------------- DimXXXXXX -----------------------------------
'  Gruppo di routine per il dimensionamento degli array dinamici
' ------------------------------------------------------------------------------

' ................................ DimCarGen ...................................
'  Routine per il dimensionamento degli array relativi a carichi
'
DimCarGen:
   REDIM F(IZ, 3), P(JZ, 2)
RETURN

' ................................ DimCarRet ...................................
'  Routine per il dimensionamento degli array relativi a carichi
'
DimCarRet:
   REDIM Q(RZ - 1, PZ), MN(RZ, PZ), FYN(RZ, PZ), FX(KZ)
   IF Wnkl = 1 THEN REDIM QF(0 TO RZ), MNF(0 TO RZ + 1), FYF(0 TO RZ + 1)
RETURN

' ................................ DimGeomGen ..................................
'  Routine per il dimensionamento degli array relativi alla geometria
'
DimGeomGen:
   REDIM X(IZ), Y(IZ), V$(IZ), KV(IZ, 3), NI(IZ + KZ), PI(KZ)
   REDIM Estr(JZ, nMaxEstr)
   REDIM TA$(JZ), NT(JZ), B(JZ), K(JZ), TrRig(JZ, 2)
   REDIM Ates(0 TO 0), Acom(0 TO 0)
   REDIM N(IZ, 3)
RETURN

' ................................ DimGeomRet ..................................
'  Routine per il dimensionamento degli array relativi alla geometria
'
DimGeomRet:
   REDIM NP(RZ), HI(PZ), H(RZ, 0 TO PZ), DL(0 TO RZ)
   REDIM NB(RZ - 1, PZ), NR(RZ, PZ)
   IF Wnkl = 1 THEN REDIM NF(0 TO RZ), BF(0 TO RZ), KF(0 TO RZ)
   IF TrRig <> 0 THEN REDIM TrRigTra(RZ - 1, PZ, 2), TrRigPil(RZ, PZ, 2)
   IF TrRig = 2 THEN REDIM TrRigFon(0 TO RZ, 2)
RETURN

' ................................ DimGeomSpa ..................................
'  Routine per il dimensionamento degli array relativi alla geometria
'  dell'insieme spaziale di telai piani
'
DimGeomSpa:
   DIM TT$(TZ), X0(TZ), Y0(TZ), Alfa(TZ)
   DIM Xm(PSZ), Ym(PSZ), Fsx(PSZ), Fsy(PSZ), Msz(PSZ)
   DIM Mas(PSZ), Mro(PSZ)
   nGdL = 3 * PSZ
   DIM Ks(nGdL, nGdL), Rs(nGdL), IPVT(nGdL)
   DIM Klat(nGdL, nGdL), MA(nGdL, nGdL)
   REDIM SpePar(12), SpeT(1), SpeR(1)
   REDIM SpoAssTel(1), SpoRelTel(1)
RETURN

' ................................ DimMatRig ...................................
'  Routine per il dimensionamento di matrice di rigidezza e vettore carico
'
DimMatRig:
   IF NL = 0 THEN
      REDIM KTT(0 TO 0, 0 TO 0), KTA(0 TO 0, NZ - NL)
   ELSE
      REDIM KTT(NL, NL), KTA(NL, NZ - NL)
   END IF
   REDIM KAA(NZ - NL, SB), R(NZ), Rt(KZ), Ra(NZ - KZ)
RETURN

' ................................ DimRisGen ...................................
'  Routine per il dimensionamento degli array relativi a risultati
'
DimRisGen:
   REDIM St(KZ), M1(JZ), M2(JZ), T1(JZ), T2(JZ), N1(JZ), N2(JZ)
RETURN

' ................................ DimRisRet ...................................
'  Routine per il dimensionamento degli array relativi a risultati
'
DimRisRet:
   REDIM D(KZ), MD(RZ - 1, PZ), MS(RZ - 1, PZ), TD(RZ - 1, PZ), TS(RZ - 1, PZ)
   REDIM MP(RZ, PZ), MT(RZ, PZ), TR(RZ, PZ), SN(RZ, PZ)
   IF Wnkl = 1 THEN REDIM MFD(0 TO RZ), MFS(0 TO RZ), TFD(0 TO RZ), TFS(0 TO RZ)
RETURN

' ................................ DimTabSez ...................................
'  Routine per il dimensionamento degli array relativi ai tipi di sezione
'
DimTabSez:
   DIM FS$(LZ), ZB(LZ), ZH(LZ), ZA(LZ), ZI(LZ), ZAT(LZ)
RETURN

 
' -------------------------------- Presentazione -------------------------------
'  Routine per la visualizzazione della schermata di presentazione del
'  programma
' ------------------------------------------------------------------------------
Presentazione:
   CLS
   PRINT TAB(31); "TELSPA 2.3 - rev. 02.99"
   PRINT
   PRINT "       Le ipotesi di calcolo e le modalit operative  del programma TELSPA"
   PRINT "       (versione base) sono riportate nella terza parte, cap.7, del libro:"
   PRINT "       A.Ghersi, R.Coraggio,  Il personal computer nel calcolo di edifici."
   PRINT "       Analisi matriciale di strutture intelaiate, CUEN, Napoli, 1990."
   PRINT
   PRINT "       Informazioni  sulla versione 2.3  sono fornite  nel file TELSPA.DOC"
   PRINT
   PRINT
RETURN


' -------------------------------- Opzioni -------------------------------------
'  Routine che individua l'opzione output ridotto definita nel mandare in
'  esecuzione il programma
' ------------------------------------------------------------------------------
Opzioni:
   C$ = UCASE$(COMMAND$)
   ' output ridotto
   IF INSTR(C$, "/OR") <> 0 THEN
      OutRid = 1
   ELSE
      OutRid = 0
   END IF
   ' calcolo rigidezza
   IF INSTR(C$, "/RIG") <> 0 THEN
      StampaRig = 1
   ELSE
      StampaRig = 0
   END IF
   ' tipo di sovrapposizione modale
   IF INSTR(C$, "/S=CQC") <> 0 THEN
      SovrCQC = 1
   ELSE
      SovrCQC = 0
   END IF
   ' rotazione degli impalcati impedita
   IF INSTR(C$, "NOROT") <> 0 THEN
      NoRotaz = 1
   ELSE
      NoRotaz = 0
   END IF
RETURN


' -------------------------------- Completata ----------------------------------
'  Routine per l'indicazione della fine di una fase
' ------------------------------------------------------------------------------
Completata:
   LOCATE CSRLIN - 1, 44
   PRINT "completata            "
RETURN


' -------------------------------- InCorso -------------------------------------
'  Routine per l'indicazione dell'inizio di una fase
' ------------------------------------------------------------------------------
InCorso:
   LOCATE CSRLIN - 1, 44
   PRINT "in corso            "
RETURN


' -------------------------------- NomeProgramma -------------------------------
'  Routine per la visualizzazione dell'intestazione del programma
' ------------------------------------------------------------------------------
NomeProgramma:
   CLS
   PRINT "TELSPA 2.0 - rev. 05.94"
   PRINT
RETURN


' -------------------------------- LeggeGeomSpa --------------------------------
'  Routine per la lettura dei dati geometrici ed elastici dell'insieme spaziale
' ------------------------------------------------------------------------------
LeggeGeomSpa:
   GOSUB NomeProgramma
   PRINT "lettura geometria insieme spaziale": GOSUB InCorso
  
   ER = 0
   CALL InputIntestaz(Contr, IT$)
   GOSUB InputIndSpa
   GOSUB DimGeomSpa
   GOSUB DimTabSez
   GOSUB InputPosTelai
   CALL InputModElast(Contr, SM, EL, GL, E, G)
   CALL InputSezioni(Contr, LZ, FS$(), ZB(), ZH(), ZA(), ZI(), ZAT(), ER)
   IF ER = 1 THEN END
 
   IF NOT (Contr = 1 AND FO$ = "CON") THEN GOSUB Completata
RETURN

' ................................ InputIndSpa .................................
'  Routine per la lettura delle indicazioni generali dell'insieme spaziale;
'
InputIndSpa:
   INPUT #1, TZ, PSZ, LZ
   IF Contr = 1 THEN
      PRINT #2, "numero di telai:"; TZ
      PRINT #2, "numero di impalcati:"; PSZ
      PRINT #2, "numero di tipi di sezione:"; LZ
   END IF
RETURN

' ................................ InputPosTelai ...............................
'  Routine per la lettura della posizione dei telai piani che compongono
'  l'insieme spaziale
'
InputPosTelai:
   TelRet = 0
   TelVCed = 0
   TelWnkl = 0
   FOR T = 1 TO TZ
      INPUT #1, TT$(T), X0(T), Y0(T), Direz$
      TT$(T) = UCASE$(TT$(T))
      IF TT$(T) = "R" THEN TelRet = 1
      SELECT CASE Direz$
         CASE "X", "x"
            Alfa(T) = 0
         CASE "Y", "y"
            Alfa(T) = 90
         CASE ELSE
            Alfa(T) = VAL(Direz$)
      END SELECT
      IF Contr = 1 THEN
         PRINT #2, "Telaio: "; T
         PRINT #2, "   Tipo:"; TT$(T),
         PRINT #2, "x ="; X0(T), "y ="; Y0(T), "Direzione (o angolo) ="; Direz$
      END IF
      IF TT$(T) <> "G" AND TT$(T) <> "R" THEN
         ER = 1
         PRINT "Il codice "; TT$(T); " che identifica la tipologia del telaio"; T;
         PRINT " non  ammissibile"
      END IF
   NEXT T
RETURN


' -------------------------------- StampaGeomSpa -------------------------------
'  Routine per la stampa dei dati geometrici ed elastici dell'insieme spaziale
' ------------------------------------------------------------------------------
StampaGeomSpa:
   PRINT "stampa  geometria insieme spaziale"
   IF FO$ = "CON" THEN CLS  ELSE GOSUB InCorso

   CALL OutputIntestaz("TELSPA 2.3", FI$, IT$)
   GOSUB OutputPosTelai
   CALL OutputTabSez(DefTag, LZ, FS$(), ZB(), ZH(), ZA(), ZAT(), ZI())
   CALL OutputModElast(SM, DefTag, EL, GL)
  
   IF FO$ = "CON" THEN
      CALL Aspetta(2): GOSUB NomeProgramma
   ELSE
      GOSUB Completata
   END IF
RETURN

' ................................ OutputPosTelai ..............................
'  Routine per la stampa della posizione dei telai piani che compongono
'  l'insieme spaziale
'
OutputPosTelai:
   PRINT #2, "DATI GEOMETRICI DELL'INSIEME SPAZIALE"
   PRINT #2,
   PRINT #2, "telaio   tipo    x (m)     y (m)   angolo (gradi)"
   PRINT #2,
   FOR T = 1 TO TZ
      PRINT #2, USING " ###       \\  "; T; TT$(T);
      PRINT #2, USING "###.##     ###.##        ###.##"; X0(T); Y0(T); Alfa(T)
   NEXT T
RETURN


' -------------------------------- LeggeGeomTel --------------------------------
' Routine per la lettura dei dati geometrici delle strutture piane componenti
' l'insieme spaziale
' ------------------------------------------------------------------------------
LeggeGeomTel:
   GOSUB NomeProgramma
   PRINT
   PRINT "telaio n."; T; "     tipo: "; TT$(T)
   PRINT
   PRINT "lettura dati geometrici": GOSUB InCorso
  
   ER = 0
   SELECT CASE TT$(T)
      CASE "G"
         ' telaio generico
         GOSUB InputIndGen
         GOSUB DimGeomGen
         CALL InputNodiGen(Contr, IZ, X(), Y(), V$(), VCed, KV(), ER)
         CALL InputTraversiGen(Contr, KZ, NI())
         CALL InputAsteGen(Incr, Contr, JZ, LZ, Estr(), NT(), TA$(), Wnkl, B(), K(), TrRig, TrRig(), Ates(), Acom(), ER)
      CASE "R"
         ' telaio a maglie trapezie
         GOSUB InputIndRet
         IZ = RZ * (PZ + 1) + 2
         JZ = (2 * RZ - 1) * PZ + RZ + 1     ' massimo numero di aste possibile
         KZ = INT(RZ * PZ / 2)               ' massimo numero di traversi possibile
         VCed = 0
         GOSUB DimGeomRet
         GOSUB DimGeomGen: REDIM KV(1, 1)
         CALL InputNumRitti(Contr, RZ, NP())
         CALL InputAltezze(Contr, PZ, RZ, AltVar, HI(), H())
         CALL InputLuci(Contr, Wnkl, RZ, DL())
         CALL InputAsteRet(Contr, Wnkl, TrRig, PZ, RZ, LZ, NP(), NB(), NR(), NF(), BF(), KF(), TrRigTra(), TrRigPil(), TrRigFon(), ER)
         IF ER = 1 THEN END
         IF DefEst = 0 THEN CALL ContrRitti(PZ, RZ, NP(), NB(), NR(), ZA(), ZI(), ER)
   END SELECT
   IF ER = 1 THEN END
   IF VCed = 1 THEN TelVCed = 1
   IF Wnkl = 1 THEN TelWnkl = 1
  
   IF NOT (Contr = 1 AND FO$ = "CON") THEN GOSUB Completata
RETURN

' ................................ InputIndGen .................................
'  Routine per la lettura delle indicazioni generali del telaio di tipo G
'
InputIndGen:
   INPUT #1, IZ, JZ, KZ
   IF Contr = 1 THEN
      PRINT #2, "numero di nodi:"; IZ
      PRINT #2, "numero di aste:"; JZ
      PRINT #2, "numero di traversi orizzontali inestensibili:"; KZ
   END IF
RETURN

' ................................ InputIndRet .................................
'  Routine per la lettura delle indicazioni generali del telaio di tipo R
'
InputIndRet:
   INPUT #1, RZ, PZ, Wnkl
   SELECT CASE Wnkl
      CASE 0
         TrRig = 0:  Wnkl = 0
      CASE 1
         TrRig = 0:  Wnkl = 1
      CASE 10
         TrRig = 1:  Wnkl = 0
      CASE 11
         TrRig = 1:  Wnkl = 1
      CASE 21
         TrRig = 2:  Wnkl = 1
   END SELECT
   IF Contr = 1 THEN
      PRINT #2, "numero di pilastri:"; RZ
      PRINT #2, "numero di piani:"; PZ
      IF Wnkl = 1 THEN PRINT #2, "e' presente la trave di fondazione"
      IF TrRig > 0 THEN PRINT #2, "si considerano tratti rigidi agli estremi delle aste"
   END IF
RETURN


' -------------------------------- ConverteGeometria ---------------------------
'  Routine per la conversione dei dati geometrici degli schemi piani
'  a maglie trapezie
' ------------------------------------------------------------------------------
ConverteGeometria:
   IF TT$(T) = "G" THEN RETURN
   PRINT "conversione dati geometrici": GOSUB InCorso
 
   SELECT CASE TT$(T)
      CASE "R"
         ' telaio a maglie trapezie
         CALL ConverteGeomRet(PZ, RZ, Wnkl, TrRig, DefEst, DL(), H(), NB(), NR(), NF(), BF(), KF(), TrRigTra(), TrRigPil(), TrRigFon(), ZA(), ZI(), IZ, JZ, KZ, X(), Y(), V$(), NI(), TA$(), Estr(), NT(), B(), K(), TrRig())
         REDIM PI(KZ)
   END SELECT
 
   GOSUB Completata
RETURN
 

' -------------------------------- LeggeCorrispTravImp -------------------------
'  Routine per la lettura e stampa della corrispondenza traversi impalcati
' ------------------------------------------------------------------------------
LeggeCorrispTravImp:
   PRINT "lettura corrisp. traversi-impalcati": GOSUB InCorso

   ER = 0
   FOR K = 1 TO KZ
      INPUT #1, PI(K)
      IF Contr = 1 THEN
         PRINT #2, "traverso"; K; " appartenente all'impalcato"; PI(K)
      END IF
      IF PI(K) < 1 OR PI(K) > PSZ THEN
         PRINT "Il numero d'ordine"; PI(K);
         PRINT "dell'impalcato corrispondente al traverso"; K; " non  accettabile"
         ER = 1
      END IF
   NEXT K
   IF ER = 1 THEN END
  
   IF NOT (Contr = 1 AND FO$ = "CON") THEN GOSUB Completata
RETURN


' -------------------------------- StampaGeomTel -------------------------------
'  Routine per la stampa dei dati geometrici delle strutture piane componenti
'  l'insieme spaziale
' ------------------------------------------------------------------------------
StampaGeomTel:
   PRINT "stampa dati geometrici"
   IF FO$ = "CON" THEN CLS  ELSE GOSUB InCorso
  
   PRINT #2, : PRINT #2, : PRINT #2,
   PRINT #2, "TELAIO N."; T
   PRINT #2,
   SELECT CASE TT$(T)
      CASE "G"
         ' telaio generico
         CALL OutputNodiGen(IZ, X(), Y(), V$(), VCed, KV())
         CALL OutputAsteGen(Incr, SM, DefTag, Wnkl, TrRig, JZ, NT(), TA$(), Estr(), FS$(), ZB(), ZH(), ZA(), ZAT(), ZI(), B(), K(), TrRig(), Ates(), Acom())
      CASE "R"
         ' telaio a maglie trapezie
         CALL OutputPiani(AltVar, PZ, RZ, NP(), HI(), H())
         CALL OutputAsteRet(SM, PZ, RZ, Wnkl, TrRig, NP(), DL(), NB(), NR(), NF(), BF(), KF(), TrRigTra(), TrRigPil(), TrRigFon(), FS$(), ZB(), ZH(), ZI())
   END SELECT
   '
   PRINT #2, : PRINT #2, : PRINT #2,
   PRINT #2, "CORRISPONDENZA TRA TRAVERSI ED IMPALCATI"
   PRINT #2,
   PRINT #2, "traverso       impalcato"
   FOR K = KZ TO 1 STEP -1
      PRINT #2, USING "  ###            ###"; K; PI(K)
   NEXT K
  
   IF FO$ = "CON" THEN
      CALL Aspetta(2): GOSUB NomeProgramma
   ELSE
      GOSUB Completata
   END IF
RETURN
  

' -------------------------------- Schema --------------------------------------
'  Routine che visualizza lo schema geometrico
' ------------------------------------------------------------------------------
Schema:
   CALL DisegnaSchema(IZ, JZ, X(), Y(), TA$(), Estr())
   GOSUB NomeProgramma
   PRINT
   PRINT "telaio n."; T
RETURN


' -------------------------------- DefinisceIncognite --------------------------
'  Routine per la numerazione delle incognite  e la valutazione della semibanda
'  (gli array che costituiscono la matrice di rigidezza vengono ridimensionati)
' ------------------------------------------------------------------------------
DefinisceIncognite:
   PRINT "numerazione incognite": GOSUB InCorso

   CALL Incognite(IZ, JZ, KZ, TA$(), Estr(), V$(), NI(), N(), NL, NZ, SB)
   GOSUB DimMatRig

   GOSUB Completata
   PRINT "- numero totale di incognite"; NZ;
   IF NL <> 0 THEN PRINT " (tra cui"; NL; "spostamenti di traversi)";
   PRINT
   PRINT "- ampiezza della semibanda  "; SB
RETURN


' -------------------------------- MatRigTel -----------------------------------
'  Routine per la costruzione della matrice di rigidezza delle strutture piane
'  componenti l'insieme spaziale
' ------------------------------------------------------------------------------
MatRigTel:
   PRINT "costruzione matrice rigidezza": GOSUB InCorso
  
   FOR J = 1 TO JZ
      L = NT(J)
      CALL MatAstaGlobale(TA$(J), DefTag, X(Estr(J, 1)), X(Estr(J, 2)), Y(Estr(J, 1)), Y(Estr(J, 2)), ZA(L), ZAT(L), ZI(L), K(J) * 1000, B(J), TrRig(J, 1), TrRig(J, 2), E, G, Lun, SnAl, CsAl, Ld(), Kg())
      REDIM NumEstr(2)
      NumNodi = 2
      NumEstr(1) = Estr(J, 1)
      NumEstr(2) = Estr(J, 2)
      CALL FormaMat(NumNodi, NumEstr(), N(), Kg(), NL, SB, KTT(), KTA(), KAA())
      LOCATE CSRLIN - 1, 44
      PRINT USING "eseguita per il ### %"; J / JZ * 100
   NEXT J
   IF VCed = 1 THEN CALL VincoliCed(IZ, N(), KV(), NL, SB, KTT(), KTA(), KAA())
  
   GOSUB Completata
RETURN


' -------------------------------- TriangTel -----------------------------------
'  Routine che triangolarizza la matrice di rigidezza delle strutture piane
'  componenti l'insieme spaziale
' ------------------------------------------------------------------------------
TriangTel:
   PRINT "triangolar. matrice rigidezza": GOSUB InCorso
 
   CALL Triang.K2(KTT(), KTA(), KAA(), SB, 1)
   GOSUB CompletaMat
 
   GOSUB Completata
RETURN

' ................................ CompletaMat .................................
'  Routine per il riempimento del triangolo inferiore della sottomatrice Ktt
'
CompletaMat:
   FOR R = 2 TO KZ
      FOR C = 1 TO R - 1
         KTT(R, C) = KTT(C, R)
      NEXT C
   NEXT R
RETURN


' -------------------------------- FormaMatSpa ---------------------------------
'  Routine per l'inserimento dei termini delle matrici di rigidezza laterali
'  delle strutture piane componenti nella matrice di rigidezza dell'insieme
'  spaziale.
' ------------------------------------------------------------------------------
FormaMatSpa:
   PRINT "inserimento termini nella matrice spaziale": GOSUB InCorso
  
   ' parametri geometrici del telaio
   Al = Alfa(T) * PiGr / 180
   CsAl = COS(Al)
   SnAl = SIN(Al)
   Dist = X0(T) * SnAl - Y0(T) * CsAl
   ' inserisce termini nella matrice di rigidezza dell'insieme spaziale
   ' N.B. nel caso di rotazione degli impalcati impedita sarebbe stato pi
   '      corretto ridurre i gradi di libert dello schema; per semplicit
   '      ho eliminato i termini misti della matrice di rigidezza e incrementato
   '      il valore dei termini rotazionali
   FOR R = 1 TO KZ
      FOR C = 1 TO KZ
         R1 = PI(R): R2 = PI(R) + PSZ: R3 = PI(R) + 2 * PSZ
         C1 = PI(C): C2 = PI(C) + PSZ: C3 = PI(C) + 2 * PSZ
         K11 = KTT(R, C) * CsAl ^ 2
         K12 = KTT(R, C) * SnAl * CsAl
         K13 = KTT(R, C) * Dist * CsAl
         K22 = KTT(R, C) * SnAl ^ 2
         K23 = KTT(R, C) * Dist * SnAl
         K33 = KTT(R, C) * Dist ^ 2
         Ks(R1, C1) = Ks(R1, C1) + K11
         Ks(R2, C2) = Ks(R2, C2) + K22
         IF NoRotaz = 0 THEN
            Ks(R1, C2) = Ks(R1, C2) + K12
            Ks(R2, C1) = Ks(R2, C1) + K12
            Ks(R1, C3) = Ks(R1, C3) + K13
            Ks(R2, C3) = Ks(R2, C3) + K23
            Ks(R3, C1) = Ks(R3, C1) + K13
            Ks(R3, C2) = Ks(R3, C2) + K23
            Ks(R3, C3) = Ks(R3, C3) + K33
         ELSE
            ' aumento fittiziamente la rigidezza torsionale
            ' per ridurre il periodo torsionale dello schema
            Ks(R3, C3) = Ks(R3, C3) + K33 * 1000
         END IF
      NEXT C
   NEXT R
  
   GOSUB Completata
RETURN


' -------------------------------- TriangSpa -----------------------------------
'  Routine che triangolarizza la matrice di rigidezza dell'insieme spaziale
' ------------------------------------------------------------------------------
TriangSpa:
   PRINT "triangolar. matrice rigidezza spaziale": GOSUB InCorso
  
   CALL DuplicaMat(Ks(), Klat())
   CALL Triang(Ks(), IPVT())
  
   GOSUB Completata
RETURN


' -------------------------------- LeggeCarSpa ---------------------------------
' Routine per la lettura dei dati che definiscono i carichi agenti
' sull'insieme spaziale (masse e spettro nel caso di analisi modale).
' ------------------------------------------------------------------------------
LeggeCarSpa:
   GOSUB NomeProgramma
   PRINT "    Schema di carico n."; CAR
   PRINT "lettura dati di carico insieme spaziale": GOSUB InCorso
  
   IF ICS = 1 THEN
      FOR PS = PSZ TO 1 STEP -1
         INPUT #1, Xm(PS), Ym(PS)
         INPUT #1, Fsx(PS), Fsy(PS), Msz(PS)
         IF Contr = 1 THEN
            PRINT #2, "impalcato"; PS
            PRINT #2, "  x0 ="; Xm(PS), "y0 ="; Ym(PS)
            PRINT #2, "  Fx ="; Fsx(PS), "Fy ="; Fsy(PS), "Mz ="; Msz(PS)
         END IF
      NEXT PS
   ELSE
      IF Contr = 1 THEN PRINT #2, "richiesta analisi modale, con uso di"; ICS; "modi"
      FOR PS = PSZ TO 1 STEP -1
         INPUT #1, Xm(PS), Ym(PS)
         INPUT #1, Mas(PS), Mro(PS)
         IF Contr = 1 THEN
            PRINT #2, "impalcato"; PS
            PRINT #2, "  x0 ="; Xm(PS), "y0 ="; Ym(PS)
            PRINT #2, "Massa ="; Mas(PS), "rho ="; Mro(PS)
         END IF
      NEXT PS
      INPUT #1, Direz$
      IF Contr = 1 THEN PRINT #2, "Direzione (o angolo) sisma ="; Direz$
      SELECT CASE Direz$
         CASE "X", "x"
            AngSisma = 0
         CASE "Y", "y"
            AngSisma = 90
         CASE ELSE
            AngSisma = VAL(Direz$)
      END SELECT
      CALL InputSpettro(Contr, SpePar(), SpeT(), SpeR())
   END IF

   IF NOT (Contr = 1 AND FO$ = "CON") THEN GOSUB Completata
RETURN


' -------------------------------- StampaCarSpa --------------------------------
'  Routine per la stampa dei carichi agenti sull'insieme spaziale
' ------------------------------------------------------------------------------
StampaCarSpa:
   PRINT "stampa dati di carico insieme spaziale"
   IF FO$ = "CON" THEN CLS  ELSE GOSUB InCorso
  
   PRINT #2, : PRINT #2, : PRINT #2,
   IF ICS = 1 THEN
      PRINT #2, "SCHEMA DI CARICO N."; CAR
      CALL OutputForzeSpa(SM, PSZ, Xm(), Ym(), Fsx(), Fsy(), Msz())
   ELSE
      PRINT #2, "SCHEMA DI CARICO N."; CAR; "- ANALISI MODALE"
      CALL OutputMasseSpa(PSZ, Xm(), Ym(), Mas(), Mro())
      PRINT #2, : PRINT #2,
      PRINT #2, USING "Angolo sisma =### gradi"; AngSisma
      CALL OutputSpettro(SpePar(), SpeT(), SpeR())
   END IF
  
   IF FO$ = "CON" THEN
      CALL Aspetta(2): GOSUB NomeProgramma
   ELSE
      GOSUB Completata
   END IF
RETURN


' -------------------------------- SommaCarSpa ---------------------------------
'  Routine che valuta le azioni traslanti di incastro dovute
'  ai carichi applicati in un punto prefissato di ciascun impalcato
' ------------------------------------------------------------------------------
SommaCarSpa:
   IF FO$ = "CON" THEN GOSUB NomeProgramma
   PRINT "valutazione azioni carichi spaziali": GOSUB InCorso
  
   IF ICS = 1 THEN
      FOR PS = 1 TO PSZ
         Rs(PS) = Fsx(PS)
         Rs(PS + PSZ) = Fsy(PS)
         IF NoRotaz = 0 THEN
            Rs(PS + 2 * PSZ) = Msz(PS) + Fsy(PS) * Xm(PS) - Fsx(PS) * Ym(PS)
         END IF
      NEXT PS
   ELSE
      CALL AzzeraMat(Rs())
   END IF
 
   GOSUB Completata
RETURN

' -------------------------------- AnalisiModale -------------------------------
'  Routine per l'analisi modale
' ------------------------------------------------------------------------------
AnalisiModale:
   PRINT "determinazione modi": GOSUB InCorso

   nMod = ICS
   nGdL = 3 * PSZ
   REDIM EigVal(nGdL), EigVet(nGdL, nGdL)
   REDIM T(nMod), Beta(nMod), Part(nMod), Forze(nGdL, nMod), Energia(nMod)
   ' prepara la matrice delle masse
   FOR PS = 1 TO PSZ
      ix = PS:    iy = PS + PSZ:    IZ = PS + 2 * PSZ
      MA(ix, ix) = Mas(PS)
      MA(iy, iy) = Mas(PS)
      MA(IZ, IZ) = Mas(PS) * (Mro(PS) ^ 2 + Xm(PS) ^ 2 + Ym(PS) ^ 2)
      IF NoRotaz = 0 THEN
         MA(ix, IZ) = -Mas(PS) * Ym(PS):  MA(IZ, ix) = MA(ix, IZ)
         MA(iy, IZ) = Mas(PS) * Xm(PS):   MA(IZ, iy) = MA(iy, IZ)
      END IF
   NEXT PS
   CALL AnalisiModale3D(Klat(), MA(), PSZ, nMod, AngSisma, EigVal(), EigVet(), T(), Part(), Forze(), Energia())
   REDIM Omega(nMod)
   FOR J = 1 TO nMod
      Omega(J) = SQR(EigVal(J))
   NEXT J

   GOSUB Completata

   PRINT "stampa modi": GOSUB InCorso

   FOR J = 1 TO nMod
      IF SpePar(1) = 1 THEN
         Beta(J) = Risposta2(T(J), SpePar(2), SpeT(), SpeR())
      ELSE
         Beta(J) = Risposta(T(J), SpePar())
      END IF
      Molt0 = Beta(J) * Part(J) * 9.81
      Molt1 = Molt0 / EigVal(J)
      FOR PS = 1 TO PSZ
         Fsx(PS) = Molt0 * Forze(PS, J)
         Fsy(PS) = Molt0 * Forze(PS + PSZ, J)
         Msz(PS) = Molt0 * Forze(PS + 2 * PSZ, J)
         Msz(PS) = Msz(PS) + Fsx(PS) * Ym(PS) - Fsy(PS) * Xm(PS)
      NEXT PS
      FOR K = 1 TO nGdL
         Rs(K) = Molt1 * EigVet(K, J)
      NEXT K
      CALL OutputModi3D(SM, PSZ, J, EigVal(), EigVet(), T(), Beta(), Part(), EnIso, EnTot, Smorz)
      PRINT #2,
      CALL OutputForzeSpa(SM, PSZ, Xm(), Ym(), Fsx(), Fsy(), Msz())
      CALL OutputSpostSpa(PSZ, Rs())
   NEXT J
   PRINT #2,
   REDIM Rgrec(nMod, nMod)
   IF SovrCQC = 0 THEN
      PRINT #2, "Sovrapposizione modale effettuata col metodo SRSS"
   ELSE
      zita = .05
      CALL CoeffInvMod(nMod, Omega(), zita, Rgrec())
      PRINT #2, "Sovrapposizione modale effettuata col metodo CQC"
      IF OutRid = 0 THEN
         PRINT #2,
         PRINT #2, USING "smorzamento  = ###.## %"; zita * 100
         PRINT #2, "omega =       ";
         FOR m = 1 TO nMod
            PRINT #2, USING "####.##"; Omega(m);
         NEXT m
         PRINT #2,
         FOR I = 1 TO nMod
            IF I = 1 THEN
               PRINT #2, "coefficienti =";
            ELSE
               PRINT #2, "              ";
            END IF
            FOR N = 1 TO nMod
               PRINT #2, USING "###.###"; Rgrec(I, N);
            NEXT N
            PRINT #2,
         NEXT I
      END IF
   END IF

   GOSUB Completata

   OPEN FT$ + "MATR.SPA" FOR INPUT AS #3
   OPEN FT$ + "GEOM.SPA" FOR INPUT AS #4
   FOR T = 1 TO TZ
      GOSUB NomeProgramma
      GOSUB ReadGeomTel
      Izer = 0:   GOSUB ReadCar
      TrascInv = -1
      FOR m = 1 TO nMod
         PRINT "modo"; m
         Molt0 = Beta(m) * Part(m) * 9.81
         Molt1 = Molt0 / EigVal(m)
         I = 0
         FOR K = 1 TO 3 * PSZ
            Rs(K) = Molt1 * EigVet(K, m)
         NEXT K
         GOSUB Spostamenti
         GOSUB CaratSollec
         GOSUB ConverteRisult
         Buff = 10 + m
         OPEN "OutMod" + LTRIM$(STR$(m)) FOR OUTPUT AS #Buff
         Outp = m:   GOSUB StampaRisultTel
         IF NOT ValTrasc THEN TrascInv = 0
         CLOSE #Buff
         REDIM R(NZ), F(IZ, 3)
      NEXT m
      IF TrascInv THEN
         PRINT #2, : PRINT #2, : PRINT #2,
         PRINT #2, "TELAIO N."; T; "- inviluppo dei modi"
         PRINT #2,
         PRINT #2, "I valori di deformazioni e sollecitazioni sono trascurabili"
      ELSE
         PRINT "sovrapposizione modale": GOSUB InCorso
         FOR m = 1 TO nMod
            Buff = 10 + m
            OPEN "OutMod" + LTRIM$(STR$(m)) FOR INPUT AS #Buff
         NEXT m
         CALL InvModCom(SovrCQC, nMod, Rgrec(), KZ, SpoAssTel(), SpoRelTel())
         SELECT CASE TT$(T)
            CASE "G"
               CALL InvModGen(SovrCQC, nMod, Rgrec(), NZ, JZ, R(), M1(), M2(), T1(), T2(), N1(), N2())
            CASE "R"
               CALL InvModRet(SovrCQC, nMod, Rgrec(), KZ, RZ, PZ, Wnkl, D(), MS(), MD(), TS(), TD(), MP(), MT(), TR(), SN(), MFS(), MFD(), TFS(), TFD())
         END SELECT
         FOR m = 1 TO nMod
            Buff = 10 + m
            CLOSE #Buff
         NEXT m
         GOSUB Completata
         Outp = -1:  GOSUB StampaRisultTel
      END IF
   NEXT T
   CLOSE #3
   CLOSE #4
   FOR m = 1 TO nMod
      KILL "OutMod" + LTRIM$(STR$(m))
   NEXT m
RETURN

' -------------------------------- LeggeCarTel ---------------------------------
'  Routine per la lettura dei carichi nodali e sulle aste per le singole
'  strutture piane componenti l'insieme spaziale
' ------------------------------------------------------------------------------
LeggeCarTel:
   PRINT "lettura dati di carico": GOSUB InCorso
  
   GOSUB DimCarGen
   SELECT CASE TT$(T)
      CASE "G"
         ' telaio generico
         '* le variabili MoAn e nMod non servono; uso V1 e V2 per evitare confusione
         CALL InputCarGen(Incr, V1, Contr, IZ, JZ, QN, QA, V2, F(), P(), nPassi, PassoOut, ER)
      CASE "R"
         ' telaio a maglie trapezie
         GOSUB DimCarRet
         CALL InputCarRet(Contr, Wnkl, IV, IM, IN, IO, PZ, RZ, KZ, NP(), V$(), NB(), NF(), ZA(), ZI(), Q(), QF(), MN(), MNF(), FYN(), FYF(), FX())
   END SELECT
  
   GOSUB Completata
RETURN


' -------------------------------- ConverteCarichi -----------------------------
'   Routine per la conversione dei dati di carico degli schemi piani
'   particolari
' ------------------------------------------------------------------------------
ConverteCarichi:
   IF TT$(T) = "G" THEN RETURN
   PRINT "conversione dati di carico": GOSUB InCorso

   SELECT CASE TT$(T)
      CASE "R"
         CALL ConverteCarRet(RZ, PZ, Wnkl, NB(), NF(), ZA(), ZI(), IV, IM, IN, IO, Q(), QF(), MN(), MNF(), FYN(), FYF(), FX(), KZ, V$(), NI(), P(), F())
   END SELECT
 
   GOSUB Completata
RETURN


' -------------------------------- StampaCarTel --------------------------------
'  Routine per la stampa dei carichi agenti sulle singole strutture piane
'  componenti l'insieme spaziale
' ------------------------------------------------------------------------------
StampaCarTel:
   IF (TT$(T) = "G" AND QN + QA = 0) OR (TT$(T) = "R" AND IV + IM + IN + IO = 0) THEN RETURN
   PRINT "stampa dati di carico"
   IF FO$ = "CON" THEN CLS  ELSE GOSUB InCorso
  
   PRINT #2, : PRINT #2, : PRINT #2,
   PRINT #2, "TELAIO N."; T; "- schema di carico n."; CAR
   SELECT CASE TT$(T)
      CASE "G"
         ' telaio generico
         PRINT #2, : PRINT #2,
         CALL OutputCarGen(SM, CAR, IZ, JZ, QN, QA, F(), P())
      CASE "R"
         ' telaio a maglie trapezie
         CALL OutputCarRet(SM, Wnkl, IV, IM, IN, IO, PZ, RZ, KZ, NP(), V$(), NB(), NF(), ZA(), ZI(), Q(), QF(), MN(), MNF(), FYN(), FYF(), FX())
   END SELECT

   IF FO$ = "CON" THEN
      CALL Aspetta(2): GOSUB NomeProgramma
   ELSE
      GOSUB Completata
   END IF
RETURN


' -------------------------------- TerminiNoti ---------------------------------
'  Routine per il calcolo del vettore termini noti
' ------------------------------------------------------------------------------
TerminiNoti:
   PRINT "preparazione termini noti": GOSUB InCorso
  
   CALL TermNoto(TrRig, IZ, JZ, X(), Y(), ZA(), ZI(), E, TA$(), Estr(), NT(), K(), B(), TrRig(), N(), P(), F(), R())
  
   GOSUB Completata
RETURN


' -------------------------------- TriangR -------------------------------------
'  Routine per l'applicazione al vettore termini noti R delle trasformazioni
'  eseguite sulla matrice di rigidezza
' ------------------------------------------------------------------------------
TriangR:
   PRINT "triangolarizzazione termini noti": GOSUB InCorso
  
   CALL Risolve.K2(KTT(), KTA(), KAA(), SB, 1, R())
   FOR R = 1 TO NL
      Rt(R) = R(R)
   NEXT R
   FOR R = NL + 1 TO NZ
      Ra(R - NL) = R(R)
   NEXT R
  
   GOSUB Completata
RETURN


' -------------------------------- SommaCarTel ---------------------------------
'  Routine che aggiunge alle azioni traslanti di incastro sull'insieme spaziale
'  il contributo fornito dalle azioni traslanti di incastro del telaio piano
'  (conseguenti a dissimmetrie geometriche o di carico)
' ------------------------------------------------------------------------------
SommaCarTel:
   PRINT "somma azioni traslanti del telaio": GOSUB InCorso
  
   ' parametri caratteristici telaio
   Al = Alfa(T) * PiGr / 180
   CsAl = COS(Al)
   SnAl = SIN(Al)
   Dist = X0(T) * SnAl - Y0(T) * CsAl
   FOR K = 1 TO KZ
      Rs(PI(K)) = Rs(PI(K)) + Rt(K) * CsAl
      Rs(PI(K) + PSZ) = Rs(PI(K) + PSZ) + Rt(K) * SnAl
      IF NoRotaz = 0 THEN
         Rs(PI(K) + 2 * PSZ) = Rs(PI(K) + 2 * PSZ) + Rt(K) * Dist
      END IF
   NEXT K
  
   GOSUB Completata
RETURN


' -------------------------------- RisolveSistema ------------------------------
'  Routine per il calcolo delle componenti di movimento degli impalcati
' ------------------------------------------------------------------------------
RisolveSistema:
   PRINT "soluzione sistema": GOSUB InCorso
  
   CALL Risolve(Ks(), Rs(), IPVT())
  
   GOSUB Completata
RETURN


' -------------------------------- StampaRisultSpa -----------------------------
'  Routine per la stampa dei risultati dell'insieme spaziale
' ------------------------------------------------------------------------------
StampaRisultSpa:
   PRINT "stampa risultati insieme spaziale"
   IF FO$ = "CON" THEN CLS  ELSE GOSUB InCorso
   
   PRINT #2, : PRINT #2,
   PRINT #2, "RISULTATI"
   CALL OutputSpostSpa(PSZ, Rs())

   IF FO$ = "CON" THEN
      CALL Aspetta(2): GOSUB NomeProgramma
   ELSE
      GOSUB Completata
   END IF
RETURN


' -------------------------------- Spostamenti ---------------------------------
'  Routine per il calcolo delle componenti di movimento dei nodi
'  del singolo telaio piano
' ------------------------------------------------------------------------------
Spostamenti:
   PRINT "calcolo componenti di movimento": GOSUB InCorso
   
   REDIM SpoAssTel(KZ), SpoRelTel(KZ)
   ' aggiunge azioni traslanti di incastro
   I = 0
   FOR K = 1 TO KZ
      I = I + 1
      F(NI(I), 1) = F(NI(I), 1) - Rt(K)
      DO
         I = I + 1
      LOOP UNTIL NI(I) = 0
   NEXT K
   ' calcolo traslazioni traversi
   Al = Alfa(T) * PiGr / 180
   CsAl = COS(Al)
   SnAl = SIN(Al)
   Dist = X0(T) * SnAl - Y0(T) * CsAl
   FOR K = 1 TO KZ
      Rt(K) = Rs(PI(K)) * CsAl + Rs(PI(K) + PSZ) * SnAl + Rs(PI(K) + 2 * PSZ) * Dist
      SpoAssTel(K) = Rt(K)
      IF K = 1 THEN
         SpoRelTel(K) = SpoAssTel(K)
      ELSE
         SpoRelTel(K) = SpoAssTel(K) - SpoAssTel(K - 1)
      END IF
   NEXT K
   ' calcolo altre componenti di movimento
   CALL InserMat(Rt(), R(), 1, 1)
   CALL InserMat(Ra(), R(), NL + 1, 1)
   CALL Risolve.K2(KTT(), KTA(), KAA(), SB, 2, R())
  
   GOSUB Completata
RETURN

' -------------------------------- CaratSollec ---------------------------------
'  Routine per il calcolo delle azioni sulle aste delle strutture piane
'  componenti l'insieme spaziale
' ------------------------------------------------------------------------------
CaratSollec:
   PRINT "calcolo caratt. sollecitazione": GOSUB InCorso
  
   GOSUB DimRisGen
   REDIM Forza(KZ), SommaF(KZ)
   CALL ProdottoMat(KTT(), Rt(), St())
   I = 0
   FOR K = 1 TO KZ
      I = I + 1
      F(NI(I), 1) = F(NI(I), 1) + St(K)
      Forza(K) = F(NI(I), 1)
      DO
         I = I + 1
         IF NI(I) <> 0 THEN Forza(K) = Forza(K) + F(NI(I), 1)
      LOOP UNTIL NI(I) = 0
   NEXT K
   SommaF(KZ) = Forza(KZ)
   FOR K = KZ - 1 TO 1 STEP -1
      SommaF(K) = SommaF(K + 1) + Forza(K)
   NEXT K
   REDIM Easta(JZ), Gasta(JZ)
   FOR J = 1 TO JZ
      Easta(J) = E
      Gasta(J) = G
   NEXT J
   CALL CarSoll(DefTag, TrRig, JZ, X(), Y(), ZA(), ZAT(), ZI(), Easta(), Gasta(), TA$(), Estr(), NT(), K(), B(), TrRig(), N(), P(), F(), R(), N1(), T1(), M1(), N2(), T2(), M2())
   GOSUB Completata
RETURN

' -------------------------------- ConverteRisult ------------------------------
'  Routine per la conversione delle caratteristiche di sollecitazione degli
'  schemi piani particolari
' ------------------------------------------------------------------------------
ConverteRisult:
   IF TT$(T) = "G" THEN RETURN
   PRINT "conversione caratt. sollecitaz.": GOSUB InCorso
     
   SELECT CASE TT$(T)
      CASE "R"
         GOSUB DimRisRet
         CALL ConverteRisRet(RZ, PZ, Wnkl, DefEst, NB(), NR(), NF(), ZA(), ZI(), D(), MS(), MD(), TS(), TD(), MP(), MT(), TR(), SN(), MFS(), MFD(), TFS(), TFD(), KZ, NI(), N(), R(), F(), M1(), M2(), T1(), T2(), N1())
   END SELECT
  
   GOSUB Completata
RETURN


' -------------------------------- StampaRisultTel -----------------------------
'  Routine per la stampa dei risultati
' ------------------------------------------------------------------------------
StampaRisultTel:
   IF Outp = -1 THEN
      PRINT "stampa inviluppo modale"
   ELSE
      PRINT "stampa risultati"
   END IF
   IF FO$ = "CON" THEN CLS  ELSE GOSUB InCorso
  
   PRINT #2, : PRINT #2, : PRINT #2,
   PRINT #2, "TELAIO N."; T;
   IF Outp = 0 THEN
      PRINT #2,
   ELSEIF Outp = -1 THEN
      PRINT #2, "- inviluppo dei modi"
   ELSE
      IF OutRid = 0 THEN PRINT #2, "- modo n."; Outp
   END IF
   SELECT CASE TT$(T)
      CASE "G"
         CALL CarSollTrascurG(SM, JZ, N1(), T1(), M1(), N2(), T2(), M2(), ValTrasc)
      CASE "R"
         CALL CarSollTrascurR(SM, RZ, PZ, Wnkl, MS(), MD(), TS(), TD(), MP(), MT(), TR(), SN(), MFS(), MFD(), TFS(), TFD(), ValTrasc)
   END SELECT
   IF Outp <= 0 OR OutRid = 0 THEN
      CALL OutputSpoOriz(KZ, SpoAssTel(), SpoRelTel())
      IF ValTrasc THEN
         PRINT #2,
         PRINT #2, "I valori delle caratteristiche della sollecitazione sono trascurabili"
      ELSE
         SELECT CASE TT$(T)
            CASE "G"
               CALL OutputRisGen(SM, Outp, IZ, JZ, KZ, NL, V$(), KV(), NI(), N(), R(), M1(), M2(), T1(), T2(), N1(), N2(), F())
            CASE "R"
               CALL OutputRisRet(SM, Outp, RZ, PZ, Wnkl, NP(), NB(), NR(), NF(), ZA(), ZI(), MS(), MD(), TS(), TD(), MP(), MT(), TR(), SN(), MFS(), MFD(), TFS(), TFD(), IZ, KZ, NL, V$(), N(), R(), F())
         END SELECT
      END IF
      IF StampaRig AND Outp <> -1 THEN
         PRINT #2,
         PRINT #2, "Rigidezza del telaio a ciascun piano"
         PRINT #2,
         PRINT #2, "piano     forza      taglio    spo.ass.    spo.rel.     rigidezza"
         PRINT #2, "           (kN)       (kN)       (mm)        (mm)        (kN/mm)"
         FOR K = KZ TO 1 STEP -1
            PRINT #2, USING " ##    #####.##    #####.##   ####.####   ####.####"; K; Forza(K); SommaF(K); SpoAssTel(K) * 1000; SpoRelTel(K) * 1000;
            IF SpoRelTel(K) <> 0 THEN
               PRINT #2, USING "   ######.####"; SommaF(K) / SpoRelTel(K) / 1000
            ELSE
               PRINT #2, "       -------"
            END IF
         NEXT K
      END IF
   END IF
   IF Outp > 0 THEN
      Buff = 10 + Outp
      CALL WriteRisCom(Buff, KZ, SpoAssTel(), SpoRelTel())
      SELECT CASE TT$(T)
         CASE "G"
            CALL WriteRisGen(Buff, NZ, JZ, R(), M1(), M2(), T1(), T2(), N1(), N2())
         CASE "R"
            CALL WriteRisRet(Buff, KZ, RZ, PZ, Wnkl, D(), MS(), MD(), TS(), TD(), MP(), MT(), TR(), SN(), MFS(), MFD(), TFS(), TFD())
      END SELECT
   END IF

   IF FO$ = "CON" THEN
      CALL Aspetta(2): GOSUB NomeProgramma
   ELSE
      GOSUB Completata
   END IF
RETURN

' -------------------------------- OutputIpotesi -------------------------------
'  Routine per la stampa delle note sulle ipotesi di calcolo
' ------------------------------------------------------------------------------
OutputIpotesi:
   PRINT #2, : PRINT #2,
   PRINT #2, "Il calcolo e' stato eseguito :"
   PRINT #2, "- trascurando l'interazione tra aste di telai distinti che incidono"
   PRINT #2, "  in uno stesso punto nodale"
   IF TelRet = 1 AND DefEst <> 1 THEN
      PRINT #2, "- trascurando la deformazione estensionale dei pilastri"
      PRINT #2, "  di telai a maglie rettangolari o trapezie"
   ELSE
      PRINT #2, "- tenendo conto della deformazione estensionale delle aste"
   END IF
   IF DefTag = 1 THEN
      PRINT #2, "- tenendo conto della deformazione a taglio delle aste"
      IF TelWnkl = 1 THEN
         PRINT #2, "- trascurando la deformazione a taglio delle aste su suolo alla Winkler"
      END IF
   ELSE
      PRINT #2, "- trascurando la deformazione a taglio delle aste"
   END IF
   IF NoRotaz THEN
      PRINT #2, "- considerando impedita la rotazione degli impalcati"
   END IF
   PRINT #2,
   PRINT #2, "Attenzione:"
   PRINT #2, "- l'effettivo sforzo normale in ciascun pilastro dello schema spaziale"
   PRINT #2, "  deve essere valutato  come somma  degli sforzi normali dei telai cui"
   PRINT #2, "  esso appartiene"
   IF TelVCed = 1 OR TelWnkl = 1 THEN
      PRINT #2, "- la correttezza dei risultati ottenuti  potrebbe essere inficiata  da"
      PRINT #2, "  cedimenti differenziali  dovuti alla presenza di  vincoli cedevoli o"
      PRINT #2, "  travi di fondazione"
   END IF
   PRINT #2, : PRINT #2,
RETURN


' -------------------------------- EraseFiles ----------------------------------
'   Routine per la cancellazione dei files temporanei
' ------------------------------------------------------------------------------
EraseFiles:
   KILL FT$ + "MATR.SPA"
   KILL FT$ + "GEOM.SPA"
RETURN


' -------------------------------- ReadCar -------------------------------------
'  Routine per la lettura dei vettori carichi telaio generico
'  dai file temporanei su memoria di massa
' ------------------------------------------------------------------------------
ReadCar:
   IF Izer <> 0 THEN PRINT "lettura carichi": GOSUB InCorso

   GOSUB DimCarGen
   IF Izer <> 0 THEN
      CALL InputMat(F(), 5)
      CALL InputMat(P(), 5)
      CALL InputMat(Ra(), 5)
      CALL InputMat(Rt(), 5)
   ELSE
      ' non occorre azzerare F() e P(), gi nulli perch appena dimensionati
      CALL AzzeraMat(Ra())
      CALL AzzeraMat(Rt())
   END IF

   IF Izer <> 0 THEN GOSUB Completata
RETURN


' -------------------------------- ReadGeomTel ---------------------------------
'  Routine per la lettura dei dati geometrici e della matrice di rigidezza
'  del singolo telaio nei file temporanei su memoria di massa
' ------------------------------------------------------------------------------
ReadGeomTel:
   GOSUB NomeProgramma
   PRINT "    Schema di carico n."; CAR
   PRINT
   PRINT "    telaio n."; T
   PRINT "lettura da file temporanei": GOSUB InCorso

   IF TT$(T) = "R" THEN
      GOSUB ReadGeomRet
   END IF
   GOSUB ReadGeomGen
   GOSUB ReadK

   GOSUB Completata
RETURN

' ................................ ReadGeomRet .................................
'  Routine per la lettura dei dati geometrici per telai a maglie rettangolari
'
ReadGeomRet:
   INPUT #4, RZ, PZ, Wnkl, TrRig
   GOSUB DimGeomRet
   CALL InputMat(NP(), 4)
   CALL InputMat(DL(), 4)
   CALL InputMat(NB(), 4)
   CALL InputMat(NR(), 4)
   IF Wnkl = 1 THEN
      CALL InputMat(NF(), 4)
      CALL InputMat(BF(), 4)
      CALL InputMat(KF(), 4)
   END IF
RETURN

' ................................ ReadGeomGen .................................
'  Routine per la lettura dei dati geometrici per telai generici
'
ReadGeomGen:
   INPUT #4, IZ, JZ, KZ, VCed, Wnkl, TrRig
   GOSUB DimGeomGen
   CALL InputMat(X(), 4)
   CALL InputMat(Y(), 4)
   FOR I = 1 TO IZ
      INPUT #4, V$(I)
   NEXT I
   IF VCed = 1 THEN CALL InputMat(KV(), 4)
   CALL InputMat(N(), 4)
   FOR J = 1 TO JZ
      INPUT #4, Estr(J, 1), Estr(J, 2), TA$(J), NT(J)
      IF Wnkl = 1 THEN INPUT #4, B(J), K(J)
      IF TrRig THEN INPUT #4, TrRig(J, 1), TrRig(J, 2)
   NEXT J
   FOR J = 1 TO IZ + KZ
      INPUT #4, NI(J)
   NEXT J
   CALL InputMat(PI(), 4)
RETURN

' ................................ ReadK .......................................
'  Routine per la lettura della matrice di rigidezza del singolo telaio
'
ReadK:
   INPUT #3, NZ, NL, SB
   GOSUB DimMatRig
   CALL InputMat(KTT(), 3)
   CALL InputMat(KTA(), 3)
   CALL InputMat(KAA(), 3)
RETURN

' -------------------------------- WriteCar ------------------------------------
'  Routine per la memorizzazione dei vettori carichi telaio generico
'  nei file temporanei su memoria di massa
' ------------------------------------------------------------------------------
WriteCar:
   PRINT "memorizzazione carichi": GOSUB InCorso

   CALL WriteMat(F(), 5)
   CALL WriteMat(P(), 5)
   CALL WriteMat(Ra(), 5)
   CALL WriteMat(Rt(), 5)

   GOSUB Completata
RETURN


' -------------------------------- WriteGeomTel --------------------------------
'  Routine per la memorizzazione dei dati geometrici e della matrice di
'  rigidezza del singolo telaio nei file temporanei su memoria di massa
' ------------------------------------------------------------------------------
WriteGeomTel:
   PRINT "memorizzazione in file temporanei": GOSUB InCorso

   IF TT$(T) = "R" THEN
      GOSUB WriteGeomRet
   END IF
   GOSUB WriteGeomGen
   GOSUB WriteK

   GOSUB Completata
RETURN

' ................................ WriteGeomRet ................................
'  Routine per la memorizzazione dei dati geometrici per telai a maglie
'  rettangolari
'
WriteGeomRet:
   WRITE #4, RZ, PZ, Wnkl, TrRig
   CALL WriteMat(NP(), 4)
   CALL WriteMat(DL(), 4)
   CALL WriteMat(NB(), 4)
   CALL WriteMat(NR(), 4)
   IF Wnkl = 1 THEN
      CALL WriteMat(NF(), 4)
      CALL WriteMat(BF(), 4)
      CALL WriteMat(KF(), 4)
   END IF
RETURN

' ................................ WriteGeomGen ................................
'  Routine per la memorizzazione dei dati geometrici per telai generici
'
WriteGeomGen:
   WRITE #4, IZ, JZ, KZ, VCed, Wnkl, TrRig
   CALL WriteMat(X(), 4)
   CALL WriteMat(Y(), 4)
   FOR I = 1 TO IZ
      WRITE #4, V$(I)
   NEXT I
   IF VCed = 1 THEN CALL WriteMat(KV(), 4)
   CALL WriteMat(N(), 4)
   FOR J = 1 TO JZ
      WRITE #4, Estr(J, 1), Estr(J, 2), TA$(J), NT(J)
      IF Wnkl = 1 THEN WRITE #4, B(J), K(J)
      IF TrRig THEN WRITE #4, TrRig(J, 1), TrRig(J, 2)
   NEXT J
   FOR J = 1 TO IZ + KZ
      WRITE #4, NI(J)
   NEXT J
   CALL WriteMat(PI(), 4)
RETURN

' ................................ WriteK ......................................
'  Routine per la memorizzazione della matrice di rigidezza del singolo telaio
'
WriteK:
   WRITE #3, NZ, NL, SB
   CALL WriteMat(KTT(), 3)
   CALL WriteMat(KTA(), 3)
   CALL WriteMat(KAA(), 3)
RETURN

' ================================ CarSollTrascurG =============================
'  Procedura che valuta se le caratteristiche della sollecitazione sono
'  tutte minori di un minimo e quindi trascurabili - per telaio generico
' ------------------------------------------------------------------------------
SUB CarSollTrascurG (SM, JZ, N1(), T1(), M1(), N2(), T2(), M2(), ValTrasc)

   IF SM = 0 THEN ValMin = .005 ELSE ValMin = .05
   ValTrasc = -1
   FOR J = 1 TO JZ
      IF ABS(N1(J)) > ValMin THEN ValTrasc = 0: EXIT FOR
      IF ABS(T1(J)) > ValMin THEN ValTrasc = 0: EXIT FOR
      IF ABS(M1(J)) > ValMin THEN ValTrasc = 0: EXIT FOR
      IF ABS(N2(J)) > ValMin THEN ValTrasc = 0: EXIT FOR
      IF ABS(T2(J)) > ValMin THEN ValTrasc = 0: EXIT FOR
      IF ABS(M2(J)) > ValMin THEN ValTrasc = 0: EXIT FOR
   NEXT J

END SUB

' ================================ CarSollTrascurR =============================
'  Procedura che valuta se le caratteristiche della sollecitazione sono
'  tutte minori di un minimo e quindi trascurabili - per telaio a maglie rett.
' ------------------------------------------------------------------------------
SUB CarSollTrascurR (SM, RZ, PZ, Wnkl, MS(), MD(), TS(), TD(), MP(), MT(), TR(), SN(), MFS(), MFD(), TFS(), TFD(), ValTrasc)
  
   IF SM = 0 THEN ValMin = .005 ELSE ValMin = .05
   ValTrasc = -1
   FOR R = 1 TO RZ - 1
      FOR P = PZ TO 1 STEP -1
         IF ABS(MS(R, P)) > ValMin THEN ValTrasc = 0: EXIT SUB
         IF ABS(MD(R, P)) > ValMin THEN ValTrasc = 0: EXIT SUB
         IF ABS(TS(R, P)) > ValMin THEN ValTrasc = 0: EXIT SUB
         IF ABS(TD(R, P)) > ValMin THEN ValTrasc = 0: EXIT SUB
      NEXT P
   NEXT R
   FOR R = 1 TO RZ
      FOR P = PZ TO 1 STEP -1
         IF ABS(MT(R, P)) > ValMin THEN ValTrasc = 0: EXIT SUB
         IF ABS(MP(R, P)) > ValMin THEN ValTrasc = 0: EXIT SUB
         IF ABS(TR(R, P)) > ValMin THEN ValTrasc = 0: EXIT SUB
         IF ABS(SN(R, P)) > ValMin THEN ValTrasc = 0: EXIT SUB
      NEXT P
   NEXT R
   IF Wnkl = 1 THEN
      FOR F = 0 TO RZ
         IF ABS(MFS(F)) > ValMin THEN ValTrasc = 0: EXIT SUB
         IF ABS(MFD(F)) > ValMin THEN ValTrasc = 0: EXIT SUB
         IF ABS(TFS(F)) > ValMin THEN ValTrasc = 0: EXIT SUB
         IF ABS(TFD(F)) > ValMin THEN ValTrasc = 0: EXIT SUB
      NEXT F
   END IF

END SUB

' ================================ CoeffInvMod =================================
'  Calcola i coefficienti per inviluppo modale
' ------------------------------------------------------------------------------
SUB CoeffInvMod (nMod, Omega(), zita, Rgrec())

   FOR I = 1 TO nMod
      FOR N = 1 TO nMod
         Bin = Omega(I) / Omega(N)
         RgrecNom = 8 * zita ^ 2 * (1 + Bin) * Bin ^ 1.5
         RgrecDen = (1 - Bin ^ 2) ^ 2 + 4 * zita ^ 2 * Bin * (1 + Bin) ^ 2
         Rgrec(I, N) = RgrecNom / RgrecDen
      NEXT N
   NEXT I

END SUB

' ================================ InviluppoMod ================================
'  Effettua inviluppo modale di un insieme di valori (che legge da file)
' ------------------------------------------------------------------------------
SUB InviluppoMod (SovrCQC, nMod, Rgrec(), R0)

   REDIM R(nMod)
   FOR I = 1 TO nMod
      INPUT #10 + I, R(I)
   NEXT I
   R0 = 0
   IF SovrCQC = 0 THEN
      FOR I = 1 TO nMod
         R0 = R0 + R(I) ^ 2
      NEXT I
   ELSE
      FOR I = 1 TO nMod
         FOR N = 1 TO nMod
            R0 = R0 + Rgrec(I, N) * R(I) * R(N)
         NEXT N
      NEXT I
   END IF
   R0 = SQR(R0)

END SUB

' ================================ InvModCom ===================================
'  Effettua inviluppo modale per dati comuni a telaio generico e rettangolare
' ------------------------------------------------------------------------------
SUB InvModCom (SovrCQC, nMod, Rgrec(), KZ, SpoAssTel(), SpoRelTel())

   ' spostamenti orizzontali
   FOR K = 1 TO KZ
      CALL InviluppoMod(SovrCQC, nMod, Rgrec(), SpoAssTel(K))
      CALL InviluppoMod(SovrCQC, nMod, Rgrec(), SpoRelTel(K))
   NEXT K

END SUB

' ================================ InvModGen ===================================
'  Effettua inviluppo modale per il telaio generico
' ------------------------------------------------------------------------------
SUB InvModGen (SovrCQC, nMod, Rgrec(), NZ, JZ, R(), M1(), M2(), T1(), T2(), N1(), N2())

   ' spostamenti (incognite trovate)
   FOR N = 1 TO NZ
      CALL InviluppoMod(SovrCQC, nMod, Rgrec(), R(N))
   NEXT N
   ' caratteristiche di sollecitazione
   FOR J = 1 TO JZ
      CALL InviluppoMod(SovrCQC, nMod, Rgrec(), M1(J))
      CALL InviluppoMod(SovrCQC, nMod, Rgrec(), M2(J))
      CALL InviluppoMod(SovrCQC, nMod, Rgrec(), T1(J))
      CALL InviluppoMod(SovrCQC, nMod, Rgrec(), T2(J))
      CALL InviluppoMod(SovrCQC, nMod, Rgrec(), N1(J))
      CALL InviluppoMod(SovrCQC, nMod, Rgrec(), N2(J))
   NEXT J

END SUB

' ================================ InvModRet ===================================
'  Effettua inviluppo modale per il telaio a maglie rettangolari
' ------------------------------------------------------------------------------
SUB InvModRet (SovrCQC, nMod, Rgrec(), KZ, RZ, PZ, Wnkl, D(), MS(), MD(), TS(), TD(), MP(), MT(), TR(), SN(), MFS(), MFD(), TFS(), TFD())
 
   ' spostamenti orizzontali dei traversi
   FOR K = 1 TO KZ
      CALL InviluppoMod(SovrCQC, nMod, Rgrec(), D(K))
   NEXT K
   ' travi
   FOR R = 1 TO RZ - 1
      FOR P = 1 TO PZ
         CALL InviluppoMod(SovrCQC, nMod, Rgrec(), MS(R, P))
         CALL InviluppoMod(SovrCQC, nMod, Rgrec(), MD(R, P))
         CALL InviluppoMod(SovrCQC, nMod, Rgrec(), TS(R, P))
         CALL InviluppoMod(SovrCQC, nMod, Rgrec(), TD(R, P))
      NEXT P
   NEXT R
   ' pilastri
   FOR R = 1 TO RZ
      FOR P = 1 TO PZ
         CALL InviluppoMod(SovrCQC, nMod, Rgrec(), MT(R, P))
         CALL InviluppoMod(SovrCQC, nMod, Rgrec(), MP(R, P))
         CALL InviluppoMod(SovrCQC, nMod, Rgrec(), TR(R, P))
         CALL InviluppoMod(SovrCQC, nMod, Rgrec(), SN(R, P))
      NEXT P
   NEXT R
   ' fondazione
   IF Wnkl = 1 THEN
      FOR F = 0 TO RZ
         CALL InviluppoMod(SovrCQC, nMod, Rgrec(), MFS(F))
         CALL InviluppoMod(SovrCQC, nMod, Rgrec(), MFD(F))
         CALL InviluppoMod(SovrCQC, nMod, Rgrec(), TFS(F))
         CALL InviluppoMod(SovrCQC, nMod, Rgrec(), TFD(F))
      NEXT F
   END IF

END SUB

' ================================ OutputForzeSpa ==============================
'  Procedura per la stampa delle forze sugli impalcati
' ------------------------------------------------------------------------------
SUB OutputForzeSpa (SM, PSZ, Xm(), Ym(), Fsx(), Fsy(), Msz())
  
   PRINT #2, : PRINT #2,
   PRINT #2, "CARICHI AGENTI SULL'INSIEME SPAZIALE"
   PRINT #2,
   PRINT #2, "Impalcato   Xm (m)     Ym (m)     ";
   IF SM = 0 THEN
      PRINT #2, "Fx (kN)    Fy (kN)    Mz (kNm)"
   ELSE
      PRINT #2, "Fx (t)     Fy (t)     Mz (tm)"
   END IF
   PRINT #2,
   FOR PS = PSZ TO 1 STEP -1
      PRINT #2, USING "   ###     ###.##     ###.##"; PS; Xm(PS); Ym(PS);
      IF SM = 0 THEN
         PRINT #2, USING "    #####.##   #####.##   ######.##"; Fsx(PS); Fsy(PS); Msz(PS)
      ELSE
         PRINT #2, USING "    ####.###   ####.###   #####.###"; Fsx(PS); Fsy(PS); Msz(PS)
      END IF
   NEXT PS

END SUB

' ================================ OutputMasseSpa ==============================
'  Procedura per la stampa delle masse sugli impalcati
' ------------------------------------------------------------------------------
SUB OutputMasseSpa (PSZ, Xm(), Ym(), Mas(), Mro())
  
   PRINT #2, : PRINT #2,
   PRINT #2, "MASSE DISPOSTE SUGLI IMPLACATI"
   PRINT #2,
   PRINT #2, "Impalcato   Xm (m)     Ym (m)      massa (t)     rho (m)"
   PRINT #2,
   FOR PS = PSZ TO 1 STEP -1
      PRINT #2, USING "   ###     ###.##     ###.##"; PS; Xm(PS); Ym(PS);
      PRINT #2, USING "       ####.###      ###.##"; Mas(PS); Mro(PS)
   NEXT PS

END SUB

' ================================ OutputSpostSpa ==============================
'  Procedura per la stampa delle forze sugli impalcati
' ------------------------------------------------------------------------------
SUB OutputSpostSpa (PSZ, Rs())

   PRINT #2, : PRINT #2,
   PRINT #2, "COMPONENTI DI MOVIMENTO DEGLI IMPALCATI"
   PRINT #2, : PRINT #2,
   PRINT #2, "Impalcato     Vx (mm)      Vy (mm)    rotaz.x1000"
   PRINT #2,
   FOR PS = PSZ TO 1 STEP -1
      PRINT #2, USING "   ##  "; PS;
      PRINT #2, USING "      ###.###"; Rs(PS) * 1000; Rs(PS + PSZ) * 1000; Rs(PS + 2 * PSZ) * 1000
   NEXT PS

END SUB

' ================================ WriteRisCom =================================
'  Salva in un file i risultati comuni a telaio generico e a maglie rettangolari
' ------------------------------------------------------------------------------
SUB WriteRisCom (Buff, KZ, SpoAssTel(), SpoRelTel())

   ' spostamenti orizzontali
   FOR K = 1 TO KZ
      WRITE #Buff, SpoAssTel(K), SpoRelTel(K)
   NEXT K

END SUB

' ================================ WriteRisGen =================================
'  Salva in un file i risultati del telaio generico
' ------------------------------------------------------------------------------
SUB WriteRisGen (Buff, NZ, JZ, R(), M1(), M2(), T1(), T2(), N1(), N2())

   ' spostamenti (incognite trovate)
   FOR N = 1 TO NZ
      WRITE #Buff, R(N)
   NEXT N
   ' caratteristiche di sollecitazione
   FOR J = 1 TO JZ
      WRITE #Buff, M1(J), M2(J), T1(J), T2(J), N1(J), N2(J)
   NEXT J

END SUB

' ================================ WriteRisRet =================================
'  Salva in un file i risultati del telaio a maglie rettangolari
' ------------------------------------------------------------------------------
SUB WriteRisRet (Buff, KZ, RZ, PZ, Wnkl, D(), MS(), MD(), TS(), TD(), MP(), MT(), TR(), SN(), MFS(), MFD(), TFS(), TFD())
  
   ' spostamenti orizzontali dei traversi
   FOR K = 1 TO KZ
      WRITE #Buff, D(K)
   NEXT K
   ' travi
   FOR R = 1 TO RZ - 1
      FOR P = 1 TO PZ
         WRITE #Buff, MS(R, P), MD(R, P), TS(R, P), TD(R, P)
      NEXT P
   NEXT R
   ' pilastri
   FOR R = 1 TO RZ
      FOR P = 1 TO PZ
         WRITE #Buff, MT(R, P), MP(R, P), TR(R, P), SN(R, P)
      NEXT P
   NEXT R
   ' fondazione
   IF Wnkl = 1 THEN
      FOR F = 0 TO RZ
         WRITE #Buff, MFS(F), MFD(F), TFS(F), TFD(F)
      NEXT F
   END IF

END SUB

